/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/sys/boot.h		$Revision: $
 * 
 *  Purpose:
 *	Parameters exchanged between kernel loader and the kernel.
 *
 *	The loader invokes the kernel as:
 *
 *	entry_point(argc, argv, env);
 *		int		argc;
 *		char	       *argv[];
 *		char	       *env[];
 *
 *	argc   -- Count of valid entries in argv
 *	argv   -- pointers to strings handed down as arguments.
 *	env    -- generally useful pointers and values.
 *
 *	Low level kernel startup code saves these arguments in
 *		int 	boot_argc;
 *		char  **boot_argv;
 *		char  **boot_env;
 *	It then reformats these arguments for compatibility
 * 	with expectations of other kernel code (Historical reasons).
 *      After reformatting the arguments take the form...
 *
 *	entry_point(pdc_entry, boot_flags, init_string, io_config);
 *		int	      (*pdc_entry)();
 *		int		boot_flags;
 *		char	       *init_string;
 *		struct bt_info *io_config;
 *
 *	pdc_entry   -- the page 0 address to do PDC calls
 *	boot_flags  -- just that...flags
 *	init_string -- any argv[1] for proc 1 (init)
 *	io_config   -- the I/O configuration that hpuxboot found
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  no
 *
 *  Shipped:  yes
 *	/usr/include/pa/boot.h
 *	/usr/conf/pa/boot.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif

#ifndef _MACHINE_SYS_BOOT_INCLUDED
#define _MACHINE_SYS_BOOT_INCLUDED

/* Defines for boot_flags						*/

#if	! defined(RDB_BOOT)
#define	RDB_BOOT	0x01	/* System is booted under RDB		*/
#define RDB_BOOT_BIT	  31	/*	for assembly			*/
#define RDB_STOP	0x02	/* Stop at boot time			*/
#define RDB_STOP_BIT	  30	/*	for assembly			*/
#define RDB_NORS232   0x4000	/* Not using RS232 port for RDB         */
#define RDB_NORS232_BIT   17	/*      for assembly                    */
#endif	/* RDB_BOOT */

#  define KGDB_STOP   0x02	/* Boot w/ kgdb enable, stop and wait	*/
				/* for the debugger client to attach	*/
				/* NB: overlap w/ RDB ok since RDB_STOP */
				/* is only used if RDB_BOOT is specified*/
				/* also.				*/
#  define KGDB_BOOT   0x8000	/* Just enable KGDB, but don't wait for */
				/* attach				*/
#  define KGDB_CONSOLE_BIT 0x20000 /* KWDB console                      */
#  define KGDB_BOOT_BIT   16   /* KGDB_BOOT bit # for BB instruction    */

#define	BF_IO_VALID	0x08	/* io_config argument is valid		*/
#define	BF_IO_VALID_BIT   28	/* 	for assembly code		*/
#define	BF_OVERWRITE	0x04	/* Overwrite I/O configuration on boot	*/
#define	BF_OVERWRITE_BIT  29	/*	for assembly code		*/

#define BF_MIRROR_S	0x10	/* Flags for kernel disk mirroring      */
#define BF_MIRROR_P	0x20
#define BF_MIRROR	(BF_MIRROR_P | BF_MIRROR_S)
#define BF_MIRROR_X	0x40

#define BF_FDLOCK	0x80
#define BF_FDLOCK_BIT     24

#define	BF_NO_DUMP	0x100	/* Boot without dump			*/
#define	BF_NO_CONSOLE	0x200	/* Boot without console			*/
#define	BF_NO_SWAP	0x400	/* Boot without swap			*/

#define BF_LV_M		0x1000	/* Maintenance mode boot (for LVM)	*/
#define BF_LV_Q		0x2000	/* Quorum override (for LVM)		*/

#define KWDB_SPP_UDP	0x10000 /* KWDB boot SPP using UDP/IP Protocol  */

#ifndef	LOCORE

#ifdef _KERNEL

extern int boot_flags;
extern int vemon_present;

#if defined(__LP64__)
#define is_virtual_os() (vemon_present)
#else
#define is_virtual_os() (0)
#endif	/* __LP64__ */

#endif  /* _KERNEL */


#define BT_VERSION	1	   /* version of boot_info structure	*/

/* Indices into bt_devs[] */
#define	BT_CONSDEV	0	   /* console driver			*/
#define	BT_KBDDEV	1	   /* keyboard driver (nuke this?)	*/
#define	BT_BOOTDEV	2	   /* boot device driver		*/
#define	BT_ROOTDEV	BT_BOOTDEV /* root disk driver			*/
#define	BT_SWAPDEV	3	   /* swap device driver		*/
#define	BT_DUMPDEV	4	   /* dump device driver		*/
#define	BT_EXTRAS	5	   /* start of extra devices		*/
#define	BT_DESTDEV	BT_EXTRAS  /* destination device (install only)	*/
#define	MAXDEVICES	10

/* Flags for bt_devs[].bt_flags */
#define	BTF_VALID	0x01	   /* entry contains valid io device	*/
#define	BTF_OVERRIDE	0x02	   /* Override the device 		*/
#define	BTF_NO_DEVICE	0x04	   /* disable the device 		*/
#define	BTF_DEFAULT	0x08	   /* Force the device to default	*/

/* I/O Configuration structure determined by hpuxboot			*/
#ifndef _IO_NAME_TYPE
#define _IO_NAME_TYPE
#define MAX_ID 16
typedef char io_name_type[MAX_ID];
#endif

#define MAX_IO_PATH_ELEMENTS 15
typedef struct {
	int	addr[MAX_IO_PATH_ELEMENTS];
	int	num_elements;
} hdw_path_type;

typedef struct {
	io_name_type	name[MAX_IO_PATH_ELEMENTS];
	int		num_elements;
} module_path_type;

struct bt_info {
    unsigned short	 bt_size;	/* Size of structure?		*/
    unsigned short	 bt_version;	/* Version number		*/
    struct bt_dev {
	unsigned int	 bt_flags;	/* Flags (default, override)	*/
    	dev_t  		 bt_dev;	/* dev_t			*/
	unsigned char	 bt_bmajor;	/* Block major number		*/
	unsigned char	 bt_cmajor;	/* Char major number		*/
	hdw_path_type	 bt_hdw;	/* Hardware path		*/
	module_path_type bt_mod;	/* Module path			*/
	unsigned int	 reserved;	/* Unused			*/
    } bt_devs[MAXDEVICES];
    unsigned int 	bt_mem;		/* pages of memory to config    */
    unsigned int 	bt_ncpu;	/* number of processors to run  */
    unsigned int reserved[211];		/* future (211 = 4k total)	*/
};

/* Loadable software table entry */
struct bt_lsm {
	int lsm_flags;
	char lsm_fname[128];
};

/* Indices into boot env[] */
#define BTENV_COUNT		0
#define BTENV_SYSCALL		1
#define BTENV_LDRSTART		2
#define BTENV_LDRSIZE		3
#define BTENV_BOOTFLAGS		4
#define BTENV_INITSTRING	5
#define BTENV_BTINFO		6
#define BTENV_LSMTABLE		7
#define BTENV_LANPACKET		8
#define BTENV_END		9

/* This GOOD_BOOT_INFO define is an unfortunate, necessary hack.  There
 * is no good way to tell the difference between newer versions of the boot
 * loader that support the bt_mem and bt_ncpu fields, and earlier versions
 * that do not.  The version identifier doesn't help, because if the
 * value doesn't match the one the kernel is expecting, the kernel
 * issues a really nasty message.  What we're doing here is using
 * the reported size of the structure, and the fact that the size
 * changed at the time that the new fields were added, to flag that
 * a newer version of the loader passed along the information we need.
 */

#define GOOD_BOOT_INFO (4001)

#endif	/* !LOCORE */
#endif  /* _MACHINE_SYS_BOOT_INCLUDED */
