/* @(#) cpu.h $Date: 2001/02/02 16:37:32 $Revision: r11.11/1 PATCH_11.11 (PHKL_23315)
*/

/*
 * BEGIN_DESC
 * 
 *  File: 
 *      pa/sys/cpu.h
 * 
 *  Purpose:
 *      <<please update with a synopis of the functionality provided by this file>>
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/cpu.h
 *	/usr/conf/pa/cpu.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _MACHINE_CPU_INCLUDED /* allow multiple inclusions */
#define _MACHINE_CPU_INCLUDED

#ifndef _SYS_STDSYMS_INCLUDED
#    include <sys/stdsyms.h>
#endif  /*  _SYS_STDSYMS_INCLUDED  */

/*
 * IO buffer alignment.  This function takes a pointer to a buffer
 * and returns a pointer which is CPU_IOLINE aligned.
 */
#define CPU_IOLINE      64
#define ALIGN_IOBUF(P) ((char *)(((uintptr_t)(P) + CPU_IOLINE-1) \
	& ~((uintptr_t)(CPU_IOLINE-1))))

/*
 * Cache alignment---this is used for functions that want to align themselves
 * on cache line boundaries to improve performance by minimizing cache misses.
 */
#define CPU_CACHE_LINE  32

/*
 * I/O space starting address.
 */
#ifdef __LP64__
#define IOSPACE_BEG_ADDR  \
	((ulong_t) ((vm_wide) ? (0xF000000000000000L) : (0xF0000000)))
#else
#define IOSPACE_BEG_ADDR  \
	((ulong_t) ((vm_wide) ? (0xF0000000L) : (0xF0000000)))
#endif
#define IOSPACE		IOSPACE_BEG_ADDR

/*
 * Physical address mappings of I/O space for PA 2.0 in wide (64-bit) mode.
 */

#ifdef __LP64__
#define IOSPACE_MASK    ((unsigned long)0xF000000000000000)
#define IO_BROADCAST    0xFFFFFFFFFFFC0000
#define PDC_LOW         0xF000000000000000
#define PDC_HIGH        0xF100000000000000
#define FPA             0xFFFFFFFFFFF80000
#define FLEX_MASK       0xFFFFFFFFFFFC0000
#define FLEX_DATA       0xFFFFFFFFFFF80001
#define BC_RANGE_ALIGN  0xFFFFFFFFFFFF0000
/*
 * Physical address mappings of I/O space for PA 1.x in narrow (32-bit) mode.
 */
#else  /* !__LP64__ */
#define IOSPACE_MASK    ((unsigned)0xF0000000)
#define IO_BROADCAST    0xFFFC0000
#define PDC_LOW         0xEF000000
#define PDC_HIGH        0xF1000000
#define FPA             0xFFF80000
#define FLEX_MASK       0xFFFC0000
#define FLEX_DATA       0xFFF80001
#define BC_RANGE_ALIGN  0xFFFF0000
#endif /* !__LP64__ */

#ifdef __LP64__
/* extru bit pos for 40-bit PHYS ADDR */
#define	BPOS_40B_PHYSADDR	((sizeof(long *) << 3) - 40 + 4 - 1)
#define	NXT_BPOS		(BPOS_40B_PHYSADDR + 4)
#endif /* !__LP64__ */

#define DMA_ENABLE      0x00000001
#define SPA_ENABLE      0x00000020

#define LBROAD_eir      ((volatile struct processorhpa *)IO_BROADCAST)->io_eir
#define LBROAD_flex     ((volatile struct io_module *)IO_BROADCAST)->io_flex
#define LBROAD_command  ((volatile struct io_module *)IO_BROADCAST)->io_command
#define NUM_MODS_PER_BUS        64

/*
 *  HP Precision Architecture memory addresses of variables for communications
 *  between HP Precision Architecture and the 9920. The communications area is
 *  one page long just after the area used by PDC.
 */
#define FIRST_PAGE      0x8000  /* First page of memory past the
                                   area reserved by PDC. */

/*
 * for I/O modules, (opt & IODC_INTERRUPTS) is true if module interrupts
 */
#  define IODC_INTERRUPTS         0x80
#  define EXTENDED_ERR_LOGGING    0x08

/* Module type numbers */

#  define MOD_TYPE_NPROC	0
#  define MOD_TYPE_MEMORY	1
#  define MOD_TYPE_B_DMA_IO	2
#  define MOD_TYPE_B_DIRECT_IO	3
#  define MOD_TYPE_A_DMA_IO	4
#  define MOD_TYPE_A_DIRECT_IO	5
#  define MOD_TYPE_OTHER	6
#  define MOD_TYPE_BUSCONV	7
#  define MOD_TYPE_CIO_CHANNEL	8
#  define MOD_TYPE_CONSOLE	9
#  define MOD_TYPE_FOREIGN_IO  10
#  define MOD_TYPE_BUS_ADAPTER 11
#  define MOD_TYPE_IOA         12
#  define MOD_TYPE_BUS_BRIDGE  13
#  define MOD_TYPE_FABRIC      14
#  define MOD_TYPE_CELL	       15

#  define IS_BUSCONV(x) (x == MOD_TYPE_IOA || x == MOD_TYPE_BUSCONV)

/* Processor HVERSION Model Numbers */
/* NOTE: To avoid name copyright issues, don't use machine nicknames */

#  define CPU_M720	0x200
#  define CPU_M7LC	0x604	/*  */
#  define CPU_M715_D	0x60A
#  define CPU_M715_E	0x60B
#  define CPU_M715_F	0x60C
#  define CPU_M715_G	0x619
#  define CPU_M725_C	0x60D
#  define CPU_M725_D	0x60E
#  define CPU_M743i_A	0x603
#  define CPU_M743i_B	0x604
#  define CPU_M743v_A	0x617
#  define CPU_M743v_B	0x618
#  define CPU_C360	0x5C6   /* for C360-specific PDC_MODEL get CPU_ID bug */

#  define CPU_M808	0x103   /* Restricted 815. Use SW_CAP */
#  define CPU_M810	0x100
#  define CPU_M815	0x103
#  define CPU_M822	0x101
#  define CPU_M825	0x008
#  define CPU_M832	0x102
#  define CPU_M834	0x00A	/* 835 2 user WS */
#  define CPU_M835	0x00A
#  define CPU_M836	0x00A	/* 835 2 user Server */
#  define CPU_M840	0x004
#  define CPU_M842	0x104
#  define CPU_M843	0x104	/* 842 Server */
#  define CPU_M844	0x00B	/* 845 2 user WS */
#  define CPU_M845	0x00B
#  define CPU_M846	0x00B	/* 845 2 user Server */
#  define CPU_M850	0x080
#  define CPU_M850_OPDC	0x00C	/* Old number for 850 */
#  define CPU_M852	0x105
#  define CPU_M853	0x105	/* 852 Server */
#  define CPU_M855	0x081
#  define CPU_M860	0x082	/* PN10C */
#  define CPU_M870	0x083
#  define CPU_T520	0x183   /* for T520-specific pepsi bug */
#  define CPU_JADE      0x184   /* 893 Server */
#  define CPU_MV2200	0x597	/* SPP/V class 2200 or 2250 SN */
#  define CPU_MV2200_MN	0x5C0	/* SPP/V class 2200 or 2250 MN */
#  define CPU_MV2500	0x5B4	/* SPP/V class 2500 SN */
#  define CPU_MV2500_MN	0x5C1	/* SPP/V class 2500 MN */
#  define CPU_N4000_36	0x5BE	/* Prelude PCX-W 360MHz */
#  define CPU_N4000_44	0x5CC	/* Prelude PCX-W 440MHz */
#  define CPU_HALFDOME	0x5C7	/* Halfdome PCX-W */

/* SW_CAP fields */

#  define UNRESTRICTED 0      /* unrestricted */
#  define SERVER       1      /* 2 user server */     
#  define WS           2      /* 2 user workstation */
#  define RESTRICTED   1      /* general limited I/O slots */

/* Central Agent SVERSION Model Numbers */
#define SVER_MODEL_DNA          0xaa

/* Processor SVERSION Fields */

#  define GET_PROCESSOR_LEVEL(sversion) (((unsigned int)sversion) & 0x3)
#  define LEVEL_0_0	0
#  define LEVEL_1_0	1
#  define LEVEL_1_5	2
#  define LEVEL_2_0	3

/* Memory SVERSION Model Numbers */

#  define MEM_ARCH                0x00008
#  define MEM_IMPL_DEP            0x00009

#  define MEM_CONTROL_INIT        0x0000
#  define MEM_CONTROL_NORMAL      0x0100
#  define MEM_HERR_SPA            0x0400
#  define MEM_AERR_CORR           0x6000
#  define MEM_HERR_UNCORR         0x6400
#  define MEM_AERR_MAP            0x6800   /* treat the same as MEM_AERR_CORR */

/* Memory HVERSION Model Numbers */
#define MEM_850                 0x004    /* 850-model memory controller */
#define MEM_COBRA		0x013
#define MEM_CORAL		0x015
#define MEM_OUTFIELD		0x019
#define MEM_CORAL2    		0x01A
#define MEM_MIOC		0x026
#define MEM_CHEETAH		0x037
#define MEM_V2200		0x05F		/* SPP/V class 2200 or 2000 */
#define MEM_V2500		0x085		/* SPP/V class 2500 */

/* Bus Converter HVERSION Model Numbers */

#  define CHEETAH_BC              0x004
#  define BC1_SMODEL		  0x0000C

#  define BC_MODE_MASK            0x180
#  define BC_OFF                  0x000
#  define BC_INCLUDE              0x080
#  define BC_EXCLUDE              0x100
#  define BC_PEEK                 0x180
#  define BC_POWER_LOST_MASK      0x2
#  define BC_POWER_FAIL_MASK      0x5
#  define BC_SEVERE_ERROR_MASK    0x180
#  define BC_PORT_MASK            0x8
#  define BC_UPPER_PORT           0x0
#  define BC_LOWER_PORT           0x8

/* Fields within IO_STATUS register */

#  define SE           0x0200
#  define HE           0x0100
#  define FE           0x0080
#  define RY           0x0040
#  define DG           0x0010
#  define ESTAT_MASK   0xFC00

/* I/O Module States */

#  define ST_ACTIVE(s) (!(s & (RY|DG)))
#  define ST_READY(s)  (s & RY		    && !(s & (DG|FE|HE|SE|ESTAT_MASK)))
#  define ST_AERR(s)   (s & (RY|ESTAT_MASK) && !(s & (DG|FE|HE|SE)))
#  define ST_SERR(s)   (s & (RY|SE)	    && !(s & (DG|FE|HE)))
#  define ST_HERR(s)   (s & (RY|HE)	    && !(s & (DG|FE)))
#  define ST_FERR(s)   (s & (RY|FE)	    && !(s & DG))
#  define ST_DIAG(s)   (s & DG)

/* Commands that are written to IO_COMMAND register */
#  define CMD_CLEAR       3
#  define CMD_RESET       5
#  define CMD_PORT        0x020
#  define CMD_RESET_ST    0x105

/* And just where IS the IO_COMMAND register? */
#define COMMAND 48

#ifdef RDB
#  define MEMRESERVED     NBPG    /* Reserved first page for RDB */
#else
#  define MEMRESERVED     0
#endif /* RDB */

#define GW_PAGE_VADDR	0xC0000000L /* Virtual addr of kernel gateways,
                                     * used by libc for system calls.
				     */
/* 
 * Temp hack so as not to brake r_ic6 libc. To be removed before r_ic7
 */

#ifdef __LP64__
#define GW_PAGE_VADDR_INT      0xC0000000C0000000L
#else
#define GW_PAGE_VADDR_INT      0xC0000000L
#endif /* __LP64__ */
/* 
 * Determine ratio of hash entries to pdir entries. The default is 
 * to use the small table. Its possible that some Multi-user machines
 * or specific PA implementations may want to use the large table.
 */
#ifdef _KERNEL
#ifdef	LARGE_HTBL
#define HASH_PDIR_RATIO 2
#else	/* !LARGE_HTBL */
#define HASH_PDIR_RATIO 1       
#endif	/* LARGE_HTBL */
#endif /* _KERNEL */

#ifdef __LP64__
#define DRV_PSW 0x0804000F	/* PSW for device drivers in 64-bit mode */
#else  /* !__LP64__ */
#define DRV_PSW 0x0004000F	/* PSW for device drivers in 32-bit mode */
#endif /* !__LP64__ */

/*
 * Word size for all HP Precision Architecture processors is 4 bytes (32 bits)
 */
#define WORD_SIZE       4
#define WORD_SIZE_BITS  32

#ifdef MP_SDTA
#  define SDTA_PROTID	61002	/* the WD bit is not turned on */
#endif /* MP_SDTA */

#  define MP_RENDEZ_FAILED  -2
#  define MP_NOT_VALID      -1
#  define MPBLOCK 0	/* MUST BE 0, to save instructions in asm_utl.s */
#  define MPIDLE  1
#  define MPUSER  2
#  define MPSYS   3
#  define MPSWAIT 4
/*
 *  Definitions for sdta_flags word
 */
#  define SDTA_F_IGNORE	0x1	/* ignore faults when this bit set */

/*
 * Interrupt Control Stack Size
 */

#ifdef __LP64__
/* wide (64-bit) mode. */
#  ifdef SEMAPHORE_DEBUG
#    define ICS_SIZE	roundup(56*1024, NBPG)	/* 56kb */
#  else /* SEMAPHORE_DEBUG */
#    define ICS_SIZE	roundup(44*1024, NBPG)	/* 44kb */
#  endif /* SEMAPHORE_DEBUG */
#else  /* !__LP64__ */
/* narrow (32-bit) mode. */
#  ifdef SEMAPHORE_DEBUG
#    define ICS_SIZE	roundup(28*1024, NBPG)	/* 28kb */
#  else /* SEMAPHORE_DEBUG */
#    define ICS_SIZE	roundup(22*1024, NBPG)	/* 22kb */
#  endif /* SEMAPHORE_DEBUG */
#endif /* !__LP64__ */

/*
 * This section has structures used by C.
 */
#ifndef LOCORE
#include <sys/types.h>

#ifdef	SPP_RUNWAY_ERR_ENABLED
/*
 * On the SPP, the io_eir and io_command CSRs in the processor hpa must be
 * written as 64 bits with the 32-bit data occupying the upper half of the
 * doubleword.  The following macro is used for this operation.  On other
 * platforms, this macro expands to a simple assignment.
 */
#define	PROC_HPA_WRITE(LOC, VAL)	{			\
	if (is_SPP())						\
		/*lint -save -e740 -e571 */				\
		*((uint64_t *)&(LOC)) = ((uint64_t)(VAL))<<32;	\
		/*lint -restore */				\
	else							\
		LOC = VAL;					\
}
#else	/* SPP_RUNWAY_ERR_ENABLED */
#define	PROC_HPA_WRITE(LOC, VAL)	LOC = VAL
#endif	/* SPP_RUNWAY_ERR_ENABLED */
/* map of processor's SRS in hard physical address space */
struct processorhpa {
        volatile uint32_t   io_eir;
        volatile uint32_t   nothing[11];
        volatile uint32_t   io_command;
};
#define _PROCESSORHPA_T

/* Should point to the primary lasi. Used for soft power switch */
extern struct lasicorehpa *primary_lasi_regs;	

struct corehpa {			/* s700 */
	volatile char	core_ioss_reset;	/* wo	IOSS Reset	      */
	volatile char	reserved1[31];
	volatile char 	core_asp_version;	/* ro	ASP version ctl byte  */
	volatile char	reserved2[31];
	volatile char	error_log;	        /* rw	Asp error log byte    */
};

/* Core commands 			  s700 */
#define CORE_RESET	0x1

struct corehpa2 {
        volatile int           irr; 		/* +0x00 - Interrupt Request  */
        volatile int           imr; 		/* +0x04 - Interrupt Mask     */
        volatile int           ipr; 		/* +0x08 - Interrupt Pending  */
	volatile int	       icr;		/* +0x0C - LASI int control   */
	volatile int	       iar;		/* +0x10 - INT Adress         */
        volatile int           reserved1[3]; 	/* +0x14		      */
        volatile unsigned char led_control;	/* +0x20 - LED Control	      */
        volatile char          reserved2[3]; 	/* +0x21		      */
        volatile int           status; 		/* +0x24 - I/O Subsys Status  */
};

/* Structures and defines for Asp interrupt handling. */

struct asp_isr_link {
    int          (*isr)();
    long         arg1;
    long         arg2;
};

/* map of s700 memory controller (Viper) registers */
struct viper {
        volatile int     reserved1[9]; 		/* +0x00		      */
        volatile int     io_spa;		/* +0x24 - IO_SPA	      */
        volatile int     reserved2[22];		/* +0x28		      */
        volatile int     control;		/* +0x80 - VIPER_CONTROL      */
        volatile int     status;		/* +0x84 - VIPER_STATUS       */
        volatile int     intr_word;		/* +0x88 - INTERRUPT_WORD     */
        volatile int     reserved3[13];		/* +0x8C		      */
        volatile int     mem_control;		/* +0xC0 - MEM_CONTROL	      */
        volatile int     mem_write_chk_data;    /* +0xC4 - MEM_WRITE_CHK_DATA */
        volatile int     mem_limit;             /* +0xC8 - MEMORY_LIMIT       */
        volatile int     reserved4;             /* +0xCC                      */
        volatile int     mem_err_word_0;        /* +0xD0 - MEM_ERROR_WORD_0   */
        volatile int     mem_err_word_1;        /* +0xD4 - MEM_ERROR_WORD_1   */
        volatile int     mem_err_check_byte;    /* +0xD8 - MEM_ERROR_CK_BYTE  */
	volatile int     mem_read_chk_data;     /* +0xDC - MEM_READ_CHK_DATA  */
	volatile int     error_address;         /* +0xE0 - ERROR_ADDRESS      */
        volatile int     reserved5[967];        /* +0xE4		      */
};


/*
 * In Cobra, this structure is based at 0xFFFBC000, but in the general
 * case, it will be pointed to by the variable "venom".
 */
struct venom {
	/* Venom User HPA */
	volatile int	reserved0[0x20];	/* 0x0000 */
	volatile int	user_control;		/* 0x0080 */
	volatile int	span_count;		/* 0x0084 */
	volatile int	graphics_address;	/* 0x0088 */
	volatile int	reserved1;
	volatile int	zslope_fraction;	/* 0x0090 */
	volatile int	zslope_integer;		/* 0x0094 */
	volatile int	z_fraction;		/* 0x0098 */
	volatile int	z_integer;		/* 0x009C */
	volatile int	reserved2[8];
	volatile int	blue_slope;		/* 0x00C0 */
	volatile int	blue_color;		/* 0x00C4 */
	volatile int	reserved3[2];
	volatile int	red_slope;		/* 0x00D0 */
	volatile int	red_color;		/* 0x00D4 */
	volatile int	reserved4[2];
	volatile int	green_slope;		/* 0x00E0 */
	volatile int	green_color;		/* 0x00E4 */
	volatile int	reserved5[0x3C6];
	/* Venom Supervisor HPA */
	volatile int	reserved6[0x20];	/* 0x1000 */
	volatile int	sup_control;		/* 0x1080 */
	volatile int	z_address;		/* 0x1084 */
};

/* For Asp arbitration macros */
#define VIPER_CONTROL_COPY      0xDC

/* map of a module's registers in hard physical address space.
 Some are undefined for any particular module. */
struct io_module {
        volatile uint32_t nothing;		/* reg 0 */
        volatile uint32_t io_eim;
        volatile uint32_t io_dc_adata;
        volatile uint32_t io_ii_cdata;
        volatile uint32_t io_dma_link;		/* reg 4 */
        volatile uint32_t io_dma_command;
        volatile uint32_t io_dma_address;
        volatile uint32_t io_dma_count;
        volatile uint32_t io_flex;		/* reg 8 */
        volatile uint32_t io_spa_address;
        volatile uint32_t reserved1[2];
        volatile uint32_t io_command;		/* reg 12 */
        volatile uint32_t io_status;
        volatile uint32_t io_control;
        volatile uint32_t io_data;
        volatile uint32_t reserved2;		/* reg 16 */
        volatile uint32_t chain_addr;
        volatile uint32_t sub_mask_clr;
        volatile uint32_t reserved3[13];
        volatile uint32_t undefined[480];
        volatile uint32_t unpriv[512];
};

struct io_module_adma {
        volatile uint32_t reserved0;		/* reg 0     (0x00) SRS */
        volatile uint32_t io_eim;
        volatile uint32_t io_dc_adata;
        volatile uint32_t io_ii_cdata;
        volatile uint32_t reserved1[4];		/* reg 4-7   (0x10) */
        volatile uint32_t reserved2;		/* reg 8     (0x20) */
        volatile uint32_t io_spa_address;
        volatile uint32_t reserved3[2];
        volatile uint32_t io_command;		/* reg 12    (0x30) */
        volatile uint32_t io_status;
        volatile uint32_t io_control;
        volatile uint32_t io_data_inout;

        volatile uint32_t io_err_sadd;		/* reg  0    (0x40) ARS */
        volatile uint32_t io_err_info;
        volatile uint32_t io_err_madd;
        volatile uint32_t reserved4;
        volatile uint32_t io_ars_dma_link;	/* reg  4    (0x50) */
        volatile uint32_t io_ars_dma_command;
        volatile uint32_t io_ars_dma_address;
        volatile uint32_t io_ars_dma_count;
        volatile uint32_t io_ars_reserved[8];	/* reg 8-15  (0x60) */

        volatile uint32_t io_admars_reserved0;	/* reg 0     (0x80) ADMARS */
        volatile uint32_t io_admars_eim;
        volatile uint32_t io_admars_reserved1;            
        volatile uint32_t io_admars_cdata;
        volatile uint32_t io_admars_dma_link;	/* reg 4     (0x90)*/
        volatile uint32_t io_admars_dma_command;
        volatile uint32_t io_admars_dma_address;
        volatile uint32_t io_admars_dma_count;
        volatile uint32_t io_admars_reserved2[4];/* reg 8-11  (0xA0) */
        volatile uint32_t io_admars_io_command;  /* reg 12    (0xB0) */
        volatile uint32_t io_admars_io_status;
        volatile uint32_t io_admars_io_reserved3;
        volatile uint32_t io_admars_data_inout;

        volatile uint32_t lower_trs1[16];	/* (0x0C0) lower page TRSes */
        volatile uint32_t lower_trs2[16];	/* (0x100) */
        volatile uint32_t lower_trs3[16];	/* (0x140) */
        volatile uint32_t lower_trs4[16];	/* (0x180) */
        volatile uint32_t lower_trs5[16];	/* (0x1C0) */
        volatile uint32_t lower_trs6[16];	/* (0x200) */
        volatile uint32_t lower_trs7[16];	/* (0x240) */
        volatile uint32_t lower_trs8[16];	/* (0x280) */
        volatile uint32_t lower_trs9[16];	/* (0x2C0) */
        volatile uint32_t hvrs[320];		/* (0x300) HVRSes */
        volatile uint32_t upper_trs[512];	/* upper page TRSes */
};

struct bc_module {
        volatile uint32_t unused1[12];
        volatile uint32_t io_command;
        volatile uint32_t io_status;
        volatile uint32_t io_control;
        volatile uint32_t unused2[1];
        volatile uint32_t io_err_resp;
        volatile uint32_t io_err_info;
        volatile uint32_t io_err_req;
        volatile uint32_t unused3[11];
        volatile uint32_t io_io_low;
        volatile uint32_t io_io_high;
};

/* XXX: delete this when all code (pll_utl.c & pdc_rqsts.h) have changed
 * to use iodc_hversion, iodc_spa, and iodc_type.
 */

struct mod_type {
    unsigned int hw_rev:16,      /* HVERSION model and rev */
                 io_spa:1,       /* 1 = io_spa, 0 = memory spa */
                 :1,             /* zero bit */
                 :1,             /* SV bit */
                 shift:5,        /* max spa_size = 2 ** shift */
                 :1,             /* mr bit (unused) */
                 :1,             /* wd bit (unused) */
                 :1,             /* reserved bit */
                 mtype:5;        /* module type */
};

/* definitions for mod_type.io_spa */
#  define MEM_MOD 0
#  define IO_MOD  1

/* XXX: delete this superfluous definition when all code (cent0.c, scsi1.c,
 * pll_utl.c, & pdc_rqsts.h) have changed use iodc_sversion.
 */

struct swrev {
    unsigned int srev:4,         /* software revision number */
                 model:20,       /* software model number */
                 opt:8;          /* module dependent option */
};

/* IODC Fields */

typedef struct {
    unsigned int model:12,
                 rev:4;
} iodc_hversion_type;

typedef struct {
    unsigned int io:1,
                 Z:1,
                 sv:1,
                 shift:5;
} iodc_spa_type;

typedef struct {
    unsigned int mr:1,
                 wd:1,
                 R:1,
                 type:5;
} iodc_type_type;

typedef struct {
    unsigned int rev:4,
                 model:20,
                 opt:8;
} iodc_sversion_type;



#define NUM_SBUS_MODS	   16	/* Num Modules on S700 buses */

#  if	defined(_KERNEL) || defined(ANALYZE) || defined(RDB)


/* Defined for the LAN-DDB. This structure will generate code who's data 
 * alignment will actually match IODC return data (llt)
 */
typedef struct real_IODC_data {
        unsigned int    hw_model:12;
        unsigned int    hw_rev:4;
        unsigned int    spa_io:1;
        unsigned int    spa_Z:1;
        unsigned int    spa_sv:1;
        unsigned int    spa_shift:5;
        unsigned int    more:1;
        unsigned int    word:1;
        unsigned int    resv:1;
        unsigned int    type:5;
        unsigned int    sw_rev:4;
        unsigned int    sw_model:20;
        unsigned int    sw_opt:8;
        u_char          iodc_rev;
        u_char          iodc_dep;
        u_short         iodc_reserved;
        u_short         iodc_check;
        u_short         iodc_length;
} real_IODC_bytes;


/*
 * structures/unions that make low-level kernel debugging easier
 */

#ifdef __LP64__
typedef union _psw {
	uint64_t word;
	struct {
		unsigned int __RV0 : 24;
		unsigned int __CB0 : 8;
		unsigned int __Y   : 1;
		unsigned int __Z   : 1;
		unsigned int __RV1 : 2;
		unsigned int __W   : 1;
		unsigned int __E   : 1;
		unsigned int __S   : 1;
		unsigned int __T   : 1;
		unsigned int __H   : 1;
		unsigned int __L   : 1;
		unsigned int __N   : 1;
		unsigned int __X   : 1;
		unsigned int __B   : 1;
		unsigned int __C   : 1;
		unsigned int __V   : 1;
		unsigned int __M   : 1;
		unsigned int __CB  : 8;
		unsigned int __O   : 1;
		unsigned int __G   : 1;
		unsigned int __F   : 1;
		unsigned int __R   : 1;
		unsigned int __Q   : 1;
		unsigned int __P   : 1;
		unsigned int __D   : 1;
		unsigned int __I   : 1;
	} field;
} psw_t;
/*
 *  32-bit version of processor status word for narrow (32-bit) mode
 */
typedef union _psw32 {
	uint32_t word;
	struct {
		unsigned int __Y   : 1;
		unsigned int __Z   : 1;
		unsigned int __RV1 : 3;
		unsigned int __E   : 1;
		unsigned int __S   : 1;
		unsigned int __T   : 1;
		unsigned int __H   : 1;
		unsigned int __L   : 1;
		unsigned int __N   : 1;
		unsigned int __X   : 1;
		unsigned int __B   : 1;
		unsigned int __C   : 1;
		unsigned int __V   : 1;
		unsigned int __M   : 1;
		unsigned int __CB  : 8;
		unsigned int __RV2 : 1;
		unsigned int __G   : 1;
		unsigned int __F   : 1;
		unsigned int __R   : 1;
		unsigned int __Q   : 1;
		unsigned int __P   : 1;
		unsigned int __D   : 1;
		unsigned int __I   : 1;
	} field;
} psw32_t;
#else  /* ! __LP64__ */
typedef union _psw {
	uint32_t word;
	struct {
		unsigned int __Y   : 1;
		unsigned int __Z   : 1;
		unsigned int __RV1 : 3;
		unsigned int __E   : 1;
		unsigned int __S   : 1;
		unsigned int __T   : 1;
		unsigned int __H   : 1;
		unsigned int __L   : 1;
		unsigned int __N   : 1;
		unsigned int __X   : 1;
		unsigned int __B   : 1;
		unsigned int __C   : 1;
		unsigned int __V   : 1;
		unsigned int __M   : 1;
		unsigned int __CB  : 8;
		unsigned int __RV2 : 1;
		unsigned int __G   : 1;
		unsigned int __F   : 1;
		unsigned int __R   : 1;
		unsigned int __Q   : 1;
		unsigned int __P   : 1;
		unsigned int __D   : 1;
		unsigned int __I   : 1;
	} field;
} psw_t;
#endif /* !__LP64__ */

#  endif /* _KERNEL || ANALYZE || RDB */

#endif  /* not LOCORE */

#endif /* not _MACHINE_CPU_INCLUDED */
