/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/sys/frame.h		$Revision: $
 * 
 *  Purpose:
 *	Areas Associated with HP Precision Architecture Procedure Calling
 *	Convention
 *
 *	New structures and defines have been added for the 64-bit procedure
 *	calling convention.  Please note that there are now multiple declarations
 *	of these elements.  When making changes to this file, please make all
 *	appropriate changes to both versions of the structures and defines.
 * 
 *  Classification:			Release to Release Consistency Req:
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/frame.h
 *	/usr/conf/pa/frame.h
 *
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _MACHINE_FRAME_INCLUDED
#define _MACHINE_FRAME_INCLUDED


/*
 * Frame Marker
 */
#ifndef	LOCORE
# ifdef _KERNEL
typedef struct frame_marker {
	int	fm_edp;		/* External DP */
	int	fm_esr4;	/* External SR4 */
	int	fm_erp;		/* External/Stub RP (RP') */
	int	fm_crp;		/* Current RP */
	int	fm_sl;		/* Static Link */
	int	fm_clup;	/* Clean Up */
	int	fm_ep;		/* Extension Pointer */
	int	fm_psp;		/* Previous SP */
} frame_marker_t;

#  ifdef __LP64__
typedef struct frame_marker64 {
	long	fm_crp64;	/* Current RP */
	long	fm_psp64;	/* Previous SP */
} frame_marker64_t;
#  endif /* __LP64__ */

# else /* !_KERNEL */

#  ifdef __LP64__
typedef struct frame_marker {
	long	fm_crp;		/* Current RP */
	long	fm_psp;		/* Previous SP */
} frame_marker_t;
#  else /* !__LP64__ */
typedef struct frame_marker {
	int	fm_edp;		/* External DP */
	int	fm_esr4;	/* External SR4 */
	int	fm_erp;		/* External/Stub RP (RP') */
	int	fm_crp;		/* Current RP */
	int	fm_sl;		/* Static Link */
	int	fm_clup;	/* Clean Up */
	int	fm_ep;		/* Extension Pointer */
	int	fm_psp;		/* Previous SP */
} frame_marker_t;
#  endif /* __LP64__ */
# endif /* _KERNEL */
#endif /* LOCORE */

/*
 * Useful Constants
 */
#ifdef __LP64__
#define	NUMARGREGS 8 /* number of arguments in registers */
#else /* !__LP64__ */
#define	NUMARGREGS 4 /* number of arguments in registers */
#endif /* __LP64__ */

#define	NUMARGREGS32 4 /* number of arguments in registers */
#define	NUMARGREGS64 8 /* number of arguments in registers */

/*
 * Navigation Macros
 */
/* Stack Pointer to Frame Marker */
#define	sptofm(sp)	(((struct frame_marker *)(sp))-1)

#if defined(_KERNEL) && defined(__LP64__)
#define	sptofm64(sp)	(((struct frame_marker64 *)(sp))-1)
#endif

/* Stack Pointer to Argument */
/*
 * This macro returns the address of the last argument, which resides
 * at the lowest address of the arguments, and preceding arguments
 * reside at successively higher addresses.
 *
 *    ArgN, ... Arg1, Arg0
 *    Low address      Higher address
 *    ^
 */
#if defined(_KERNEL) || !defined(__LP64__)
#define	sptoarg(sp,i)	(((int *)sptofm(sp))-1-(i))
#endif

/* Argument Pointer to Argument */
/*
 * This macro returns the address of the first argument, which resides
 * at the lowest address of the arguments, and subsequent arguments
 * reside at successively higher addresses.
 *
 *    Arg0, Arg1, ... ArgN
 *    Low address      Higher address
 *    ^
 */
#ifdef __LP64__
#define	aptoarg(ap)	((long *)(ap)-NUMARGREGS64)
#endif

/*
 * Assembly Language Offsets for Argument or Stack Pointer
 *	to Status Save and Function Return Areas
 *	(should use genassym to get these offsets)
 */
#ifdef __LP64__
#define	FM_SIZE	 16
#else /* !__LP64 */
#define	FM_SIZE	 32
#endif /* __LP64 */
#define	FM_SIZE32	 32
#define	FM_SIZE64	 16

#ifdef _KERNEL
#define	FM_PSP	 -4
#define	FM_EP	 -8
#define	FM_CLUP	-12
#define	FM_SL	-16
#define	FM_CRP	-20
#define	FM_ERP	-24
#define	FM_ESR4	-28
#define	FM_EDP	-32

#ifdef __LP64__
#define	FM_PSP64	 -8
#define	FM_CRP64	-16
#endif /* __LP64__ */

#else /* !_KERNEL */

# ifdef __LP64__
#define	FM_PSP	 -8
#define	FM_CRP	-16
# else /* !__LP64__ */
#define	FM_PSP	 -4
#define	FM_EP	 -8
#define	FM_CLUP	-12
#define	FM_SL	-16
#define	FM_CRP	-20
#define	FM_ERP	-24
#define	FM_ESR4	-28
#define	FM_EDP	-32
# endif /* __LP64__ */
#endif /* _KERNEL */

#endif /* _MACHINE_FRAME_INCLUDED */
