/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/vm/hdl_pfdat.h		$Revision: $
 * 
 *  Purpose:
 *	This file contains the Hardware Dependent Specific fields for the 
 *	pfdat structure.
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/hdl_pfdat.h
 *	/usr/conf/pa/hdl_pfdat.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _HDL_PFDAT_INCLUDED
#define _HDL_PFDAT_INCLUDED

/*
 * Pad out the pfdat to a 32-byte cache-line size boundary.
 * Sizeof(struct pfdat) is 96 bytes for performance kernels.
 * We only do this for LP64 kernels as the needed padding
 * for ILP32 kernels (an extra 12 bytes) is too wasteful.
 */
#ifdef __LP64__
#define	HDL_PF_PAD	1
#else
#define	HDL_PF_PAD	0
#endif

struct pdkpfdat {
#ifdef __LP64__
	unsigned long pf_pfn : 32,  /* phys page #, saves divides */
		      pf_fill : 32;
#else
	unsigned      pf_pfn : 21,  /* phys page #, saves divides */
                      pf_fill : 11;
#endif
};

struct hdlpfdat {
	ushort hdlpf_flags;	/* HDL set when translation exists */
	ushort hdlpf_savear;	/* HDL set to saved access rights */
	prot_t hdlpf_saveprot;	/* HDL set to saved protection id */
};

#define HDLPF_TRANS	0x01  /* translation exists */
#define HDLPF_NOTIFY	0x02  /* virtual dma */
#define HDLPF_PROTECT	0x04  /* in the middle of a user protect/unprotect */
#define HDLPF_STEAL	0x08  /* translation should be removed */
#define HDLPF_ONULLPG	0x10  /* page 0 of text: nullref page is special cased
				 from default pregion acces rights for
				 backward compatibility */
#define HDLPF_MOD       0x20  /* physical page differs from bstore image */
#define HDLPF_REF       0x40  /* physical page has been referenced lately */
#define HDLPF_READA     0x80  /* Read-ahead page in transit. */

#define HDL_CW        0x1  /* copy on write page */
#define HDL_NEWTRANS  0x2  /* add new translation */
#define HDL_ADDTRANS  0x4  /* if trans exists change access, else add trans*/
#define HDL_REPTRANS  0x8  /* replace existing translation */
#define HDL_ANY_TRANS 0x10 /* get any virtual translation associated with */
			   /* the target pfn */
#define HDL_MATCH_TRANS 0x20 /* get a matching translation */

#ifdef	_KERNEL
#define HDL_GETBITS(PFN) (HDL_GETMODBIT(PFN)|HDL_GETREFBIT(PFN))

#define HDL_GETMODBIT(PFN) ((hdl_getbits(PFN)&VPG_MOD))
#define HDL_GETREFBIT(PFN) ((hdl_getbits(PFN)&VPG_REF))

#define BAD_PFN ((physpfn_t)~0U)

#ifndef _PROTOTYPES
void pdexchange_pages();
#endif /* _PROTOTYPES */

/* 
 * This is a macro instead of a routine for performance reasons.  It 
 * currently only does vasserts before calling pdexchange_pages, so 
 * it is not worth the procedure overhead
 * that would occur if this was implemented as a procedure.
 *
 * Assert that the source page is not a user page.
 * Assert that source and destination page are locked.
 * Assert that source and destination page use-counts are 1.
 * Assert that source and destination pages have translations.
 *
 * Note that exchanging pages > 4K is not supported.
 */

#define hdl_exchange_pages(sid, soff, spfn, aid, aoff, apfn, mp_protect) {  \
	pfd_t *spfd = PFDAT(spfn);					    \
	pfd_t *apfd = PFDAT(apfn);					    \
        ushort s_pfd_flags;                                                 \
        ushort a_pfd_flags;                                                 \
	VASSERT(spfd->pf_flags & P_SYS);				    \
	VASSERT(pfdatlockheld(spfd));					    \
	VASSERT(pfdatlockheld(apfd));					    \
	VASSERT(spfd->pf_use == 1);					    \
	VASSERT(apfd->pf_use == 1);					    \
	VASSERT(hdl_vpfn((sid),(soff)) == (spfn));			    \
	VASSERT(hdl_vpfn((aid),(aoff)) == (apfn));			    \
        s_pfd_flags = spfd->pf_flags;                                       \
        a_pfd_flags = apfd->pf_flags;                                       \
        if (s_pfd_flags & P_KERN_DYNAMIC) {                                 \
                apfd->pf_flags |=                                           \
                        (P_KERN_DYNAMIC | P_KERN_NO_LGPG);                  \
        }                                                                   \
        else {                                                              \
                apfd->pf_flags &=                                           \
                        ~(P_KERN_DYNAMIC | P_KERN_NO_LGPG);                 \
        }                                                                   \
        if (a_pfd_flags & P_KERN_DYNAMIC) {                                 \
                spfd->pf_flags |=                                           \
                        (P_KERN_DYNAMIC | P_KERN_NO_LGPG);                  \
        }                                                                   \
        else {                                                              \
                spfd->pf_flags &=                                           \
                        ~(P_KERN_DYNAMIC | P_KERN_NO_LGPG);                 \
        }                                                                   \
	pdexchange_pages(sid, soff, spfn, aid, aoff, apfn, mp_protect);	    \
}

#endif	/* _KERNEL */

#endif /* _HDL_PFDAT_INCLUDED */
