/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/vm/hdl_preg.h		$Revision: $
 * 
 *  Purpose:
 *      <<please update with a synopis of the functionality provided by this file>>
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/hdl_preg.h
 *	/usr/conf/pa/hdl_preg.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _MACHINE_HDL_PREG_INCLUDED
#define _MACHINE_HDL_PREG_INCLUDED

/*
 * Data structures to store page modes for mprotect()
 */
struct hdl_protrange {
    pgcnt_t idx;	/* starting index of range */
    prot_t protid;	/* protection id */
    u_short flags; 	/* flags */
};

#define PR_HDL_CACHE	0x01	/* Add this protid to fast_protid_list */

struct hdl_subregion {
    long nused;		    /* used elements in range[] */
    long nelements;		    /* actual size of range[] */
    long hint;		    /* hint index for next access */
    long refcnt;		    /* # of subpregions sharing this */
    struct hdl_protrange range[7];  /* ranges (can be bigger) */
    				    /* must be the last field */
};

typedef struct hdl_subregion hdl_subreg_t;

/* computes offset into a structure of a field */
#define Offsetof(Field,Struct) ((size_t)(&(((Struct *)0L)->Field)))

/*
 * The number of bytes of overhead in an hdl_subregion.
 * This is the sum of the sizes of all fields of the hdl_subpregion
 * excluding the range[] array, which must be the last field in the structure.
 */
#define HDL_SUBREGION_OVERHEAD	Offsetof(range[0], struct hdl_subregion)

struct hdl_subpregion {
    hdl_subreg_t *p_subreg;	/* subregion data for this pregion */
    long nelements;		/* actual size of mode[] */
    u_char mode[26];		/* modes (MPROT_*) for each range */
    				/* must be the last field */
};

typedef struct hdl_subpregion hdl_subpreg_t;

/*
 * The number of bytes of overhead in an hdl_subpregion.
 * This is the sum of the sizes of all fields of the hdl_subpregion
 * excluding the mode[] array, which must be the last field in the structure.
 */
#define HDL_SUBPREGION_OVERHEAD	Offsetof(mode[0], struct hdl_subpregion)

/*
 * HDL fields for the pregion data structures
 */
struct hdlpregion {
	u_short	hdlflags;	/* HDL pregion flags */
	u_short	hdlar;		/* HDL Access rights */
	prot_t	hdlprot;	/* HDL protection VALUE */
	hdl_subpreg_t *p_spreg;	/* for mprotect() */
};

/*
 * HDL flags for "hdlflags"
 */

#define PFHDL_ASSIGNED		0x0001	/* Space and such assigned	*/
#define PFHDL_NONULLREFPG  	0x0002	/* Trap null dereference       */
					/* in first half of first page */ 
                                        /* NBPG == 4096                 */
#define PFHDL_PRESTRICT		0x0020	/* Protection ID is restricted	*/
#define PFHDL_STATIC_PREDICT	0x0040	/* Text object using static branch
					   prediction. */

#ifdef _KERNEL
/*
 * Macro to determine if a pregion has been mprotect()ed.
 */
#define IS_MPROTECTED(prp)	((prp)->p_hdl.p_spreg != (void *)0L)

/* starting index of pregion */
#define hdl_pregstindx(PRP)	((pgcnt_t) 0)


#define LAZY_MLOCK(PRP)  (0)

/*
 * This macro defines the types of pregions to which POSIX
 * memory locks (mlocks) can be applied by user applications
 * via mlock/munlock/mlockall/munlockall.
 */

#define MLOCKABLE_PREGION(PRP)                                           \
        ( ( (PRP->p_type == PT_TEXT)   || (PRP->p_type == PT_DATA)    || \
        (PRP->p_type == PT_STACK)      || (PRP->p_type == PT_SHMEM)   || \
        (PRP->p_type == PT_MMAP))  ? 1 : 0 )

#define IsUserPregion(PRP) \
        ((PRP)->p_type == PT_TEXT || (PRP)->p_type == PT_DATA || \
         (PRP)->p_type == PT_STACK || (PRP)->p_type == PT_SHMEM || \
         (PRP)->p_type == PT_MMAP)


/*
 * Don't allow mprotection on anything except:
 *
 *      PT_DATA, PT_STACK, PT_MMAP, PT_SHMEM and PT_TEXT if the caller is
 *      EXEC_MAGIC.
 */
#define ValidMprotectType(PRP)                                           \
        ( (PRP->p_type == PT_MMAP) || (PRP->p_type == PT_DATA) ||        \
          (PRP->p_type == PT_SHMEM) || (PRP->p_type == PT_STACK) ||      \
          ((PRP->p_type == PT_TEXT) && (is_wrtext(PRP->p_vas))) )

#endif	/* _KERNEL */

#endif /* _MACHINE_HDL_PREG_INCLUDED */
