/*
 * BEGIN_DESC
 *
 *  File:
 *      @(#)B.11.11_LR	pa/svc/ica.h		$Revision: $
 *
 *  Purpose:
 *      Header definitions for kernel ica driver and trap handler.
 *
 *  Classification:			Release to Release Consistency Req:
 * 	kernel subsystem private		none
 *
 *  BE header:  no
 *
 *  Shipped:  yes
 *	/usr/include/pa/ica.h
 *	/usr/conf/pa/ica.h
 *
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 *
 * END_DESC
*/

#ifdef __NO_PA_HDRS
	PA header file -- do not include this header file for non-PA builds.
#endif

/*
 * THIS WHOLE MODULE IS ICA SPECIFIC AND MAY CHANGE AT ANY TIME.
 */

/*
 * ICA driver ioctl commands
 */

#define	IOC_ICASTART		_IO(0, 1)		/* start ica */
#define	IOC_ICASTOP		_IO(0, 2)		/* stop ica */
#define	IOC_ICAINIT		_IO(0, 3)		/* initialize */
#define	IOC_ICAGETINITLOC	_IOR(0, 4, off64_t)	/* initmap location */
#define	IOC_ICAGETHITLOC	_IOR(0, 5, off64_t)	/* hitmap location */
#define	IOC_ICAGETTEXTLENGTH	_IOR(0, 6, uint32_t)	/* ktext len (words) */
#define	IOC_ICAGETINITSTATE	_IOR(0, 7, uint32_t)	/* ever initialized? */
#define	IOC_ICAGETRUNSTATE	_IOR(0, 8, uint32_t)	/* ICA running? */
#define	IOC_ICAGETVERSION	_IOR(0, 9, uint32_t)	/* return version # */

/*
 * ICA kernel state
 */

typedef enum ica_state {
	ica_initoff = 0,
	ica_initon = 1,
	ica_ison = 2,
	ica_isoff = 3,
	ica_initfail = 4
} ica_state_t;

typedef struct ica_info {
	/*
	 * ica_initoff		do not do ICA initialization
	 * ica_initon		do ICA initialization
	 * ica_ison		initialization done and ICA is on
	 * ica_isoff		initialization done, but ICA is off
	 * ica_initfail		ICA initialization failed
	 */
	ica_state_t initflag;

	/*
	 * Pointer to the hitmap bitmap.  The i'th bit represents the i'th
	 * instruction in the kernel.  A 1 indicates that a breakpoint has
	 * been hit, and a zero that the location has not been hit (it
	 * might or might not have a breakpoint).
	 */
	int *hitmap;

	/*
	 * Pointer to the initmap bitmap.  The i'th bit represents the i'th
	 * instruction in the kernel.  A 1 indicates that this location is
	 * to receive a breakpoint upon initiaization.  A 0 means it should
	 * be left as is.  Note that since initmap is used to detect errors
	 * in the break handler, it should not be altered unless ICA is
	 * turned off.
	 */
	int *initmap;

	/*
	 * Pointer to the breakmap bitmap.  The i'th bit represents the
	 * i'th instruciton in the kernel.  A 1 indicates that this
	 * location has a breakpoint.
	 */
	int *breakmap;

	/*
	 * The length of both the hitmap and the initmap (in bytes).
	 * (Supports 64GB without even needing the sign bit.)
	 */
	int maplength;

	/*
	 * Pointer to a copy of kernel text (used when breakpoints need
	 * replacing).
	 */
	int *textcopy;

	/*
	 * Pointer to the beginning of kernel text.
	 */
	int *text;

	/*
	 * The length of kernel text (in words).
	 */
	int textlength;

	/*
	 * The number of breakpoints set at last initialization.
	 */
	int initcount;

	/*
	 * The number of breakpoints hit since last initialization.
	 */
	int hitcount;

	/*
	 * Pointer to beginning of the history array.
	 */
	uint64_t *history;

	/*
	 * The length of the history array (in elements).
	 */
	int histlength;
} ica_info_t;

/*
 * ICA history buffer size
 */

#define	HISTORYLENGTH	8192		/* length (in elements, power of 2) */
#define	HISTORYMASK	(HISTORYLENGTH-1)
