/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/pdc/iodc.h		$Revision: $
 * 
 *  Purpose:
 *	Header file for IODC routines.  Contains page zero addresses.
 *	
 *      Page Zero is architected to contain values needed by
 *      various IODC and boot routines.
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/iodc.h
 *	/usr/conf/pa/iodc.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _MACHINE_IODC_INCLUDED /* allows multiple inclusion */
#define _MACHINE_IODC_INCLUDED

#ifndef _SYS_STDSYMS_INCLUDED
#    include <sys/stdsyms.h>
#endif   /* _SYS_STDSYMS_INCLUDED  */


#define PDC_ENTRY_PT	    0x0388 /*  address of ptr to PDC_Entry */
#define FRST_MEM_HPA	    0x0384 /*  address of 1st memory HPA   */
#define MEM_10MSE	    0x038C /*  address of timer calibration  */
#define IODC_MEMCNFIG       0x0390 /*  address of mem config structure  */
#define IODC_CONSOLE        0x03A0 /*  address of console structure     */
#define IODC_BOOT_DEV       0x03D0 /*  address of boot device structure  */
#define IODC_KBD            0x0400 /*  address of keyboard device structure  */

/*  These structures  are defined in the I/O ACD or System Initialization
 *  ACD.  For the most part these are stored in page zero of memory.
 */

#ifndef LOCORE

struct device_path {
    unsigned char flags;	/* autoboot, autosearch, & timer */
    unsigned char bc[6];	/* bus converters on path to device */
    unsigned char mod;		/* fixed field of the module */
    uint32_t	  layer[6];	/* device-dependent informaiton */
};

/*  The hpa and spa fields below are stored in page zero as 32 bit values.
 *  These must be sign extended when they are promoted to 64 bit values.
 */
struct device_entry {
    struct device_path	path;
    uint32_t		hpa;	/* 32-bit hard physical address */
    uint32_t		spa;	/* 32-bit soft physical address */
    uint32_t		iodc_io; /* physaddr of ENTRY_IO entry point */
    uint32_t		class;	/* I/O device class (below) */
};

/* Constants for `class'	*/
#define CL_NULL		0	/* Invalid (no device)*/
#define	CL_RANDOM	1	/* Random-access (disk)		*/
#define	CL_SEQU		2	/* Sequential-access (tape)	*/
#define	CL_DUPLEX	7	/* Full duplex pt-to-pt (RS232)	*/
#define	CL_KEYBD	8	/* Half duplex console(keyboard)*/
#define	CL_DISPL	9	/* Half duplex console (Display)*/

#define VSC_NULL_PATH	0xff	/* Used to identify a graphic path vs a core. */

/* AUTOBAHN: remove these structures after rolling the folks who use it */
struct dev_path {
	unsigned char	io_dev_type;
	unsigned char	bus_converter[6];
	unsigned char	io_mod_adapter;
	uint32_t	io_logical_id;
	uint32_t	device_depend[5];
};

struct dev_entry {
	struct dev_path io_dev_path;
	uint32_t	dev_hpa;
	uint32_t	dev_spa;
	uint32_t	iodc_entry_io;
	uint32_t	dev_class;
};

#endif
#endif /* ! _MACHINE_IODC_INCLUDED */
