/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/sys/param.h		$Revision: $
 * 
 *  Purpose:
 *      <<please update with a synopis of the functionality provided by this file>>
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/param.h
 *	/usr/conf/pa/param.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _MACHINE_PARAM_INCLUDED /* allow multiple inclusions */
#define _MACHINE_PARAM_INCLUDED

#include <sys/sysmacros.h>
#include <machine/param_shm.h>	/* kernel src: <machine/sys/param_shm.h> */

/*
 * Machine dependent constants for HP Precision Architecture.
 */

#ifndef PGSHIFT			/* avoid redefining with assym.s */
#  ifdef KLUDGE_TLB_MP
#    define PGSHIFT  11			/* LOG2(NBPG = 2048) */
#    define HPGSHIFT 11
#  else 
#    define PGSHIFT  12			/* LOG2(NBPG = 4096) */
#    define HPGSHIFT 12
#  endif 
#  define NBPG		(1<<PGSHIFT)	/* bytes/page */
#  define NBHPG 	(1<<HPGSHIFT)   /* bytes/hardware page */
#  define PGOFSET	(NBPG-1)	/* byte offset into page */
#  define BYTESHIFT	2		/* LOG2(bytes/word) */
#  define PGNDXWDTH	(32-PGSHIFT)	/* bit width of a physical page no
				         * (page index)	*/
#  define NBPG_PA83 2048		/* 2k page size for PA-RISC 1.0 */
#endif /* not PGSHIFT */

#define	MAXPHYS	(256 * 1024)	/* Maximum size of physical I/O transfer */

#define	_BIG_ENDIAN		/* Memory storage (bytes in word) */

/*
 * At 8.0 on HP-PA when the address of a function is taken in a process that
 * uses shared libraries an indirect plabel is created.  The address points
 * to a two word entry; the actual address of the function followed by the
 * r19 value when executing in that function.  An indirect plabel is 
 * indicated by the 2nd least significant bit being set in a text pointer.
 */

#define INDIRECT_PLABEL	0x02

#ifndef	LOCORE

/*
 * Most of the machine dependent macros are now in ../h/sysmacros.h.
 */

/* bytes to word and backwards */
#define btow(x) ((unsigned long)(x) >> BYTESHIFT)       /* bytes to word */
#define	btowr(x) (btow(((ulong_t)(x))+(WORD_SIZE-1))) /* round up # bytes to
					       		# words */
#define wtob(x) ( (ulong_t)(x) << BYTESHIFT )


#define	DELAY(n)	{ register int N = (n); while (--N > 0); }

/*
 * The following are parameters to lacc that indicate if lacc is to pfault
 * to try and resolve any lack of access. If LACC_PFAULT is specified, 
 * lacc will try to resolve the lack of access rights by calling
 * pfault. If LACC_NOPFAULT is specified lacc will just return 0, on a lack
 * of the specified access and it  will not call pfault.
 */
#define LACC_PFAULT	1
#define LACC_NOPFAULT	0

/*
 * When calling the lacc() family of interfaces, the caller should pass
 * LACC_READ or LACC_WRITE through the 'rw' parameter of the interfaces to
 * specify whether a read or a write access permission check is to be
 * performed on the target memory address.
 *
 * In the past, the callers of lacc() used to pass the B_READ or B_WRITE flag
 * to indicate whether a read or a write permission check should be done
 * on the target address.  However, file systems and I/O subsystem also use
 * B_READ and B_WRITE in the buf header to represent the direction of an I/O
 * operation (i.e. whether the I/O will read from or write to a device).  For
 * example, in the context of a read() system call, the buf header associated
 * with the I/O transaction will be marked with B_READ, which means "read data
 * from a file or device and write it into memroy.  In this case, to verify
 * whether the target address is writtable, useracc() should be called to check
 * for "write" permission.  If lacc() expects B_READ or B_WRITE as the 'rw'
 * flag, callers such as physio() would compare the B_READ/B_WRITE flag in the
 * buf header, then reserve the B_READ/B_WRITE flag before passing it to 
 * lacc(). This is confusing and has led to several bugs in the kernel where
 * the callers of lacc() took the B_READ/B_WRITE flag from the buf header
 * and passed them directly to lacc() without reversing the sense.  To
 * make it clear that the 'rw' parameter of lacc() are not the same as the 'rw'
 * direction of an I/O, we have created the LACC_READ and LACC_WRITE flags for
 * use by the lacc() family of interfaces.  Old code such as:
 *	useracc(addr, len, (rw == B_READ ? B_WRITE : B_READ));
 * has been replaced by
 *	useracc(addr, len, (rw == B_READ ? LACC_WRITE : LACC_READ));
 * for clarity.
 *
 *
 * IMPORTANT:  To ensure compatibility with ISU drivers which continue to pass
 *             B_READ/B_WRITE to lacc() instead of using LACC_READ/LACC_WRITE,
 *             the numeric values of LACC_READ/LACC_WRITE must remain the same
 *             as the values of B_READ/B_WRITE.
 */
#define LACC_READ	1	/* same value as B_READ */
#define LACC_WRITE	0	/* same value as B_WRITE */

#define	lkernacc(sid, off, bcnt, rw)	\
	lacc((sid), (off), (bcnt), (rw), 0, LACC_PFAULT)
#define	luseracc(sid, off, bcnt, rw)	\
	lacc((sid), (off), (bcnt), (rw), 3, LACC_PFAULT)
#define	useracc(addr, bcnt, rw)		\
	lacc(ldusid(addr), (addr), (bcnt), (rw), 3, LACC_PFAULT)

#define	useracc_nopfault(addr, bcnt, rw)	\
		lacc(ldusid(addr), (addr), (bcnt), (rw), 3, LACC_NOPFAULT)

#define	kernacc(addr, bcnt, rw)		acc((addr), (bcnt), (rw), 0)

#define	fuibyte(base)		fubyte(base)
#define	fuiword(base)		fuword(base)

/* kernel bcopy does not support ovbcopy functionality */
#ifndef _KERNEL
#define ovbcopy			bcopy
#endif

# ifdef _KERNEL

#define IS_PLABEL(addr) \
		((((ulong_t)addr & INDIRECT_PLABEL) || \
		  (is_64bit(u.u_procp))) ? TRUE : FALSE)


# ifdef __LP64__
/* 
 * Swizzle macro that will convert a 32 bit address to a
 * 64 bit address. Take the top 2 bits from the 32bit address and
 * propagate it to the top 2 bits of the 64bit address. The macro
 * returns a 64bit value when the kernel is in wide mode. When
 * the kernel is running in narrow mode, the macro is a no-op
 */
#  define SWIZZLE_VADDR(vaddr)                 \
                  ((uintptr_t)(vaddr) |        \
                  (((uintptr_t)(vaddr) & 0xC0000000L) << 32))
# else
#  define SWIZZLE_VADDR(vaddr)       (vaddr)
# endif /* __LP64__ */

/*
 * The following are noops for PA 
 */
#  define SWIZZLE_ADDR(vaddr)        (vaddr)
#  define SWIZZLE_IF_NARROW(vaddr)   (vaddr)
#  define UNSWIZZLE_ADDR(vaddr)      (vaddr)
#  define UNSWIZZLE_IF_NARROW(vaddr) (vaddr)

#define GVA_OFFSET_BITS        10
#define GVA_OFFSET_BITS_MASK   0x3ff

/* 
 * Get the top 10 bits of the offset from the vaddr passed in.
 */
#  define OFFSET_BITS_FOR_SPACE(vaddr)                    \
                           (LP64_KERNEL ?                 \
                            (((uintptr_t)(vaddr) >> 32) &  \
                              GVA_OFFSET_BITS_MASK) : 0L)
/*
 * Retrieve the 10 bits of offset from the GVA space passed in.
 */
#  define OFFSET_BITS_FROM_SPACE(space)                    \
                           (LP64_KERNEL ? ((space) & GVA_OFFSET_BITS_MASK) : 0L)

/* 
 * Return the true space part of the space field by masking out the
 * offset part.
 */
#  define TRUE_SPACE(space)                                       \
                           (LP64_KERNEL ?                          \
                             ((space) & ~GVA_OFFSET_BITS_MASK): \
                             (space))
#ifdef __LP64__
/*
 * Mask off the space register selection bits from a 64-bit offset
 */
#  define MASK_SR(vaddr)  ((ulong_t)vaddr & 0x3fffffffffffffffL)
#endif

# endif /* _KERNEL */
#endif /* not LOCORE */

#ifdef _KERNEL
/*
 * To be used by the trap code to convert the hardware generated space
 * and offset to something that the fault handlers can use.
 */
# define DATA_FAULT_ADDR  1
# define INSTR_FAULT_ADDR 2
#endif /* _KERNEL */

#ifndef KERNELSPACE
#define KERNELSPACE 0           /* see also vmparam.h */
#endif

/*
 * Decode privilege level for HP-PA.
 */
#define PC_PRIV_MASK	3
#define PC_PRIV_KERN	0
#define PC_PRIV_USER	3

/* This macro is to be used only in PDK files */
#define USERMODE(pc)	(((pc) & PC_PRIV_MASK) != PC_PRIV_KERN)

#ifdef _KERNEL
/* Macro for PIK use */
#define TRAP_IN_USERMODE(ssp)	\
		((GetSSReg(ssp, ss_pcoq_head) & PC_PRIV_MASK) != PC_PRIV_KERN)
#endif /* _KERNEL */

/* To be used by subsystems that needs to mark some pages as
 * certain types.
 * CLIC - Cluster Interconnect.
 */
#define CLIC 0

#endif /* _MACHINE_PARAM_INCLUDED */
