/* @(#) pdc_rqsts.h $Date: 2003/09/08 16:44:16 $Revision: r11.11/4 PATCH_11.11 (PHKL_29539) */
/*
 * BEGIN_DESC
 * 
 *  File: 
 *      	pa/pdc/pdc_rqsts.h		$Revision: $
 * 
 *  Purpose:
 *	Interface between HP-UX and PDC.
 * 
 *  Classification:			Release to Release Consistency Req:
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/pdc_rqsts.h
 *	/usr/conf/pa/pdc_rqsts.h
 *
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _MACHINE_PDC_RQSTS_INCLUDED /* allows multiple inclusion */
#define _MACHINE_PDC_RQSTS_INCLUDED

#ifndef _SYS_STDSYMS_INCLUDED
#    include <sys/stdsyms.h>
#endif   /* _SYS_STDSYMS_INCLUDED  */

# ifdef _KERNEL
#     ifndef LOCORE
        extern unsigned long page_zero; /* Logical "Page 0 image" address */
#     endif
#     define PAGE_ZERO(o)(page_zero+(o))
# endif /* _KERNEL */

/*
 * PDC request codes used to make a request to the PDC.
 */

/*
 * The following MACRO has been included because the assembler does not
 * recognize long constants. This macro determines whether the assembler
 * or the compiler is currently invoked by checking the define LOCORE
 * (which is only defined for .s files) and appends an L for .c files and
 * nothing for .s files.
 *
 * NOTE: This is only a temporary kludge. When we take a new assembler
 *       that recognizes long constants, we will remove the ifdef LOCORE
 *       part of this macro
 */

#ifndef LOCORE 
#ifdef __STDC__
#define LONG(x)		x ## L
#else /* !__STDC__ */
#define LONG(x)		x/**/L
#endif /* __STDC__ */
#else /* LOCORE */
#define LONG(x)		x
#endif /* !LOCORE */

#define PDC_POW_FAIL	 	LONG(1) /* Executes power-down sequence */
#define PDC_POW_FAIL_DEFAULT 	LONG(0)

#define PDC_CHASSIS	 	LONG(2) /* Displays message; gets warnings */
#define PDC_CHASSIS_DISPLAY 	LONG(0) /* Updates LED display */
#define PDC_CHASSIS_WARNING 	LONG(1) /* Returns environmental warnings */
#define PDC_CHASSIS_UPDATES 	LONG(2) /* Both of the above */
#define PDC_RETURN_CHASSIS_INFO LONG(128)    /* Unarchiteched Option */

#define	PDC_OSTAT_OFF	 	LONG(00) /* front panel status lights */
#define	PDC_OSTAT_FAULT	 	LONG(01) /* panic() leaves this */
#define	PDC_OSTAT_TEST	 	LONG(02)
#define	PDC_OSTAT_INITIALIZE 	LONG(03) /* ISL and HPUXBOOT leave this */
#define	PDC_OSTAT_SHUTDOWN 	LONG(04)
#define	PDC_OSTAT_WARNING 	LONG(05)
#define	PDC_OSTAT_RUN	 	LONG(06) /* display_activity() leaves this */
#define	PDC_OSTAT_ALL_ON 	LONG(07)

/* Non-Blocking Return codes */
#define PDC_RET_WAIT            LONG(5)   /* caller wait for PDH event */
#define PDC_RET_PENDING         LONG(6)   /* Pending, caller wait for PDH event
                                           * to complete */
#define PDC_RET_NO_ACTION       LONG(7)   /* No action occured */
#define PDC_RET_TOO_MANY_PEND   LONG(-18) /* Too many calls Pending to add this 
					   * one */
#define PDC_RET_NOT_COMPLETE    LONG(-19) /* System Utilities did not complete 
					   * the call */
#define PDC_RET_REQ_INVALID     LONG(-20) /* The Request_ID is invalid */
#define PDC_RET_REQ_PTR_INVALID LONG(-21) /* The Request_ID pointer is invalid */

/* Common PDC return codes */

#define PDC_RET_OK	 	LONG(0)         /* Std return - call completed ok */
#define PDC_RET_ALREADYON 	LONG(1)         /* Entity already on */
#define PDC_RET_ALREADYOFF 	LONG(2)         /* Entity already off */
#define PDC_RET_COMPL_WAR       LONG(3)         /* Call completed, with warning */
#define PDC_RET_NE_PROC		LONG(-1)	/* Non-existent procedure */
#define PDC_RET_NE_OPT		LONG(-2)	/* Non-existent option - arg1 */
#define PDC_RET_COMPL_ERR	LONG(-3)	/* Can't complete without error */
#define PDC_RET_LOG_FULL	LONG(-5)	/* Read/Write Log Entry failed
						 * due to chassis log buffer
						 * empty/full
						 */
#define PDC_RET_NE_MOD		LONG(-5)	/* Module not found */
#define PDC_RET_INV_NVM		LONG(-5)	/* Invalid NVM contents */
#define PDC_RET_BAD_LOCK_KEY	LONG(-6)	/* Invalid Lock Key */
#define PDC_RET_NE_CELL_MOD	LONG(-7)	/* Cell module not found */
#define PDC_RET_CANT_LOCK	LONG(-7)	/* Unable to get profile Lock */
#define PDC_RET_BAD_CELL_DATA	LONG(-8)	/* Invalid Cell data */
#define PDC_RET_NO_CABINET	LONG(-8)	/* Non-extant cabinet */
#define PDC_RET_INV_ARG		LONG(-10)	/* Invalid argument */
#define PDC_RET_PF_ABORT	LONG(-12)	/* Call aborted by powerfail warn */
#define PDC_RET_BAD_SUM		LONG(-13)	/* Bad Checksum */
#define PDC_RET_PROF_NO_WRITE	LONG(-14)	/* Read only area changed */
#define PDC_RET_BAD_AFF_CELL	LONG(-15)	/* Invalid affected cells */
#define PDC_RET_STACK_NOT_ALIGN	LONG(-16)	/* Stack not double word */
						/* aligned */
#define PDC_RET_NOT_NARROW      LONG(-17)       /* Narrow mode not supported */
#define PDC_RET_INTRA_PD	LONG(-22)	/* intRA PD conflict in PDC */
#define PDC_RET_INTER_PD	LONG(-23)	/* intER PD conflict in PDC */
#define PDC_RET_ARG_CHG		LONG(-24)	/* Non-pointer arguments */
						/* changed between initiating */
						/* the non-blocking call and */
						/* completing it  */
#define PDC_RET_ANOTHER_PD	LONG(-25)	/* The resources specified */
						/* by the caller belong to */
						/* another PD */
#define PDC_RET_RES_UNWRITE	LONG(-26)	/* The resources specified */
						/* by the caller cannot be */
						/* written because they are */
						/* in another RC domain */
#define PDC_RET_RES_UNREACH	LONG(-27)	/* The resources specified */
						/* by the caller could not */
						/* be reached
#define PDC_RET_NO_RANGE_LEFT	LONG(-28)	/* Out of sharing ranges */
#define PDC_RET_NO_DIRECTED	LONG(-30)	/* No directed ranges */
#define PDC_RET_BAD_DIRECTED	LONG(-31)	/* Incorrect directed */
						/* address range */
#define PDC_RET_BAD_SEQ_NUM	LONG(-31)	/* Illegal sequence number */
#define PDC_RET_INSF_POWER	LONG(-32)	/* The power on request */
						/* could not be satisfied */
						/* due to an N-1 power */
						/* condition that would result*/
#define PDC_RET_AMBIG_TRANS	LONG(-33)	/* Translation failed */
						/* because it is ambiguous */
#define PDC_RET_INV_PHYS_LOC	LONG(-34)	/* Invalid physical location */
#define	PDC_RET_RC_CLOSED	LONG(-35)	/* RC link is closed */
#define	PDC_RET_GSP_IOX		LONG(-38)	/* IOX not supported by GSP */
#define PDC_RET_LED_NI		LONG(-39)	/* LED control not implemented*/
#define PDC_RET_SUB_PUSH	LONG(-41)	/* Key is unavailable, SUB */
						/* pushing new profile */
#define PDC_RET_HV_AGAIN	LONG(-1234)     /* HVERSION dependent PDC call should
					         * be retried.*/

#define	ENCODE_OSTAT(n)		(((n) & 0x7) << 17)
#define	DECODE_OSTAT(n)		(((n) >> 17) & 0x7)

#define	PDC_PIM	 	LONG(3) 	/* Accesses Processor Internal Memory */
#define PDC_PIM_READ 	LONG(0) 	/* Read registers */
#define	PDC_PIM_SIZE 	LONG(1) 	/* Get size of state */
#define PDC_PIM_LPMC 	LONG(2) 	/* Read registers and LPMC status */
#define PDC_PIM_BOOT 	LONG(3) 	/* Soft boot registers */
#define PDC_PIM_TOC 	LONG(4) 	/* TOC registers */
#define PDC_PIM_ENABLE_LPMC_TRAPPING	LONG(5)	/* Enable LPMC trapping */
#define PDC_PIM_DISABLE_LPMC_TRAPPING	LONG(6)	/* Disable LPMC trapping */

#define PDC_MODEL	 	LONG(4) /* Returns processor's model number */
#define PDC_MODEL_CPU	 	LONG(0)
#define PDC_MODEL_BOOTID 	LONG(1)
#define PDC_MODEL_COMPONENT 	LONG(2)
#define PDC_MODEL_SYS_MODEL 	LONG(3)
#define PDC_MODEL_ENABLE 	LONG(4)
#define PDC_MODEL_DISABLE 	LONG(5)
#define PDC_MODEL_CPU_ID 	LONG(6)
#define PDC_MODEL_CAPABILITIES  LONG(7)
#define PDC_MODEL_FIRM_TEST_GET LONG(8)      /* Return the boot test options */
#define PDC_MODEL_FIRM_TEST_SET LONG(9)      /* Set    the boot test options */
#define PDC_MODEL_GET_PLAT_INFO LONG(10)     /* Get platform info            */

#define PDC_CACHE	 	LONG(5) /* Returns cache parameters */
#define PDC_CACHE_DEFAULT 	LONG(0)
#define PDC_CACHE_SPACE_BITS    LONG(2)      /* Return space_id bits */

#define PDC_HPA		 	LONG(6) /* Returns processor's hard phys addr */
#define PDC_HPA_DEFAULT	 	LONG(0)

#define PDC_COPROC	 	LONG(7) /* Identifies existing coprocessors */
#define PDC_COPROC_DEFAULT 	LONG(0)

#define PDC_IODC	 	LONG(8)
#define PDC_IODC_READ	 	LONG(0) /* Returns IODC from I/O card */
#define PDC_IODC_MEM_NDINIT 	LONG(2) /* Initializes memory spa */
#define PDC_IODC_MEM_CLEAR 	LONG(4) /* Returns and clears mem errors */
#define PDC_IODC_ID_CHIEFTAIN 	LONG(5) /* ID cheif of interleaved memory*/
#define IODC_INDEX_MODTYPE 	LONG(0) /* Module info in IODC */

#define PDC_TOD		 	LONG(9) /* Sets or reads time of day clock */
#define PDC_TOD_READ	 	LONG(0) /* Read TOD clock */
#define PDC_TOD_SET	 	LONG(1) /* Set TOD clock */
#define PDC_TOD_ITFREQ	 	LONG(2) /* Returns interval timer frequency */

#define PDC_STABLE	 	LONG(10) /* Reads or writes stable storage */
#define PDC_STABLE_READ	 	LONG(0) /* Read stable storage */
#define PDC_STABLE_WRITE 	LONG(1) /* Write stable storage */
#define PDC_STABLE_SIZE	 	LONG(2) /* Returns size of stable storage */
#define PDC_STABLE_VERIFY 	LONG(3) /* Verify contents of stable storage */
#define PDC_STABLE_INIT	 	LONG(4) /* Initialize stable storage */
#define PDC_STABLE_UNARCH_READ  LONG(1000)  /* Read Unarch stable storage */
#define PDC_STABLE_UNARCH_WRITE LONG(1001)  /* Write Unarch stable storage */
#define PDC_STABLE_UNARCH_SIZE  LONG(1002)  /* Returns size of Unarch stable 
				             * storage */
#define PDC_STABLE_UNARCH_VERIFY LONG(1003)  /* Verify contents Unarch stable storage */
#define PDC_STABLE_UNARCH_INIT  LONG(1004)  /* Initialize Unarch stable storage */

#define PDC_NVOLATILE	 	LONG(11) /* Reads or writes non-volatile RAM */
#define PDC_NVOLATILE_READ 	LONG(0) /* Read non-volatile RAM */
#define PDC_NVOLATILE_WRITE 	LONG(1) /* Write non-volatile RAM */
#define PDC_NVOLATILE_SIZE 	LONG(2) /* Returns size of NV RAM */
#define PDC_NVOLATILE_VERIFY 	LONG(3) /* Verify contents of NV RAM */
#define PDC_NVOLATILE_INIT 	LONG(4) /* Initialize NV RAM */

#define PDC_ADD_VALID	 	LONG(12) /* Tests validity of physical address */
#define PDC_ADD_VALID_DEFAULT 	LONG(0)

#define PDC_INSTR	 	LONG(15) /* Instruction for PDCE_CHECK */
#define	PDC_INSTR_HPMC	 	LONG(0)

#define PDC_BLOCK_TLB	 	LONG(18) /* Manage Block TLB entries (BTLB) */
#define PDC_BTLB_INFO	 	LONG(0) /* Return BTLB configuration info  */
#define	PDC_BTLB_INSERT	 	LONG(1) /* Insert a BTLB entry		   */
#define PDC_BTLB_PURGE	 	LONG(2) /* Purge a BTLB entry		   */
#define PDC_BTLB_PURGE_ALL 	LONG(3) /* Purge all BTLB entries	   */

#define PDC_TLB		 	LONG(19) /* Manage Hardware TLB handling */
#define PDC_TLB_INFO	 	LONG(0) /* Return HW-TLB configuration info  */
#define	PDC_TLB_CONFIG		LONG(1) /* Set HW-TLB PDIR base and size */
#define PDC_CACHE_SPACEID_BITS	LONG(2)

/* Configuration parameters for PDC_TLB: */
#define PDC_TLB_CURRPDE         LONG(1)       /* cr28 points to current pde on miss */
#define PDC_TLB_RESERVD         LONG(3)       /* reserved */
#define PDC_TLB_NEXTPDE         LONG(5)       /* cr28 points to next pde on miss */
#define PDC_TLB_WORD3           LONG(7)       /* cr28 is word 3 of 16 byte pde */

#define PDC_MEM		 	LONG(20) /* Manage memory page deallocation */
#define	PDC_MEM_INFO	 	LONG(0) /* Return PDT info		*/
#define	PDC_MEM_ADD	 	LONG(1) /* Add bad page to PDT		*/
#define PDC_MEM_CLEAR	 	LONG(2) /* Clear PDT			*/
#define	PDC_MEM_READ	 	LONG(3) /* Read PDT entries		*/
#define PDC_MEM_RESET	 	LONG(4) /* Reset clear bit		*/
#define PDC_MEM_SETGM	 	LONG(5) /* Set Golden Memory value	*/

/* 
 * The PDC_PSW call is currently (release 10.0) not used by hp-ux. It is
 * included here just for completeness.
 */
#define PDC_PSW              	LONG(21) /* Manage configurable PSW bits*/
#define PDC_PSW_MASK	 	LONG(0) /* Return mask                 	*/
#define PDC_PSW_DEFAULTS 	LONG(1) /* Return defaults		*/
#define PDC_PSW_SET_DEFAULTS 	LONG(2) /* Set mask defaults    	*/

#define PDC_SYSTEM_MAP	 	LONG(22) /* Find fixed address devices   */
#define PDC_FIND_MODULE	 	LONG(0) /* Find module			*/
#define PDC_FIND_ADDRESS 	LONG(1) /* Find additional address	*/
#define PDC_TRANSLATE_PATH 	LONG(2) /* Return module info from path */

#define PDC_SOFT_POWER      	LONG(23) /* support for soft power switch    */
#define PDC_SOFT_POWER_INFO 	LONG(0) /* get info about soft power switch */
#define PDC_SOFT_POWER_ENABLE 	LONG(1) /* enable/disable soft power switch */

/* This call is supported in Prelude and used for allocating 
 * Space for PDC, IODC and Offline Diagnostics.
 */
#define PDC_ALLOC           	LONG(24) /* Allocate storage for PDC or IODC use */
#define PDC_ALLOC_DEFAULT    	LONG(0) /* Allocate storage */
#define PDC_ALLOC_PDH_RAM 	LONG(1000) /* Allocate PDH RAM */

#define PDC_CRASH_PREP		LONG(25) /* Setup I/O for Crash & Dump */
#define PDC_CRASH_PREP_DUMP	LONG(0)  /* Prepare for Dump */
#define PDC_CRASH_PREP_LOG_ERR	LONG(1)  /* Log CEC Errors */

#define PDC_SCSI_PARMS		LONG(26) /* SCSI parameters storage */
#define PDC_SCSI_PARMS_GET	LONG(0)  /* Set SCSI parameters */
#define PDC_SCSI_PARMS_SET	LONG(1)  /* Get SCSI parameters */

/**********  Start of PDC PAT procedure call defines  ****************
 *** Initial PDC PAT call definitions added based on:
 **  HDW PDC ERS Rev 1.5.2 (A-A3261-XXXXX-31) July 24, 1998
 */

/* PDC PAT CELL */
#define PDC_PAT_CELL           	LONG(64)   /* Interface for gaining and 
                                         * manipulating cell state within PD */
#define PDC_PAT_CELL_GET_NUMBER    LONG(0)    /* Return Cell number */
#define PDC_PAT_CELL_GET_INFO      LONG(1)    /* Returns info about Cell */
#define PDC_PAT_CELL_MODULE        LONG(2)    /* Returns info about Module */
#define PDC_PAT_CELL_OL_ADD        LONG(3)    /* On Line addition of a Cell */
#define PDC_PAT_CELL_OL_DELETE     LONG(4)    /* On Line Deletion of a Cell */
#define PDC_PAT_CELL_OL_POW_ON     LONG(5)    /* To turn POWER ON for Cell */
#define PDC_PAT_CELL_OL_POW_OFF    LONG(6)    /* To turn POWER OFF for Cell */
#define PDC_PAT_CELL_LOC_TO_NUMBER LONG(8)    /* Cell Location -> Number */
#define PDC_PAT_CELL_SET_ATTENTION LONG(9)    /* Set Cell Attention Indicator */
#define PDC_PAT_CELL_NUMBER_TO_LOC LONG(10)   /* Cell Number -> Location */
#define PDC_PAT_CELL_WALK_FABRIC   LONG(11)   /* Walk the Fabric */
#define PDC_PAT_CELL_GET_RDT_SIZE  LONG(12)   /* Return Route Distance Table Sizes */
#define PDC_PAT_CELL_GET_RDT       LONG(13)   /* Return Route Distance Tables */
#define PDC_PAT_CELL_GET_PDH_SIZE  LONG(14)   /* Read Local PDH Buffer Size */
#define PDC_PAT_CELL_SET_PDH       LONG(15)   /* Write Local PDH Buffer */
#define PDC_PAT_CELL_GET_REMOTE_PDH LONG(17)   /* Read Remote PDH Buffer */
#define PDC_PAT_CELL_GET_DBG_INFO   LONG(128) /* Return DBG Buffer Info */
#define PDC_PAT_CELL_SET_FABRIC_REG LONG(140) /* Set Link in-use */
#define PDC_PAT_CELL_CLR_FABRIC_REG LONG(141) /* Clear Link in-use */

/* Attention Parameter for all attention options */
#define ATTN_OFF        LONG(0)
#define ATTN_ON         LONG(1)
#define ATTN_ATTN       LONG(2)

/* PDC PAT CHASSIS LOG */

#define PDC_PAT_CHASSIS_LOG		LONG(65) /* Platform logging and */
						 /* forward progress functions */
#define PDC_PAT_CHASSIS_WRITE_LOG    	LONG(0) /* Write Log Entry */
#define PDC_PAT_CHASSIS_READ_LOG     	LONG(1) /* Read  Log Entry */
#define PDC_PAT_CHASSIS_GET_FIFO_DEPTH  LONG(2) /* Return FIFO Depth */
#define PDC_PAT_CHASSIS_READ_MULTI	LONG(3) /* Read Multiple Log Entries */

/*  PDC_PAT_COMPLEX */

#define PDC_PAT_COMPLEX             LONG(66) /* performs functions for entire complex */
#define PDC_PAT_COMPLEX_GET_FRU      	LONG(0) /* Get FRU Handles */
#define PDC_PAT_COMPLEX_GET_FRUID    	LONG(1) /* Get FRUID */
#define PDC_PAT_COMPLEX_GET_PROFILE 	LONG(2) /* Get Static Profile */
#define PDC_PAT_COMPLEX_SET_PROFILE 	LONG(3) /* Write Static Profile */
#define PDC_PAT_COMPLEX_LOCK_PROFILE    LONG(4) /* Lock Static Profile */
#define PDC_PAT_COMPLEX_UNLOCK_PROFILE  LONG(5) /* Unlock Static Profile */
#define PDC_PAT_COMPLEX_GET_CABINET  	LONG(6) /* Get Cabinet Info */
#define PDC_PAT_COMPLEX_GET_CELL_STATES LONG(7) /* Get All Cell States */
#define PDC_PAT_COMPLEX_GET_CELL_LIST 	LONG(8) /* Get Root Cell List */
#define PDC_PAT_COMPLEX_GET_DYN_PROFILE LONG(9) /* Get Dynamic Profile */
#define PDC_PAT_COMPLEX_SET_DYN_PROFILE LONG(10) /* Set Dynamic Profile */
#define PDC_PAT_COMPLEX_LOCK_DYN_PROFILE LONG(11) /* Lock Dynamic Profile */
#define PDC_PAT_COMPLEX_UNLOCK_DYN_PROFILE LONG(12) /* Unlock Dynamic Profile */
#define PDC_PAT_COMPLEX_GET_MAX_CABS	LONG(13)  /* Get Max # Cabinets */
#define PDC_PAT_COMPLEX_SET_CAB_ATTENTION LONG(14)  /* Set Cabinet Indicator */
#define PDC_PAT_COMPLEX_GET_FABRIC_ID	LONG(15)  /* Phys Loc -> Fabric Id */
#define PDC_PAT_COMPLEX_GET_PHYS_LOC	LONG(16)  /* Fabric Id -> Phys Loc */
#define PDC_PAT_COMPLEX_OPEN_RC        	LONG(128) /* Open RC Link */
#define PDC_PAT_COMPLEX_CLOSE_RC       	LONG(129) /* Close RC Link */
#define PDC_PAT_COMPLEX_CLEAR_RC_BITS  	LONG(131) /* Clear RC In-Use Bits */
#define PDC_PAT_COMPLEX_GET_RC_STATUS  	LONG(132) /* Get RC Status */
#define PDC_PAT_COMPLEX_CLEAR_RC_ERRORS	LONG(133) /* Clear RC Errors */
#define PDC_PAT_COMPLEX_UPDATE_RCD_ROUT	LONG(134) /* Update Inter-RCD Routing */
#define PDC_PAT_COMPLEX_GET_RCD_ROUTING	LONG(135) /* GET RCD Routing */

/* PDC PAT CPU  */

#define PDC_PAT_CPU                	LONG(67) /* Interface to CPU configuration
                                        	* within the protection domain */
#define PDC_PAT_CPU_INFO            	LONG(0) /* Return CPU config info */
#define PDC_PAT_CPU_DELETE          	LONG(1) /* Delete CPU */
#define PDC_PAT_CPU_ADD             	LONG(2) /* Add    CPU */
#define PDC_PAT_CPU_GET_NUMBER      	LONG(3) /* Return CPU Number */
#define PDC_PAT_CPU_GET_HPA         	LONG(4) /* Return CPU HPA */
#define PDC_PAT_CPU_STOP            	LONG(5) /* Stop   CPU */
#define PDC_PAT_CPU_RENDEZVOUS      	LONG(6) /* Rendezvous CPU */
#define PDC_PAT_CPU_GET_CLOCK_INFO  	LONG(7) /* Return CPU Clock info */
#define PDC_PAT_CPU_GET_RENDEZVOUS_STATE LONG(8) /* Return Rendezvous State */
#define PDC_PAT_CPU_PLUNGE_FABRIC 	LONG(9) /* Plunge Fabric */
#define PDC_PAT_CPU_UPDATE_CACHE_CLEANSING LONG(128) /* Manipulate Cache 
                                                 * Cleansing Mode */
/*  PDC PAT EVENT */

#define PDC_PAT_EVENT              	LONG(68) /* Interface to Platform Events */
#define PDC_PAT_EVENT_GET_CAPS     	LONG(0) /* Get Capabilities */
#define PDC_PAT_EVENT_SET_MODE     	LONG(1) /* Set Notification Mode */
#define PDC_PAT_EVENT_SCAN         	LONG(2) /* Scan Event */
#define PDC_PAT_EVENT_HANDLE       	LONG(3) /* Handle Event */
#define PDC_PAT_EVENT_GET_NB_CALL  	LONG(4) /* Get Non-Blocking call Args */

/*  PDC PAT GSM */

#define PDC_PAT_GSM                	LONG(69) /*Interface to Global Shared Memory*/
#define PDC_PAT_GSM_OPEN_RANGE      	LONG(0) /* Open Sharing Range */
#define PDC_PAT_GSM_GET_NUM_RANGES  	LONG(1) /* Return Number Share Ranges */
#define PDC_PAT_GSM_GET_RANGE_STATUS 	LONG(2) /* Return Share Range Status */
#define PDC_PAT_GSM_ADD			LONG(3) /* Add to Coherency Set */
#define PDC_PAT_GSM_REMOVE		LONG(4) /* Remove from Coherency Set */
#define PDC_PAT_GSM_GET			LONG(5) /* Get Coherency Set */
#define PDC_PAT_GSM_SET_NOSHARE		LONG(6) /* Set No Share Recovery Mode */
#define PDC_PAT_GSM_SET_NORMAL		LONG(7)	/* Set Normal Mode */
#define PDC_PAT_GSM_MAP_REMOTE		LONG(8) /* Map Remote PD Memory */
#define PDC_PAT_GSM_UNMAP_REMOTE	LONG(9) /* Unmap Remote PD Memory */
#define PDC_PAT_GSM_SET_INTR		LONG(10)/* Set GSM Interrupt */
#define PDC_PAT_GSM_CLEAR_INTR		LONG(11)/* Clear GSM Interrupt */
#define PDC_PAT_GSM_GET_INTR		LONG(12)/* Get GSM Interrupt */

/*  PDC PAT HPMC */

#define PDC_PAT_HPMC               LONG(70) /* Cause processor to go into spin loop, and 
                                             * wait for wake up from Monarch Processor */
#define PDC_PAT_HPMC_RENDEZ_CPU     LONG(0) /* go into spin loop */
#define PDC_PAT_HPMC_SET_PARAMS     LONG(1) /* Allows OS to specify interrupt which PDC 
                                             * will use to interrupt OS during machine 
                                             * check rendezvous */

/* parameters for PDC_PAT_HPMC_SET_PARAMS: */
#define HPMC_SET_PARAMS_INTR 	    LONG(1) /* Rendezvous Interrupt */
#define HPMC_SET_PARAMS_WAKE 	    LONG(2) /* Wake up processor */

/*  PDC PAT IO */

#define PDC_PAT_IO                  LONG(71) /* On-line services for I/O modules */
#define PDC_PAT_IO_GET_NUM_HOT_PLUGS 	LONG(0) /* Get Number of Hot Plug I/O Slots */
#define PDC_PAT_IO_GET_LOC_HOT_PLUGS 	LONG(1) /* Get Physical Location of all
                                         * Hot Plug I/O Slots */
#define PDC_PAT_IO_POWER_OFF_HOT_PLUG 	LONG(2) /* Power off Hot Plug I/O Slot */
#define PDC_PAT_IO_POWER_ON_HOT_PLUG 	LONG(3) /* Power on Hot Plug I/O Slot */
#define PDC_PAT_IO_SET_HOT_PLUG_AI   	LONG(4) /* Set Hot Plug I/O Slot Attention 
                                       	  * Indicator */
#define PDC_PAT_IO_GET_SLOT_STATUS   	LONG(5) /* Get Slot Status Info*/
#define PDC_PAT_IO_GET_LOC_FROM_HARDWARE LONG(6) /* Get Physical Location from */
                                            /* Hardware Path */
#define PDC_PAT_IO_GET_HARDWARE_FROM_LOC LONG(7) /* Get Hardware Path from 
                                             * Physical Location */
#define PDC_PAT_IO_DISCONNECT_TGR    	LONG(8) /* Disconnect Transfer Gate Request */
#define PDC_PAT_IO_CONNECT_TGR      	LONG(9) /* Connect Transfer Gate Request */
#define PDC_PAT_IO_RESET_IO_BUS     	LONG(10) /* Reset I/O Bus */
#define PDC_PAT_IO_GET_PCI_CONFIG_FROM_HW LONG(11) /* Get PCI Configuration
                                               * Address from Hardware Path */
#define PDC_PAT_IO_GET_HW_FROM_PCI_CONFIG LONG(12) /* Get Hardware Path 
                                               * from PCI Configuration Address */
#define PDC_PAT_IO_READ_HOST_BRIDGE_INFO LONG(13)  /* Read Host Bridge State Info */
#define PDC_PAT_IO_CLEAR_HOST_BRIDGE_INFO LONG(14) /* Clear Host Bridge State Info*/
#define PDC_PAT_IO_GET_PCI_ROUTING_TABLE_SIZE LONG(15) /* Get PCI INT Routing Table 
                                                   * Size */
#define PDC_PAT_IO_GET_PCI_ROUTING_TABLE  LONG(16) /* Get PCI INT Routing Table */
#define PDC_PAT_IO_GET_HINT_TABLE_SIZE 	LONG(17) /* Get Hint Table Size */
#define PDC_PAT_IO_GET_HINT_TABLE   	LONG(18) /* Get Hint Table */
#define PDC_PAT_IO_PCI_CONFIG_READ  	LONG(19) /* PCI Config Read */
#define PDC_PAT_IO_PCI_CONFIG_WRITE 	LONG(20) /* PCI Config Write */
#define PDC_PAT_IO_GET_NUM_IO_SLOTS 	LONG(21) /* Get Number of I/O Chassis Slots in 
                                       		  * Cabinet */
#define PDC_PAT_IO_GET_LOC_IO_SLOTS 	LONG(22) /* Get Physical Location of I/O */
                                   		     /* Chassis Slots in Cabinet */
#define PDC_PAT_IO_POWER_OFF_SLOT   	LONG(23) /* Power off I/O Chassis Slot */
#define PDC_PAT_IO_POWER_ON_SLOT   	LONG(24) /* Power on  I/O Chassis Slot */
#define PDC_PAT_IO_OLA_IO_CHASSIS   	LONG(25) /* OLA I/O Chassis */
#define PDC_PAT_IO_OLD_IO_CHASSIS   	LONG(26) /* OLD I/O Chassis */
#define PDC_PAT_IO_SET_SLOT_AI      	LONG(27) /* Set I/O Chassis Slot Attention Indicator */
#define PDC_PAT_IO_CHASSIS_STATUS_INFO 	LONG(28) /* Get I/O Chassis Slot Status Info */
#define PDC_PAT_IO_GET_PROC_VIEW        LONG(29) /* Get Processor view of IO address */
#define PDC_PAT_IO_PROG_SBA_DIR_RANGE   LONG(30) /* Program directed range */
#define PDC_PAT_IO_SET_HARD_FAIL_ENABLE	LONG(31) /* Set HFE bit */
#define PDC_PAT_IO_CLR_HARD_FAIL_ENABLE	LONG(32) /* Clear HFE bit */
#define PDC_PAT_IO_GET_HARD_FAIL_ENABLE	LONG(33) /* Get HFE bit */
#define PDC_PAT_IO_SET_CHASSIS_PI	LONG(34) /* Set I/O Chassis Slot Power
						  * Indicator */
#define PDC_PAT_IO_SET_HOT_PLUG_PI   	LONG(35) /* Set Hot Plug I/O Slot
						  * Power Indicator */


/* PDC PAT MEM */

#define PDC_PAT_MEM             	LONG(72) /* Manage memory page deallocation */
#define PDC_PAT_MEM_PD_INFO     	LONG(0) /* Return PDT info for PD     */
#define PDC_PAT_MEM_PD_CLEAR    	LONG(1) /* Clear PDT for PD           */
#define PDC_PAT_MEM_PD_READ     	LONG(2) /* Read PDT entries for PD    */
#define PDC_PAT_MEM_PD_RESET    	LONG(3) /* Reset clear bit for PD     */
#define PDC_PAT_MEM_CELL_INFO   	LONG(5) /* Return PDT info For Cell   */
#define PDC_PAT_MEM_CELL_CLEAR  	LONG(6) /* Clear PDT For Cell         */
#define PDC_PAT_MEM_CELL_READ   	LONG(7) /* Read PDT entries For Cell  */
#define PDC_PAT_MEM_CELL_RESET  	LONG(8) /* Reset clear bit For Cell   */
#define PDC_PAT_MEM_SETGM	  	LONG(9) /* Set Golden Memory value    */
#define PDC_PAT_MEM_ADD_PAGE    	LONG(10) /* ADDs a page to the cell   */
#define PDC_PAT_MEM_ADDRESS     	LONG(11) /* Get Physical Location From*/
                                    		 /* Memory Address            */
#define PDC_PAT_MEM_GET_TXT_SIZE   	LONG(12) /* Get Formatted Text Size   */
#define PDC_PAT_MEM_GET_PD_TXT     	LONG(13) /* Get PD Formatted Text     */
#define PDC_PAT_MEM_GET_CELL_TXT   	LONG(14) /* Get Cell Formatted Text   */
#define PDC_PAT_MEM_RD_STATE_INFO  	LONG(15) /* Read Mem Module State Info*/
#define PDC_PAT_MEM_CLR_STATE_INFO 	LONG(16) /*Clear Mem Module State Info*/
#define PDC_PAT_MEM_CELL_DELETE_DIMM	LONG(17) /* Delete Cell DIMM */
#define PDC_PAT_MEM_CELL_ADD_DIMM	LONG(18) /* Add Cell DIMM */
#define PDC_PAT_MEM_GET_DIMM_FRUID	LONG(19) /* Get DIMM FRUID */
#define PDC_PAT_MEM_GET_DIMM_SLOT_LOC	LONG(20) /* Get PhyLoc from DIMM slot */
#define PDC_PAT_MEM_CLEAN_RANGE    	LONG(128) /*Clean Mem in specific range*/
#define PDC_PAT_MEM_GET_TBL_SIZE   	LONG(131) /* Get Memory Table Size    */
#define PDC_PAT_MEM_GET_TBL        	LONG(132) /* Get Memory Table         */

/* PDC PAT NVOLATILE */

#define PDC_PAT_NVOLATILE          	LONG(73) /* Access Non-Volatile Memory */
#define PDC_PAT_NVOLATILE_READ      	LONG(0) /* Read Non-Volatile Memory   */
#define PDC_PAT_NVOLATILE_WRITE     	LONG(1) /* Write Non-Volatile Memory  */
#define PDC_PAT_NVOLATILE_GET_SIZE  	LONG(2) /* Return size of NVM         */
#define PDC_PAT_NVOLATILE_VERIFY    	LONG(3) /* Verify contents of NVM     */
#define PDC_PAT_NVOLATILE_INIT      	LONG(4) /* Initialize NVM             */

/* PDC PAT PD */

#define PDC_PAT_PD                 	LONG(74) /* Multi Protection Domain support */
#define PDC_PAT_PD_GET_ADDR_MAP     	LONG(0) /* Get Address Map */
#define PDC_PAT_PD_GET_PDC_REVISIONS 	LONG(1) /* Get Address Map */
#define PDC_PAT_PD_CHANGE_ROOT_CELL 	LONG(2) /* Get Address Map */
#define PDC_PAT_PD_GET_COMPLEX      	LONG(3) /* Get    Dynamic COMPLEX_PROFILE */
#define PDC_PAT_PD_SET_COMPLEX      	LONG(4) /* Write  Dynamic COMPLEX_PROFILE */
#define PDC_PAT_PD_LOCK_COMPLEX     	LONG(5) /* Lock   Dynamic COMPLEX_PROFILE */
#define PDC_PAT_PD_UNLOCK_COMPLEX   	LONG(6) /* Unlock Dynamic COMPLEX_PROFILE */
#define PDC_PAT_PD_GET_REPLACEABLE_UNIT LONG(7) /* Get On Line Replaceable unit */
#define PDC_PAT_PD_ENABLE_ALIAS_CHECKS	LONG(8) /* Enable Alias Checking */
#define PDC_PAT_PD_DISABLE_ALIAS_CHECKS LONG(9) /* Disable Alias Checking */
#define PDC_PAT_PD_GET_PLATFORM_COUNTER LONG(10) /* Get Platform counter      */
#define PDC_PAT_PD_GET_ALIVE_CELLS	LONG(11) /* Get bitmask of PD cells */
#define PDC_PAT_PD_GET_REMOTE_PD_CONFIG LONG(128) /* Read Remote PD Interrupt 
                                              * Config */
#define PDC_PAT_PD_SET_REMOTE_PD_CONFIG LONG(129) /* Write Remote PD Interrupt
                                              * Config */

#define PDC_PAT_PD_TRY_REINTERLEAVE	LONG(140) /* Attempt Re-interleave */
#define PDC_PAT_PD_REINTERLEAVE		LONG(141) /* Re-interleave */
#define PDC_PAT_PD_GET_INTERLEAVE_ADDR	LONG(142) /* Get Interleave Address */

/* PDC_PAT_REGISTER_TOC */
#define PDC_PAT_REGISTER_TOC 		LONG(75)  /* TOC Registration */
#define PDC_PAT_REGISTER_TOC_SET	LONG(0)	  /* Register Vector */
#define PDC_PAT_REGISTER_TOC_GET	LONG(1)	  /* Get Vector */

/* PDC_PAT_SYSTEM_INFO */
#define PDC_PAT_SYSTEM_INFO		LONG(127) /* Return Config Info */
#define PDC_PAT_SYSTEM_INFO_GET_SIZE	LONG(0)   /* Get System Info Size */
#define PDC_PAT_SYSTEM_INFO_GET_INFO	LONG(1)   /* Formatted System Info */
#define PDC_PAT_SYSTEM_INFO_GET_ERRORS	LONG(2)	  /* Logged HW Errors */
#define PDC_PAT_SYSTEM_INFO_GET_WARNS	LONG(3)	  /* Boot Warnings */
#define PDC_PAT_SYSTEM_INFO_GET_REVS	LONG(4)	  /* HW Revisions */
#define PDC_PAT_SYSTEM_INFO_GET_REGS	LONG(5)   /* CPU and PDH Status */

/* PDC_PAT_UFO */
#define PDC_PAT_UFO			LONG(76)  /* Unix-Fail Over */
#define PDC_PAT_UFO_CREATE_PAIR		LONG(0)	  /* Create Switchover Pair */
#define PDC_PAT_UFO_DESTROY_PAIR	LONG(1)	  /* Destroy Switchover Pair */
#define PDC_PAT_UFO_JOIN_PAIR		LONG(2)	  /* Join Switchover Pair */
#define PDC_PAT_UFO_LEAVE_PAIR		LONG(3)	  /* Leave Switchover Pair */
#define PDC_PAT_UFO_GET_PAIR_INSTANCE	LONG(4)   /* Get Pair Instance Num */
#define PDC_PAT_UFO_REQUEST_MASTERSHIP	LONG(5)	  /* Request Mastership for PD*/
#define PDC_PAT_UFO_GET_CELL_ADDR	LONG(6)   /* Get Cell Address Info */


/******************  end PDC PAT procedure defines ******************/
/*
 * This call was developed for S700 workstations to allow the kernel to find
 * the I/O devices (Core I/O). In the future (Kittyhawk and beyond) this
 * call will be replaced (on workstations) by the architected PDC_SYSTEM_MAP
 * call.
 *
 * This call is supported by all existing S700 workstations (up to  Gecko).
 */
#define PDC_MEM_MAP	 	LONG(128) /* s700: returns page information */
#define PDC_MEM_MAP_HPA	 	LONG(0) /* module # --> hpa */

/*
 * Forte/Astro firmware implements an hversion specific option
 * to PDC_MEM that returns a table of memory ranges.  This is
 * to support discontiguous memory ranges that may extend beyond
 * 3.75 Gbytes.
 */
#define PDC_MEM_TABLE	LONG(128)

/*
 * This call was defined for Gecko workstations to give drivers access
 * to the EEPROM. Currently (release 10.0) only the IBM token ring driver
 * uses this call.
 */
#define PDC_EEPROM	 	LONG(129)
#define PDC_EEPROM_READ_WORD 	LONG(0)
#define PDC_EEPROM_WRITE_WORD 	LONG(1)
#define PDC_EEPROM_READ_BYTE 	LONG(2)
#define PDC_EEPROM_WRITE_BYTE 	LONG(3)

/* 
 * This call was developed for S700 workstations that support VME.
 * It is supported by Pace and Sidewinder.
 */
#define PDC_VME		 	LONG(131)

/*
 * PDC_SEED_ERROR provides an interface to force H/W errors programmatically.
 * The lack of comments next to the options is a result of insufficient
 * documentation in the HalfDome ERS.
 */
#define PDC_SEED_ERROR			LONG(132)
#define PDC_SEED_ERROR_MOVE_MEM		LONG(0)
#define PDC_SEED_ERROR_ICACHE		LONG(2)
#define	PDC_SEED_ERROR_FILL_MEM		LONG(3)
#define PDC_SEED_ERROR_FILL_MEM_LINE	LONG(4)
#define PDC_SEED_ERROR_SINGLE_BIT_ERR	LONG(5)
#define PDC_SEED_ERROR_DOUBLE_BIT_ERR	LONG(6)
#define PDC_SEED_ERROR_MEM_ADDR_ERR	LONG(7)
#define PDC_SEED_ERROR_IO_ADDR_ERR	LONG(8)
#define PDC_SEED_ERROR_DCACHE_ERR	LONG(14)
#define PDC_SEED_ERROR_FAIL_CELL	LONG(15)


/*
 * This PDC call was developed for Kittyhawk to walk the bus, logging errors
 * and reseting modules.  When this call returns, the bus should be placed in
 * a quiescent state for dumping memory to disk.
 */
#define PDC_IO                  LONG(135)     /* Safe broadcast reset.           */
#define PDC_IO_RESET_AND_CLEAR  LONG(0)       /* Perform a broadcast reset       */
#define PDC_IO_RESET	 	LONG(1)       /* Perform  broadcast reset on BC */
#define PDC_IO_RESET_DUMP_DEVS	LONG(2)	      /* Perform reset of dump devices	*/
/*
 * This PDC_IO option unfortunately overlaps with the previous one, so be certain
 * of the system you're on when using it.  Currently the option #2 listed above
 * only appears to be used on PAT based boxes and the one below is only used on
 * sprockets based workstations.
 */
#define PDC_IO_RESET_DEVICES	LONG(2)	      /* Disable USB hardware           */

#define PDC_IO_LOG_IO_ERRORS 	LONG(0) /* Opt 0, Arg 2 Parameter: Log I/O errors */
#define PDC_IO_NO_LOGS	 	LONG(1) /* Opt 0, Arg 2 Parameter: No logs...     */

/* Note that the following defines ar also used by the
 * PDC_CRASH_PREP procedure:
 */
/* Opt 2, Arg 2 (rAddr) element flag value for unmapped device */
#define PDC_DEV_NOT_MAPPED	LONG(-1)
/* Opt 2, Arg 3 (dump_device_ptr) element 0 magic value ("DUMP") */
#define PDC_DUMP_MAGIC		0x44554d50
/* Opt 2, Arg 3 (dump_device_ptr) elements following element 0 */
#define PDC_MAXDUMPDEVS		32


/*
 * This PDC call was developed to perform safe broadcast resets in machines
 * where a regular broadcast reset would not work because of harware problems.
 *
 * Currently (as of release 10.0), Gecko and Wright brothers support this call.
 * In the future, we expect Kittyhawk to also support it.
 */
#define PDC_BROADCAST_RESET 	LONG(136)     /* Safe broadcast reset.	*/
#define PDC_DO_RESET	 	LONG(0)       /* Perform a broadcast reset	*/
#define PDC_DO_FIRM_TEST_RESET  LONG(1)       /* Do broadcast reset with bitmap */
#define PDC_BR_RECONFIGURATION  LONG(2)       /* Do broadcast reset for reconfiguration */
#define PDC_FIRM_TEST_MAGIC LONG(0xab9ec36f)    /* For this reboot only      */

/*
 * This PDC call was developed to find the LAN station address in machines
 * that keep this information in stable store (instead of keeping it with
 * the LAN hardware).
 *
 * This call is currently (release 10.0) supported by Gecko and Wright
 * Brothers. We expect that Kittyhawk will also support this call.
 */
#define PDC_LAN_STATION_ID 	LONG(138) /* LAN sttion address		*/
#define PDC_LAN_RETURN_ID 	LONG(0) /* Retrun LAN station address	*/


/*
 * This PDC call was developed for I/O Configuration code to determine if
 * a BC or IOA has already been setup by firmware.  It was developed for
 * SkyHawk where PDC has to setup one of the IOAs -- so the OS must not 
 * modify the IO_IO_HIGH/IO_IO_LOW registers.  For BCs configured in this
 * way, information about modules under the BC is returned by PDC_SYSTEM_MAP.
 */
#define PDC_CHECK_BC_RANGES     LONG(139)     /* Check BC/IOA Configuration      */
#define PDC_RETURN_BC_RANGES    LONG(0)     /* Return IO_IO_HIGH and IO_IO_LOW */

/*
 * This PDC call was developed for workstations.  Its purpose is to provide
 * a simple flexible storage management system for Non Volatile memory.
 * This call is currently (release 10.0) supported by Anole/Jason/SuperPace.
 * Hopefully all future machines will support this call.
 */

#define PDC_NV_SECTIONS	 	LONG(141)

/* PDC_PERFORMANCE is added to support onyxe.c file */

#define PDC_PERFORMANCE		    LONG(142)
#define PDC_PERF_RETURN_COUNTERS    LONG(0)
#define PDC_PERF_RETURN_PERF_INFO   LONG(1)
#define PDC_PERF_CONFIGURE_COUNTERS LONG(2)
#define PDC_PERF_START_COUNTERS     LONG(3)
#define PDC_PERF_STOP_COUNTERS      LONG(4)

#define PDC_RDR                 LONG(144)
#define PDC_RDR_READ_BUFFER     LONG(0) /* return all rdrs */
#define PDC_RDR_READ            LONG(1) /* read a specified rdr */
#define PDC_RDR_WRITE           LONG(2) /* write a wpecified rdr */

#define PDC_INTRIGUE            LONG(145)
#define PDC_INTRIGUE_WRITE      LONG(0) /* write an intrigue image */
#define PDC_INTRIGUE_GET_SIZE   LONG(1) /* return mim size of scratch buffer */
#define PDC_INTRIGUE_START      LONG(2) /* start counters */
#define PDC_INTRIGUE_STOP       LONG(3) /* stop counters */
/*
 * The following definitions are for PDC HVERSION dependent calls.
 * OS support for these calls is done through hversion_pdc_call().
 *
 * When adding HVERSION dependent PDC call definitions to this file, please
 * add a comment before the definition indicating why the call was added
 * and what machines support it.
 */

#define PDC_ARG2_NOT_RADDR 	0
#define PDC_ARG2_IS_RADDR 	1

/*
 * This PDC call was developed for PCI platforms
 */
#define PDC_PCI          	LONG(147)
#define PDC_PCI_GET_IFACE_INFO  LONG(0)
#define PDC_PCI_GET_SLOT_INFO   LONG(1)
#define PDC_PCI_PATH_TO_HPA     LONG(9)
#define PDC_PCI_GET_INT_TBL_SIZE    LONG(13)
#define PDC_PCI_GET_INT_TBL         LONG(14)
/* PATH_TO_HPA is here for legacy code: Please use PDC_PCI_PATH_TO_HPA instead */
#define PATH_TO_HPA             LONG(9) /* Please use PDC_PCI_PATH_TO_HPA */

#define PDC_RELOCATE            LONG(149)
#define PDC_RELOCATE_INFO       LONG(0) /* Returns PDC relocation info */
#define PDC_RELOCATE_CHKSUM     LONG(1) /* performs PDC checksum */
#define PDC_RELOCATE_COPY       LONG(2) /* Relocates PDC in memory*/

/*
 * Added PDC functionality for diags
 */

#define PDC_PADDR_TO_BOARD      LONG(157)
#define PDC_OPT_PADDR_64        LONG(0)
#define PDC_OPT_PADDR_32        LONG(0)

/*
 * Added PDC functionality for SPP I/O drivers.
 */
#define PDC_GET_REG	 LONG(160)
#define PDC_GET_SIZE	 LONG(0)
#define PDC_REGPROPERTY	 LONG(1)
#define PDC_GET_COUNT	 LONG(2)
#define PDC_REG_PROP	 LONG(3)

#define PDC_STARTCPU	 LONG(161)

#define PDC_OBP_BOOTARGS        	LONG(162)

#define PDC_MEMCOPY                     LONG(153)
#define PDC_MEMCOPY_DEFAULT             LONG(0)

#define PDC_OPT_OBP_BOOTARGS_GET_STRING LONG(0)
#define PDC_OPT_OBP_BOOTARGS_GET_LEN    LONG(1)


#define PDC_INITIATOR    LONG(163)
#define GET_INITIATOR    LONG(0)
#define SET_INITIATOR    LONG(1)
#define DELETE_INITIATOR LONG(2)

#define PDC_SPP_INFO	 	LONG(164)
#define PDC_SPP_INFO_PROPLEN 	LONG(0)
#define PDC_SPP_INFO_PROP 	LONG(1)
#define PDC_OPT_SPP_PEER        LONG(2)
#define PDC_OPT_SPP_CHILD       LONG(3)

#define PDC_VIRTUAL_ENV         LONG(167)

#ifndef LOCORE
/*
 * The following are structures returned by PDC for some of the calls
 * described above. Since the structures below can only be used be C
 * they will only be included if LOCORE is not defined.
 */

#  include <machine/cpu.h>	/* kernel src: <machine/sys/cpu.h> */	/* for struct mod_type, et. al */
#  include <sys/time.h>		/* for struct timeval */

struct pdc_pci_iface_info {
	u_long frequency;   /* in KHz */
	u_long bus_width;   /* 32 or 64 */
	u_long num_slots;
	u_long num_built_ins;
	u_long opt_reg_map[6];
};

struct pdc_pci_slot_info {
	u_int slot_number;
	u_int path_len;
	u_int *dev_num;
};

struct pdc_chassis_rtn_block {	/* PDC_CHASSIS */
	u_long warning;
	u_long filler[31];
};

struct pdc_pim_rtn_block {	/* PDC_PIM/PDC_PIM_SIZE */
	u_long size;
	u_long archsize;
	u_long filler[30];
};

struct pdc_pim_data_block {	/* PDC_PIM */
	int data[512];
};

struct pdc_model_rtn_block {	/* PDC_MODEL */
	u_long hversion;
	u_long sversion;
	u_long hw_id;
	u_long boot_id;
	u_long sw_id;
	u_long sw_cap;
	u_long arch_rev;
	u_long potential_key;
	u_long current_key;
	u_long width;
	u_long filler[22];
};

struct cpu_id {	/* PDC_MODEL_CPU_ID */
#ifdef __LP64__
	int	filler;
#endif /* __LP64__ */
	u_int   reserved:21;
	u_int   cpu_id:6;
	u_int   version:5;
#ifdef __LP64__
	long	width;
#endif /* __LP64__ */
};

union pdc_model_cpu_id_rtn_block {	/* PDC_MODEL_CPU_ID */
	long		alignment;
	struct cpu_id	rtn_block;
};

#define	PDC_MODEL_CPU_KEY_QUAD_STORE	0x1
#define	PDC_MODEL_CPU_KEY_WORD_TO_IO	0x2
#define	PDC_MODEL_CPU_KEY_RECIP_SQRT	0x4
#define	PDC_MODEL_CPU_KEY_FDC_GRAPHICS	0x8


struct pdc_sys_model_rtn_block {/* PDC_MODEL/PDC_MODEL_SYS_MODEL */
	u_long   mod_len;
	u_long   filler[31];
};

/* Added for SpeedyBoot/firm_ctl funtionality see pdc/firm_ctl* */
struct pdc_firm_test_get_rtn_block {	/* PDC_MODEL/PDC_FIRM_TEST_GET */
	u_long   current_tests; /* u_R_addr Raddr_ints[0]  */
	u_long   tests_supported; /* u_R_addr Raddr_ints[1]  */
	u_long   default_tests; /* u_R_addr Raddr_ints[2]  */
	u_long   filler[29];
};

/* Bit masks for PDC_MODEL/PDC_MODEL_CAPABILITIES */
#define PDC_MODEL_CAPABILITIES_OS32	0x2
#define PDC_MODEL_CAPABILITIES_OS64	0x1

/* Bit masks/Macros for PDC_MODEL/PDC_MODE_CAPABILITIES PAOS-20 */
#define PDC_MODEL_CAPABILITIES_NVA_MASK		0x30
#define PDC_MODEL_CAPABILITIES_NVA_EQUIV_ONLY	0x30
#define SUPPORTS_NON_EQUIV_ALIASING(a) \
( ((a) & PDC_MODEL_CAPABILITIES_NVA_MASK) != PDC_MODEL_CAPABILITIES_NVA_EQUIV_ONLY )

struct cache_config {		/* PDC_CACHE */
#ifdef __LP64__
	int	filler;
#endif /* __LP64__ */
	u_int   reserved0:4;
	u_int   lines_per_chunk:4;
	u_int   blocksize:3;
	u_int   shift:2;
	u_int   wt:1;
	u_int   f_sel:2;
	u_int   cst:3;
	u_int   reserved2:11;
	u_int   hv_stuff:2;
};
#define _CACHECONFIG_T


struct dtlb_config {		/* PDC_CACHE */
#ifdef __LP64__
	int	filler;
#endif /* __LP64__ */
	u_int	reserved0  :12;
	u_int	p_sel      :2;
	u_int	HV_field0  :1;
	u_int	page_size  :1;
	u_int	cst        :3;
	u_int   reserved1  :11;
	u_int	HV_field1  :2;
};


struct itlb_config {		/* PDC_CACHE */
#ifdef __LP64__
	int	filler;
#endif /* __LP64__ */
	u_int	reserved0  :12;
	u_int	p_sel      :2;
	u_int	HV_field0  :1;
	u_int	page_size  :1;
	u_int	cst        :3;
	u_int	aid	   :5;
	u_int	sr	   :6;
	u_int	HV_field1  :2;
};


struct pdc_cache_rtn_block {	/* PDC_CACHE */
	u_long	 	    ic_size;
	struct cache_config ic_conf;
	char 		   *ic_start_addr;
	u_long		    ic_stride;
	u_long		    ic_step;
	u_long		    ic_loop;
	u_long	    	    dc_size;
	struct cache_config dc_conf;
	char		   *dc_start_addr;
	u_long   	    dc_stride;
	u_long   	    dc_step;
	u_long   	    dc_loop;
	u_long	    	    it_size;
	struct itlb_config  it_conf;
	u_long 		    it_sp_addr;
	u_long	   	    it_sp_stride;
	u_long   	    it_sp_count;
	char		   *it_offset;
	u_long   	    it_stride;
	u_long   	    it_step;
	u_long   	    it_loop;
	u_long	    	    dt_size;
	struct dtlb_config  dt_conf;
	u_long 		    dt_sp_addr;
	u_long   	    dt_sp_stride;
	u_long   	    dt_sp_count;
	char		   *dt_offset;
	u_long   	    dt_stride;
	u_long   	    dt_step;
	u_long   	    dt_loop;
	u_long   	    filler[2];
};

struct pdc_spaceid_bits_rtn_block {	/* PDC_CACHE - SPACEID_BITS */
#ifdef __LP64__
	int	upperfiller;
#endif /* __LP64__ */
	u_int	reserved0  :4;
	u_int	bits_used  :12;
	u_int	reserved1  :16;
	u_long    	    filler[31];
};

struct pdc_hpa_rtn_block {	/* PDC_HPA */
	u_long		    hpa;
	u_long              cpu_num;
	u_long    	    filler[30];
};

struct pdc_coproc_rtn_block {	/* PDC_COPROC */
	u_long		ccr;
	u_long		present;
	u_long		filler[30];
};

struct pdc_iodc_rtn_block {	/* PDC_IODC */
	u_long		size;
        long		filler1[15];
        u_long		hver_resv1;
	u_long		filler2[15];
};

struct iodc_rtn_block {		/* PDC_IODC/READ */
	struct mod_type type;
	struct swrev    sw_rev;
	u_int		uid;
	u_int		undefined;
};

struct pdc_mem_rtn_block {	/* PDC_IODC/READ */
	u_long		status;
	u_long		max_spa;
	u_long		max_mem;
	u_long		filler[29];

};

struct pdc_iodc_id_chief_block {	/* PDC_IODC_ID_CHIEFTAIN */
	struct io_module *chieftain;
	long   filler[31];
};

struct pdc_relocate_info_block {        /* PDC_RELOCATE_INFO */
        u_long   pdc_size;
        u_long   pdc_alignment;
        u_long   pdc_address;
        u_long   filler[29];
};

/* The following can be used to index into pdc_iodc_clr_rtn_block.filler[]
   for Bushmaster (9000/710).  RCME ==> return and clear memory errors
   Note that the offsets are four less than the offset in to the 128 byte
   buffer returned from IODC
*/
#define BUSH_RCME_DATA0    12	/* No data is available, so this will always */
				/* equal zero. */
#define BUSH_RCME_DATA1    13	/* No data is available, so this will always */
				/* equal zero. */
#define BUSH_RCME_KSR      14 	/* Value of the K9 MEMORY_STATUS register. */
#define BUSH_RCME_FLAG     15 	/* The contents of the Page Zero SEDC flag. */
#define BUSH_RCME_STATUS_A 16 	/* Value of the ECC_STATUS register for bank */
				/* A. */
#define BUSH_RCME_ADDR_A   17  	/* The address assembled from ECC_ERR     */
				/* registers 1-3.  (This address only     */
				/* isolates to a cache line, or 8 words.) */
				/* Valid only if status_A has the 'E' bit */
				/* set, else zero.                        */
#define BUSH_RCME_SYND_A   18  	/* The contents of the syndrome byte in   */
				/* ECC_ERR1 register for bankA.           */
#define BUSH_RCME_STATUS_B 19 	/* Value of the ECC_STATUS register for   */
				/* bank B.                                */

#define BUSH_RCME_ADDR_B   20  	/* The address assembled from ECC_ERR         */
				/* registers 1-3. (This address only isolates */
				/* to a cache line, or 8 words.) Valid only   */
				/* if status_A has the 'E' bit set, else zero.*/

#define BUSH_RCME_SYND_B   21  	/* The contents of the syndrome byte in  */
				/* ECC_ERR1 register for bankB.          */

struct pdc_iodc_clr_rtn_block {	/* PDC_IODC/PDC_IODC_MEM_CLEAR */
	u_long		stat;
	u_long		sadd;
	u_long		info;
	u_long		madd;
	u_long		filler[28];
};

/*
 * Decoupled the use of the timeval struct because the format of the
 * return value from PDC_TOD is changing independent of timeval. Users
 * of this structure must now access the fields independently when copying
 * them to a timeval struct.
 */

struct pdc_tod_time_rtn_block {	/* PDC_TOD */
	u_long		tod_sec;
	u_long		tod_usec;
	u_long		filler[30];
};

struct pdc_tod_calib_rtn_block {	/* PDC_TOD */
	double	it_freq;
	u_long	todr_acc;
	u_long	it_acc;
	u_long	filler[28];
};

struct pdc_scsi_parms_get_rtn_block {
	u_long	scsi_id;
	u_long	xfer;
	u_long	bwidth;
	u_long	autoterm;
	u_long	physloc;
	u_long	filler[27];
};

struct pdc_scsi_parms_set_rtn_block {
	u_long	physloc;
	u_long	filler[31];
};

struct pdc_ss_size_rtn_block {	/* PDC_STABLE */
	u_long count;
	u_long filler[31];
};

struct pdc_nvm_size_rtn_block {	/* PDC_NVOLATIVE */
	u_long count;
	u_long filler[31];
};

struct pdc_memory_map_rtn_block { 	/* PDC_MEMORY_MAP */
	u_int	hpa;			/* mod's register set address 	*/
	u_int	more_pgs;		/* number of additional I/O pgs */
};

struct pdc_btlb_parms_rtn_block { /* PDC_BLOCK_TLB/RTN_PARMS */
	u_long min_size;
	u_long max_size;
	u_long fixed_range_info;
	u_long variable_range_info;
	u_long filler[28];
};

struct pdc_hwtlb_parms_rtn_block { /* PDC_TLB/RTN_PARMS */
	u_long min_size;
	u_long max_size;
	u_long filler[30];
};

struct pdc_hwtlb_state_rtn_block { /* PDC_TLB/RTN_PARMS */
	u_long state;
	u_long filler[31];
};

struct pdc_mem_table_rtn_block { /* PDC_MEM/TABLE */
	u_long nbr_returned; /* number of entries returned */
	u_long nbr_entries;  /* total number of entries in table */
};

struct pdc_mem_info_rtn_block {	/* PDC_MEM/RETURN_INFO*/
        u_long max_pdt_entries;	/* max number of entries in PDT	*/
        u_long nbr_entries;	/* number of entries in PDT	*/
#ifdef __LP64__
	u_int  reserved0;       /* reserved field for 64-bit kernel */
#endif /* __LP64__ */
        u_int  reserve: 30,	/* reserved			*/
		   cs:  1,	/* clear bit			*/
		   is:  1;	/* interleave status		*/
	u_long lowest_dbe;	/* lowest location of DBE	*/
	u_long golden_mem;	/* size of the Golden Memroy	*/
	u_long filler[27];
};

struct pdc_mem_read_rtn_block {	/* PDC_MEM/READ */
        u_long nbr_entries;	/* number of entries in PDT	*/
	u_long filler[31];
};

struct pdc_mem_setgm_rtn_block {	/* PDC_MEM/SETGM */
	u_long filler[32];
};


struct pdc_sys_map_find_mod_rtn_block { /* PDC_SYTEM_MAP/FIND_MODULE */
	u_long mod_addr;
	u_long mod_pgs;
	u_long add_addrs;
	u_long filler[29];
};

struct pdc_sys_map_find_addr_rtn_block { /* PDC_SYTEM_MAP/FIND_ADDRESS */
	u_long mod_addr;
	u_long mod_pgs;
	u_long filler[30];
};

struct pdc_sys_map_trans_path_rtn_block { /* PDC_SYTEM_MAP/TRANSLATE_PATH */
	u_long mod_addr;
	u_long mod_pgs;
	u_long add_addrs;
	u_long mod_index;
	u_long filler[28];
};

#define PDC_PAT_PHYS_LOC_L1(p)          (((p) >> 56) & 0x0ff)
#define PDC_PAT_PHYS_LOC_L2(p)          (((p) >> 48) & 0x0ff)
#define PDC_PAT_PHYS_LOC_L3(p)          (((p) >> 40) & 0x0ff)
#define PDC_PAT_PHYS_LOC_L4(p)          (((p) >> 32) & 0x0ff)
#define PDC_PAT_PHYS_LOC_L5(p)          (((p) >> 24) & 0x0ff)
#define PDC_PAT_PHYS_LOC_L6(p)          (((p) >> 16) & 0x0ff)
#define PDC_PAT_PHYS_LOC_L7(p)          (((p) >> 8)  & 0x0ff)
#define PDC_PAT_PHYS_LOC_SOURCE(p)      (((p) >> 4)  & 0x00f)
#define PDC_PAT_PHYS_LOC_DETAIL(p)      (((p) >> 0)  & 0x00f)

/* Generic PDC PAT physical location descriptor */
typedef ulong_t pdc_pat_phys_loc_t;

/***********************************************************************
 * Memaddr structures for PDC_CRASH_PREP
 **********************************************************************/
#  ifndef _MACHINE_IODC_INCLUDED
#    ifdef _KERNEL
#      include <machine/pdc/iodc.h>	/* For struct device_path */
#    else
#      include <machine/iodc.h>	/* For struct device_path */
#    endif
#  endif
typedef union dump_magic {
    uint32_t	number;
    char	string[4];
} dump_magic_t;

typedef struct pdc_crash_prep_devs {
    dump_magic_t        magic;
    uint32_t		count;
    uint64_t		reserved[3];
    struct device_path	dp[32];
} pdc_crash_prep_devs_t;

/********************************************************************
* Return block structures for PDC_MEM_TABLE
********************************************************************/

/* a.k.a. MemAddrTable_t in Sprockets ERS */
typedef struct pa_extended_memory_table {
    uint64_t phys_addr;
    uint32_t pages;
    uint32_t reserved;
} pa_extended_memory_table_t;

/********************************************************************
* Return block structures for PDC_PAT_MEM
********************************************************************/

struct cell_pdt_info_rtn
{
        u_int       reserved     :32;    /* the time that PDT was last cleared*/
        u_int       cs           :1;     /* "cleared status" PDT has been */
					 /* cleared*/
        u_int       page_entries :15;    /* the current number of entries */
					 /* in the  PDT */
        u_int       ic           :1;     /* "Interleaving changed"           */
        u_int       max_pdt_entries :15; /* Max number of entries for the cell*/
};

struct pat_mem_cell_info_rtn_block   /* PDC_PAT_MEM/PDC_PAT_MEM_CELL_INFO */
{
        struct       cell_pdt_info_rtn cell_info_pdt;  
        ulong_t      cell_good_mem;   /* returns the target value used by     */
                                      /* memory initialization to provide     */
                                      /* initial mem without double bit error */
        ulong_t      dbe_loc;         /* returns the location in memory where */
                                      /* the first double bit error detected  */
        ulong_t      clear_time;      /* the time that PDT was last cleared   */
#ifdef __LP64__			      /* We have to be careful with filler.   */
        ulong_t      filler[28];      /* Because cell_pdt_info_rtn is not a   */
#else				      /* long, it will be different size      */
	ulong_t	     filler[27];      /* on 32-bit and 64-bit kernels.  To    */
#endif				      /* compensate, the filler is adjusted   */

};

struct pat_mem_read_rtn_block   /* PDC_PAT_MEM/PDC_PAT_MEM_(CELL/PD)_READ */
{
        ulong_t      pat_nbr_entries; /* specifies the current total  */
                     /* number of entries in all of the cells in the PD. */
        ulong_t      filler[31];
};

struct pdt_mem_return_read          /* PDC_PAT_MEM_(CELL/PD)_READ(mem_addr) */
{     /* returns information in a memory area provided by the caller  */
        uint64_t   phys_addr :58;       /* the cache line where the   */
                                        /* memory test                */
        u_int      reserved  :4;
        u_int      perm_bit  :1;        /* Verify error by subsequent */
        u_int      et_bit    :1;        /* error type */
};

typedef struct pd_pdt_info {
	u_int	   reserved       : 31; /* Reserved */
	u_int      ke             :  1; /* Interleaving changed because */
					/* of an error in Known Good Memory */
	u_int      pdt_entries    : 16;	/* Number of PDT entries for all */
					/* active cells in PD */
	u_int      max_pdt_entries: 16;	/* Maximum number of PDT entries */
					/* in PD */
} pd_pdt_info_t;

/* PDC_PAT_MEM(Option 0) */
typedef struct pdc_pat_mem_pd_pdt_rtn_block {
	pd_pdt_info_t pd_pdt_info;	/* General information */
	ulong_t	   cs_bitmap;		/* Cleared status bitmap */
	ulong_t    ic_bitmap;		/* Interleaving changed bitmap */
	ulong_t    pd_good_mem;		/* Error-free memory for OS load */
	ulong_t	   pd_first_dbe;	/* Location of first double-bit error */
	ulong_t    clear_time;		/* Time the PDT was last cleared */
#if __LP64__				/* Adjust filler to compensate for */
	ulong_t	   filler[26];		/* the fact that pd_pdt_info is */
#else					/* different sizes in 32-bit vs */
	ulong_t	   filler[25];		/* 64-bit. */
#endif
} pdc_pat_mem_pd_pdt_rtn_block_t;

 /* PDC_PAT_MEM/ PDC_PAT_MEM_RD_STATE_INFO */
typedef struct pdc_pat_mem_read_state_rtn_block {
        ulong_t	   actual_len;
        ulong_t    reserved[31];
} pdc_pat_mem_read_state_rtn_block_t;

/* PDC_PAT_MEM(Option 11) */
typedef struct pdc_pat_mem_get_phys_rtn_block {
	pdc_pat_phys_loc_t   phys_loc[8];	/* Memory physical location */
	ulong_t    filler[24];
} pdc_pat_mem_get_phys_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_CELL
********************************************************************/
/* */
typedef struct pat_cpu_info {
	u_int	    clock_domain  : 8;	/* Clock domain of cell */
	u_int	    reserved      : 4;	/* Reserved */
	u_int	    maxcpus       : 4;	/* Max installable cpus in cell */
	u_int	    iodc_hversion :16;	/* Proc IODC Hversion */
	u_int	    cpu_present   :16;	/* Bitmask of cpus present */
	u_int	    cpu_deconfig  :16;	/* Bitmask of f/w deconfigured cpus */
} pdc_pat_cpu_info_t;

/* PDC_PAT_CELL/PDC_PAT_CELL_NUM (Option 0) */
struct pat_cell_num_rtn_block {    
        ulong_t      cell_number;       /* Cell number call was made */
        ulong_t      cell_location;     /* Physical location of the cell */
        ulong_t      reserved0;         /* reserved field for 64-bit kernel */
        ulong_t      reserved1;         /* reserved field for 64-bit kernel */
        ulong_t      filler[28];
};

/* PDC_PAT_CELL/PDC_PAT_CELL_GET_INFO (Option 1) */
typedef struct pdc_pat_cell_info_rtn_block {
	ulong_t	     count;		/* Bytes written to memaddr */
	ulong_t	     filler[31];
} pdc_pat_cell_info_rtn_block_t;

typedef struct pdc_pat_cell_range {
        ulong_t type;             /* address range type (LMMIO, GMMIO, ...) */
        ulong_t start;            /* start of address range */
        ulong_t finish;           /* end of address range */
} pdc_pat_cell_range_t;

typedef struct pdc_pat_cell_ca {
        ulong_t reserved[508];
} pdc_pat_cell_ca_t;

typedef struct pdc_pat_cell_proc {
        ulong_t id_eid;           /* processor LID register */
        ulong_t reserved[507];
} pdc_pat_cell_proc_t;

typedef struct pdc_pat_cell_mc {
        ulong_t amount;           /* amount of contiguous memory */
	ulong_t min_gni_base;     /* min GNI base address */
	ulong_t min_gni_len;      /* min GNI length */
        ulong_t reserved[505];
} pdc_pat_cell_mc_t;

#define PAT_MAX_BA_RANGES 168

typedef struct pdc_pat_cell_ba {
        ulong_t reserved1;
        ulong_t range_num;                            /* num of addr ranges */
        pdc_pat_cell_range_t addr[PAT_MAX_BA_RANGES]; /* array of addr ranges */
        ulong_t reserved2[2];
} pdc_pat_cell_ba_t;

typedef union pdc_pat_cell_mod {
        pdc_pat_cell_proc_t proc;     /* processor */
        pdc_pat_cell_mc_t   mem;      /* memory controller */
        pdc_pat_cell_ca_t   ca;       /* central agent */
        pdc_pat_cell_ba_t   ba;       /* local or system bus adapter */
} pdc_pat_cell_mod_t;

typedef struct pat_cell_mod_info {
	u_int	    entity    : 8;	/* Module type */
	u_int	    dvi       : 8;	/* Direct virtual index bit */
	u_int       ioc       : 8;	/* Which functions are IOCs? */
	u_int       reserved0 : 8;	/* Reserved */
	u_int       reserved1 : 8;	/* Reserved */
	u_int	    mod_pages :24;	/* Number of 4K pages occupied by mod */
} pdc_pat_cell_mod_info_t;

typedef struct pdc_pat_cell_mod_maddr_block {
        ulong_t cba;                /* function 0 configuration space address */
        ulong_t mod_info;	    /* module information */
        ulong_t mod_location;	    /* physical location of the module */
        ulong_t mod_path;           /* module path (device path - layers) */
        pdc_pat_cell_mod_t mod;     /* PAT cell module components */
} pdc_pat_cell_mod_maddr_block_t;

/* PDC_PAT_CELL(Option 2) */
typedef struct pdc_pat_cell_mod_rtn_block {
        ulong_t count;              /* # bytes written to memaddr */
        ulong_t reserved[31];
} pdc_pat_cell_mod_rtn_block_t;

/* PDC_PAT_CELL/PDC_PAT_CELL_LOC_TO_NUMBER (Option 8) */
struct pdc_pat_cell_loc2num_rtn_block {
	ulong_t     cell_number;	/* Complex-unique cell number */
	ulong_t     filler[31];
} pdc_pat_cell_loc2num_rtn_block_t;

/* PDC_PAT_CELL/PDC_PAT_CELL_NUMBER_TO_LOC (Option 10) */
struct pdc_pat_cell_num2loc_rtn_block {    
        pdc_pat_phys_loc_t cell_location;/* Physical location of the cell */
        ulong_t      filler[31];
} pdc_pat_cell_num2loc_rtn_block_t;

/* PDC_PAT_CELL(Option 11) */
typedef struct pdc_pat_cell_walk_fabric_rtn_block {
	ulong_t	     unreachable_mask;	/* Bitmask of unreachable cells */
	ulong_t	     filler[31];
} pdc_pat_cell_walk_fabric_rtn_block;

/* PDC_PAT_CELL(Option 12) */
typedef struct pdc_pat_cell_route_size_rtn_block {
	ulong_t	     cc2xbc;
	ulong_t	     xbc2xbc;
	ulong_t	     xbc2rc;
	ulong_t	     rc2rc;
	ulong_t	     size;
	ulong_t	     filler[27];
} pdc_pat_cell_route_size_rtn_block_t;

/* PDC_PAT_CELL(Option 13) */
typedef struct pdc_pat_cell_route_table_rtn_block {
	ulong_t	     cc2xbc;
	ulong_t	     xbc2xbc;
	ulong_t	     xbc2rc;
	ulong_t	     rc2rc;
	ulong_t	     filler[28];
} pdc_pat_cell_route_table_rtn_block_t;

/* PDC_PAT_CELL(Option 14) */
typedef struct pdc_pat_cell_get_pdh_size_rtn_block {
	ulong_t	     buffer_size;
	ulong_t	     filler[31];
} pdc_pat_cell_get_pdh_size_rtn_block_t;

/* PDC_PAT_CELL(Option 16) */
typedef struct pdc_pat_cell_read_pdh_rtn_block {
	ulong_t	     count;
	ulong_t	     filler[31];
} pdc_pat_cell_read_pdh_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_COMPLEX
********************************************************************/

typedef struct pdc_capabilities_bitmask {
   u_int reserved;
   u_int reserved2       :23;
   u_int io_olar_capable :1;	/* 1 == platform is i/o OLAR capable */
   u_int unsync_tlb      :1;	/* 1 == OS does not have to sync after purge */
   u_int pdc_hpmc_rendez :1;	/* 0 == PDC_PAT_HPMC et al unsupported */
   u_int iodc_64_bit     :1;	/* 0 == 64-bit IODC not supported */
   u_int iodc_32_bit     :1;	/* 0 == 32-bit IODC not supported */
   u_int unix_fail_over  :1;	/* 0 == Mariposa not supported */
   u_int non_blocking    :1;	/* 0 == non-blocking calls not supported */
   u_int polling_calls   :1;	/* 0 == polling calls not supported */
   u_int serialize_calls :1;	/* 0 == caller must serialize PDC calls */
} pdc_capabilities_bitmask_t;

typedef union pdc_capabilities {
   ulong_t word3;
   pdc_capabilities_bitmask_t word3_bitmask;
} pdc_capabilities_t;

#define PROFILE_NCELL		64
#define PROFILE_NPD		64
#define PROFILE_NGIRANGE	64
#define PROFILE_NTOGO		16
#define PROFILEA_PDMASK		0x3f

/* Group A.  Stable complex profile. */
typedef struct pdc_pat_complex_profile_a {
    unsigned int	checksum;   		/* Byte checksum.  LE */
    unsigned int	sequence;   		/* Sequence ID.  LE */
    unsigned short	major_rev;		/* PDC major revision.  LE */
    unsigned short	minor_rev;		/* PDC minor revision.  LE */
    unsigned char	cell_width;		/* # address bits for routing*/
    unsigned char	cell_clic;		/* Cache-line interleave */
    char		reserved_0[2];		/* Reserved */
    char		model_number[4];	/* Model number */
    char		model_string[16];	/* Model string */
    char		serial_number[10];	/* Complex serial number */
    char 		system_name[20];	/* Complex system name */
    char		original_product_number[6];/* Original product number */
    char		current_product_number[6];/* Current product number */
    char		reserved_1[2];		/* Reserved */
    unsigned char	cell_assignments[PROFILE_NCELL];
    unsigned char	vcell_assignments[PROFILE_NCELL];
    unsigned char	mariposa_sibling_cell_assignment[PROFILE_NCELL];
    unsigned int	mariposa_id[PROFILE_NPD];/* little endian */
    unsigned short	gni_cell[PROFILE_NCELL]; /* little endian */
    char		os_data[64];
    ulong_t		gi_range[PROFILE_NGIRANGE]; /* little endian */
    unsigned char	gi_platform[64];
    unsigned short	min_zi_pd[PROFILE_NPD]; /* little endian */
    unsigned short	max_zi_pd[PROFILE_NPD]; /* little endian */
    unsigned char	pd_high_availability[PROFILE_NPD];
    unsigned int	software_capabilities[PROFILE_NPD]; /* little endian */
    unsigned char	complex_routing[PROFILE_NTOGO][PROFILE_NCELL / 2];
    char		diag_license[4];
    unsigned int	sub_id;
    char		enterprise_id[8];
    unsigned char	pds_supported;
    char		reserved3[7];
    ulong_t		reserved4[83];      	/* 664 / 8 */
} pdc_pat_complex_profile_a_t;

#define CELL_ASSIGNMENTS_CB(X)		((X) >> 6)
#define CELL_ASSIGNMENTS_2CB(X,Y)	((X) = ((X) & 0x3f) | ((Y) << 6))
#define CELL_ASSIGNMENTS_CELL(X)	((X) & 0x3f)
#define CELL_ASSIGNMENTS_2CELL(X,Y)	((X) = ((X) & 0xc0) | ((Y) & 0x3f))
#define CELL_ASSIGNMENTS_BASE		0
#define CELL_ASSIGNMENTS_FLOATING	1
#define CELL_ASSIGNMENTS_FREE		2

/* Group B.  Dynamic complex profile. */
typedef struct pdc_pat_complex_profile_b {
    unsigned int    checksum;			/* Checksum of all bytes */
    unsigned int    sequence;			/* Sequence number */
    char	    os_data[4088];		/* Opaque to PDC */
} pdc_pat_complex_profile_b_t;

typedef struct pdc_complex_path {
    unsigned char   flags;
    unsigned char   bc[6];
    unsigned char   mod;
    unsigned int    layer[6];
} pdc_complex_path_t;

/* Group C.  PD profile. */
typedef struct pdc_pat_complex_profile_c {
    unsigned int	checksum;		/* Checksum of all bytes */
    unsigned int	sequence;		/* Sequence id */
    pdc_complex_path_t	pri_boot_path;		/* Primary boot path */
    char		reserved_0[8];		/* Reserved */
    ulong_t		cell_configuration;	/* Bitmask of cells */
						/* participating in PD */
    unsigned char	root_cell_selection[4];	/* PD root cell */
    unsigned char	pd_number;		/* Current PD number */
    char	    	flags;			/* Misc flags */
    char		kgm;			/* Known good memory */
    char		boot_timer;		/* # secs for boot device */
						/* ready */
    char		reserved_1[8];		/* Reserved */
    short		os_id;
    char		os_dependent[22];
    char		reserved_2[6];
    unsigned short	icod_processors;
    pdc_complex_path_t	console_path;		/* Console/display path */
    pdc_complex_path_t	alt_boot_path;		/* Alternate boot path */
    pdc_complex_path_t	keyboard_path;		/* Keyboard path */
    pdc_complex_path_t	ha_alt_boot_path;	/* HA alternate boot path */
    char		pd_name[64];		/* PD Name */
    char		cell_integration[64];	/* Cell configuration */
    char		ip_addr[16];
    char		reserved_3[8];
    char		reserved_4[128];
} pdc_pat_complex_profile_c_t;

/* Cell information */
typedef struct pdc_pat_cell_info {
    unsigned char	reserved1[3];
    unsigned char	layout_revision;
    unsigned char	reserved2[2];
    unsigned char	cell_state;
    unsigned int	reserved3   : 3;
    unsigned int	cell_status : 5;
} pdc_pat_cell_info_t;

/* Cell slot Generic configuration. */
typedef struct pdc_pat_slot_generic {
    ulong_t		info_0;
    ulong_t		info_1;
} pdc_pat_slot_generic_t;

/* Cell slot PCI configuration. */
typedef struct pdc_pat_slot_pci_0 {
    u_int		status    : 16;
    u_int		command   : 16;
    u_int		device_id : 16;
    u_int		vendor_id : 16;
} pdc_pat_slot_pci_0_t;

/* Cell slot PCI configuration. */
typedef struct pdc_pat_slot_pci_1 {
    u_int		bist            : 8;
    u_int		hdr_type        : 8;
    u_int		latency_time    : 8;
    u_int		cache_line_size : 8;
    u_int		class_code	:24;
    u_int		revision_id     : 8;
} pdc_pat_slot_pci_1_t;

typedef struct pdc_pat_slot_pci {
    pdc_pat_slot_pci_0_t slot_info_0;
    pdc_pat_slot_pci_1_t slot_info_1;
} pdc_pat_slot_pci_t;

typedef union pdc_pat_slot_union {
    pdc_pat_slot_generic_t gen;
    pdc_pat_slot_pci_t	pci;
} pdc_pat_slot_u_t;

/* Cell I/O slot information. */
typedef struct pdc_pat_slot_info {
    ulong_t		slot_header;
    pdc_pat_slot_u_t	slot_info;
    pdc_pat_phys_loc_t	slot_phys_loc;
    ulong_t		slot_path;
} pdc_pat_slot_info_t;

/* Cell DIMM information.  */
typedef struct pdc_pat_dimm_info {
    u_int		type:8;
    u_int		spd_error:1;
    u_int		ex_spd_xsum_error:1;
    u_int		reserved0:2;
    u_int		dimm_type_mismatch:1;
    u_int		unsupported_dimm:1;
    u_int		dimm_configuration:2;
    u_int		size:8;
    u_int		reserved1:8;
} pdc_pat_dimm_info_t;

/* Cell fabric information.  Platform-specific */
typedef struct pdc_pat_fabric_info {
    unsigned short	chip_id;		/* XBC or RC Chip ID */
    unsigned char	type;			/* Chip type */
    unsigned char	status;			/* IF status */
    unsigned char	reserved0;		/* Reserved */
    unsigned char	ntype;			/* Neighbor type */
    unsigned short	nid;			/* Neighbor id */
} pdc_pat_fabric_info_t;

#define PDC_PAT_CELL_HD_SLOTS	16
#define PDC_PAT_CELL_HD_DIMMS	32
#define PDC_PAT_CELL_HD_XBC	 4
#define PDC_PAT_CELL_HD_PORTS	(8*2)

/* Cell information.  Platform-specific */
typedef struct pdc_pat_cell_info_maddr {
    unsigned int	major_rev;		/* PDC major revision */
    unsigned int	minor_rev;		/* PDC minor revision */
    pdc_capabilities_t	capabilities;		/* PDC capabilities */
    ulong_t		reserved0;
    ulong_t		reserved1;
    pdc_pat_cell_info_t	cell_info;
    pdc_pat_phys_loc_t	cell_location;		/* Cell physical location */
    pdc_pat_cpu_info_t	cpu_info;		/* Cpu information */
    ulong_t		cpu_speed;		/* Clock ticks in 10msec */
    pdc_pat_phys_loc_t	chassis_location;	/* I/O chassis location */
    ulong_t		cell_io_info;		/* */
    ulong_t		reserved2;
    ulong_t		reserved3;
    unsigned char	reserved4[6];
    unsigned short	io_slot_info_size;
    pdc_pat_slot_info_t	io_slot[PDC_PAT_CELL_HD_SLOTS];	/* Slot information */
    ulong_t		memory_size;		/* Amount of phys mem bytes */
    unsigned char	reserved5[6];
    unsigned short	cell_dimm_info_size;
    pdc_pat_dimm_info_t	dimm_status[PDC_PAT_CELL_HD_DIMMS];/* Memory status */
    unsigned char	reserved6[6];
    unsigned short	fabric_info_size;
    pdc_pat_fabric_info_t fabric_info[PDC_PAT_CELL_HD_XBC][PDC_PAT_CELL_HD_PORTS];	/* Fabric information */
} pdc_pat_cell_info_maddr_t;

typedef struct pdc_pat_fan_info_s {
    unsigned int	count       : 1;	/* If 1, bitmask represents */
						/* only the number of fans */
    unsigned int	valid       : 1;	/* If 1, information is */
						/* dubious */
    unsigned int	reserved_1  : 6;
    unsigned int	state       : 8;	/* Cooling state */
    unsigned int	reserved_2  :16;
    unsigned int	bitmask;		/* Bitmask marking location */
} pdc_pat_fan_info_s_t;

typedef struct pdc_pat_iofan_info_s {
    unsigned int	count3      : 1;	/* If 1, bitmask represents */
						/* only the number of fans */
    unsigned int	valid3      : 1;	/* If 1, information is */
						/* dubious */
    unsigned int	count2      : 1;
    unsigned int	valid2      : 1;
    unsigned int	count1      : 1;
    unsigned int	valid1      : 1;
    unsigned int	count0      : 1;
    unsigned int	valid0      : 1;
    unsigned int	state3      : 2;	/* Cooling state */
    unsigned int	state2      : 2;	/* Cooling state */
    unsigned int	state1      : 2;	/* Cooling state */
    unsigned int	state0      : 2;	/* Cooling state */
    unsigned int	reserved_1  : 8;
    unsigned int	bitmask3    : 8;	/* Bitmask marking location */
    unsigned int	bitmask2    : 8;	/* Bitmask marking location */
    unsigned int	bitmask1    : 8;	/* Bitmask marking location */
    unsigned int	bitmask0    : 8;	/* Bitmask marking location */
    unsigned int	reserved_2  : 4;
    unsigned int	ice_count   : 4;
} pdc_pat_iofan_info_s_t;

typedef struct pdc_pat_fan_info_ns {
    unsigned int	count       : 1;	/* If 1, bitmask represents */
						/* only the number of fans */
    unsigned int	valid       : 1;	/* If 1, information is */
						/* dubious */
    unsigned int	reserved    :30;
    unsigned int	bitmask;		/* Bitmask marking location */
} pdc_pat_fan_info_ns_t;

typedef struct pdc_pat_iofan_info_ns {
    unsigned int	count3      : 1;	/* If 1, bitmask represents */
						/* only the number of fans */
    unsigned int	valid3      : 1;	/* If 1, information is */
						/* dubious */
    unsigned int	count2      : 1;
    unsigned int	valid2      : 1;
    unsigned int	count1      : 1;
    unsigned int	valid1      : 1;
    unsigned int	count0      : 1;
    unsigned int	valid0      : 1;
    unsigned int	reserved_1  : 16;
    unsigned int	bitmask3    : 8;	/* Bitmask marking location */
    unsigned int	bitmask2    : 8;	/* Bitmask marking location */
    unsigned int	bitmask1    : 8;	/* Bitmask marking location */
    unsigned int	bitmask0    : 8;	/* Bitmask marking location */
    unsigned int	reserved_2  : 4;
    unsigned int	ice_count   : 4;
} pdc_pat_iofan_info_ns_t;

typedef struct pdc_pat_pwr_s_info {
    unsigned int	count       : 1;
    unsigned int	valid       : 1;
    unsigned int	reserved_1  : 6;
    unsigned int	state       : 8;	/* Power supply state */
    unsigned int	reserved_2  :16;
    unsigned int	bitmask;
} pdc_pat_pwr_info_s_t;

typedef struct pdc_pat_pwr_ns_info {
    unsigned int	count       : 1;
    unsigned int	valid       : 1;
    unsigned int	reserved    :30;
    unsigned int	bitmask;
} pdc_pat_pwr_info_ns_t;

/* State values */
#define PDC_PAT_STATE_UNKNOWN	0
#define PDC_PAT_STATE_NPLUS	1
#define PDC_PAT_STATE_N		2
#define PDC_PAT_STATE_NMINUS	3

typedef struct pdc_pat_hd_cab_info {
    unsigned int	reserved_1;
    unsigned int	reserved_2   : 4;
    unsigned int	sub3_status  : 4;	/* SUB 3 Status */
    unsigned int	sub2_status  : 4;	/* SUB 2 Status */
    unsigned int	sub1_status  : 4;	/* SUB 1 Status */
    unsigned int	sub0_status  : 4;	/* SUB 0 Status */
    unsigned int	subs_present : 4;	/* Total SUBs in cabinet */
    unsigned int	type         : 8;	/* Cabinet type */
} pdc_pat_hd_cab_info_t;

/* Platform-specific */
typedef union pdc_pat_hv_cab_info {
    uint64_t 			generic;
    pdc_pat_hd_cab_info_t	hd;
} pdc_pat_hv_cab_info_t;

/* PDC_PAT_COMPLEX(Option 4) */
typedef struct pdc_pat_complex_lock_profile_rtn_block {
    ulong_t         lock_key;	/* Lock magic cookie */
    ulong_t         filler[31];
} pdc_pat_complex_lock_profile_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 6) */
typedef struct pdc_pat_complex_get_cab_info_rtn_block {
    pdc_pat_fan_info_s_t	cab_fans_present;
    pdc_pat_fan_info_ns_t	cab_fans_failed;
    pdc_pat_pwr_info_s_t	cab_power_present;
    pdc_pat_pwr_info_ns_t	cab_power_failed;
    pdc_pat_pwr_info_ns_t	cab_power_slots_present;
    pdc_pat_fan_info_ns_t	cab_fan_slots_present;
    pdc_pat_iofan_info_s_t	io_fans_present_0;
    pdc_pat_iofan_info_ns_t	io_fans_failed_0;
    pdc_pat_iofan_info_ns_t	io_fan_slots_present_0;
    pdc_pat_pwr_info_s_t	io_power_present;
    pdc_pat_pwr_info_ns_t	io_power_failed;
    pdc_pat_pwr_info_ns_t	io_power_slots_present;
    pdc_pat_pwr_info_s_t	bp_power_present;
    pdc_pat_pwr_info_ns_t	bp_power_failed;
    pdc_pat_pwr_info_ns_t 	bp_power_slots_present;
#if __LP64__
    ulong_t		ac_power_present;
    ulong_t		reserved_1;
    ulong_t		reserved_2;
    pdc_pat_hv_cab_info_t hv_cabinet_info;
    pdc_pat_phys_loc_t  sub_0_location;
    pdc_pat_phys_loc_t  sub_1_location;
    pdc_pat_phys_loc_t  sub_2_location;
    pdc_pat_phys_loc_t  sub_3_location;
    pdc_pat_iofan_info_s_t	io_fans_present_4;
    pdc_pat_iofan_info_ns_t	io_fans_failed_4;
    pdc_pat_iofan_info_ns_t	io_fan_slots_present_4;
    ulong_t		filler[6];
#endif
} pdc_pat_complex_get_cab_info_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 7) */
typedef struct pdc_pat_complex_get_cell_states_rtn_block {
    ulong_t         slots_present;
    ulong_t         cells_present;
    ulong_t         cells_powered;
    ulong_t         cells_ready;
    ulong_t	    filler[28];
} pdc_pat_complex_get_cell_states_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 8) */
typedef struct pdc_pat_complex_get_cell_list_rtn_block {
    unsigned char   root_cell[PROFILE_NPD];	/* The root cell list is */
						/* divided into bytes. */
						/* Each byte contains the */
						/* number of the root cell */
						/* for that protection */
						/* domain. */
#ifdef __LP64__				
    ulong_t	    filler[24];
#else
    ulong_t	    filler[16];
#endif
} pdc_pat_complex_get_cell_list_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 9) */
typedef struct pdc_pat_complex_get_dyn_profile_rtn_block {
    ulong_t	    count;			/* Actual count */
    ulong_t	    filler[31];
} pdc_pat_complex_get_dyn_profile_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 11) */
typedef struct pdc_pat_complex_lock_dyn_profile_rtn_block {
    ulong_t	    lock_key;			/* Lock magic cookie */
    ulong_t	    filler[31];
} pdc_pat_complex_lock_dyn_profile_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 13) */
typedef struct pdc_pat_complex_get_max_cabs_rtn_block {
    ulong_t	    max_cabinets;		/* Max cabinets in complex */
    ulong_t	    filler[31];
} pdc_pat_complex_get_max_cabs_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 15) */
typedef struct pdc_pat_complex_get_chip_rtn_block {
    ulong_t         chip_id;            	/* Chip ID */
    ulong_t         filler[31];
} pdc_pat_complex_get_chip_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 16) */
typedef struct pdc_pat_complex_get_loc_rtn_block {
    ulong_t	    location;			/* Location of given chip id */
    ulong_t	    filler[31];
} pdc_pat_complex_get_loc_rtn_block_t;

/* PDC_PAT_COMPLEX(Option 132) */
typedef struct pdc_pat_complex_get_rc_status_block {
    ulong_t         in_use_bitmask;
    uint32_t        reserved1;
    uint32_t        nid;
    uint32_t        reserved2;
    uint32_t        rc_status;
#if __LP64__
    ulong_t	    filler[29];
#else
    ulong_t	    filler[27];
#endif
} pdc_pat_complex_get_rc_status_block_t;

/* PDC_PAT_COMPLEX(Option 135) */
typedef struct pdc_pat_complex_rcd_routing_maddr {
    unsigned char	rcd_routing[PROFILE_NTOGO][PROFILE_NCELL / 2];
} pdc_pat_complex_rcd_routing_maddr_t;

/********************************************************************
* Return block structures for PDC_PAT_CHASSIS_LOG
********************************************************************/
/* PDC_PAT_CHASSIS_LOG(Option 2) */
typedef struct pdc_pat_chassis_log_depth_rtn_block {
    ulong_t	    max_entries;		/* Maximum number of */
						/* chassis log entries */
    ulong_t	    filler[31];
} pdc_pat_chassis_log_depth_rtn_block_t;

/* PDC_PAT_CHASSIS_LOG(Option 3) */
typedef struct pdc_pat_chassis_read_multi_rtn_block {
    ulong_t	    count;			/* Actual # of entries */
    ulong_t	    filler[31];
} pdc_pat_chassis_read_multi_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_PD
********************************************************************/
/* PDC_PAT_PD(0) */
typedef struct pdc_pat_pd_addr_map_rtn_block {
    ulong_t	actual_len;       /* actual # bytes in address map */
    ulong_t	reserved[31];
} pdc_pat_pd_addr_map_rtn_block_t;

#define PDC_PD_ADDR_MAP_ENTRY_TYPE	1    /* address map entry type */
#define PDC_PD_ADDR_MAP_TYPE_REGULAR	0    /* type: regular memory */
#define PDC_PD_ADDR_MAP_USE_UNSPECIFIED	0    /* usage: unspecified */
#define PDC_PD_ADDR_MAP_USE_FIRMWARE	12   /* usage: firmware reserved */
#define PDC_PD_ADDR_MAP_USE_GI		128  /* usage: Global Interleaved */
#define PDC_PD_ADDR_MAP_USE_GNI		129  /* usage: Global NonInterleaved */

typedef struct pdc_pat_pd_addr_map_maddr {
    uint8_t	entry_type;	/* 1 denotes memory entry type */
    uint8_t	r1;		/* Reserved */
    uint8_t	r2;		/* Reserved */
    uint8_t	r3;		/* Reserved */
    uint8_t	r4;		/* Reserved */
    uint8_t	r5;		/* Reserved */
    uint8_t	type;		/* Memory type */
				/* 0 = Regular */
				/* 1 = MMIO */
				/* 2 = SAPIC */
				/* 3 = IA-32 I/O Port */
				/* 4 = Firmware */
				/* 5 = Reserved */
				/* 6 = Bad memory */
    uint8_t	usage;
    uint64_t	phys_addr;	/* Physical address of memory */
    uint32_t	length;		/* Length of memory in 4K pages */
    uint32_t	r6;		/* Reserved */
    uint64_t	oem;		/* OEM reserved (cell number, view flag) */
} pdc_pat_pd_addr_map_maddr_t;

/* PDC_PAT_PD(1) */
typedef struct pdc_pat_pd_get_pdc_rev_rtn_block {
   ulong_t	legacy_revision;
   ulong_t	pat_revision;
   pdc_capabilities_t pdc_cap_rtn;
#if __LP64__
   ulong_t	reserved[29];
#else
   ulong_t	reserved[28];
#endif
} pdc_pat_pd_get_pdc_rev_rtn_block_t;

/* PDC_PAT_PD(3) */
typedef struct pdc_pat_pd_get_profile_rtn_block {
    ulong_t	count;
    ulong_t	filler[31];
} pdc_pat_pd_get_profile_rtn_block_t;

/* PDC_PAT_PD(5) */
typedef struct pdc_pat_pd_lock_profile_rtn_block {
    ulong_t	lock_key;
    ulong_t	filler[31];
} pdc_pat_pd_lock_complex_rtn_block_t;

/* PDC_PAT_PD(7) */
typedef struct pdc_pat_pd_get_olru_rtn_block {
    ulong_t	location;
    ulong_t	filler[31];
} pdc_pat_pd_get_olru_rtn_block_t;

/* HPUX_REVIST */
/* 1/21/2000.  HD PDC ERS and PDC PAT ARS disagree on number */
/* of return values for get platform counter block.  PDC ERS */
/* has a third return value for uniqueness. */
/* PDC_PAT_PD(10) */
typedef struct pdc_pat_pd_get_platform_counter_rtn_block {
    long*	counter_address;
    u_long	frequency;
    u_long	filler[30];
} pdc_pat_pd_get_platform_counter_rtn_block_t;

/* PDC_PAT_PD(11) */
typedef struct pdc_pat_pd_get_alive_cells_rtn_block {
    ulong_t	cell_bitmask;			/* Bitmask of cells */
						/* alive in PD */
    ulong_t	filler[31];
} pdc_pat_pd_get_alive_cells_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_IO
********************************************************************/

/* PDC_PAT_IO_GET_NUM_HOT_PLUGS( Option 0) */
typedef struct pdc_pat_io_num_rtn_block {
    ulong_t		num;		/* Number of hot-plug I/O slots */
					/* in the specified cell. */
    ulong_t		reserved[31];
} pdc_pat_io_num_rtn_block_t;

/* PDC_PAT_IO_GET_SLOT_STATUS( Option 5) */
typedef struct pdc_pat_io_slot_rtn_block {
    ulong_t 		slot_status1;	/* Slot status */
    ulong_t 		slot_status2;	/* More slot status */
    ulong_t 		reserved[30];
} pdc_pat_io_slot_rtn_block_t;

/* PDC_PAT_IO(Option 6) */
typedef struct pdc_pat_io_path2loc_rtn_block {
    ulong_t		location;	/* Physical location */
    ulong_t		filler[31];
} pdc_pat_io_path2loc_rtn_block_t;

/* PDC_PAT_IO(Option 7) */
typedef struct pdc_pat_io_loc2path_rtn_block {
    ulong_t		path;		/* Module path */
    ulong_t		filler[31];
} pdc_pat_io_loc2path_rtn_block_t;

/* PDC_PAT_IO(Option 11) */
typedef struct pdc_pat_io_path2pciaddr_rtn_block {
    ulong_t		config_addr;	/* PCI base config address. */
    ulong_t		filler[31];
} pdc_pat_io_path2pciaddr_rtn_block_t;

/* PDC_PAT_IO(Option 12) */
typedef struct pdc_pat_io_pciaddr2path_rtn_block {
    ulong_t		path;		/* Hardware path */
    ulong_t		filler[31];
} pdc_pat_io_pciaddr2path_rtn_block_t;

/* PDC_PAT_IO(Option 13) */
typedef struct pdc_pat_io_get_bridge_info_rtn_block {
    ulong_t		actual_length;	/* Number of bytes in memaddr. */
    ulong_t		filler[31];
} pdc_pat_io_get_bridge_info_rtn_block_t;

/* PDC_PAT_IO(Option 15) */
typedef struct pdc_pat_io_get_pciint_table_size_rtn_block {
    ulong_t		num;		/* Number of entries in the */
					/* PCI interrupt routing table. */
    ulong_t		filler[31];
} pdc_pat_io_get_pciint_table_size_rtn_block_t;

/* PDC_PAT_IO(Option 17) */
typedef struct pdc_pat_io_get_pcihint_table_size_rtn_block {
    ulong_t		num;		/* Number of entries in the */
					/* PCI hint table. */
    ulong_t		filler[31];
} pdc_pat_io_get_pcihint_table_size_rtn_block_t;

/* PDC_PAT_IO(Option 19) */
typedef struct pdc_pat_io_pciread_rtn_block {
    ulong_t		value;
    ulong_t		filler[31];
} pdc_pat_io_pciread_rtn_block_t;

/* PDC_PAT_IO(Option 21) */
typedef struct pdc_pat_io_get_slots_per_cab_rtn_block {
    ulong_t		slots;		/* Number of hot plug I/O slots */
					/* in the cell specified by cabinet */
    ulong_t		filler[31];
} pdc_pat_io_get_slots_per_cab_rtn_block_t;

/* PDC_PAT_IO(Option 28) mem_addr return */
typedef struct io_chassis_status_info {
    pdc_pat_phys_loc_t	location;
    uint8_t		cell;
    uint8_t		reserved1[3];
    uint32_t		reserved2:28;
    uint32_t		connected:1;
    uint32_t		powered:1;
    uint32_t		populated:1;
    uint32_t		present:1;
} io_chassis_status_info_t;

/* PDC_PAT_IO(Option 29) */
typedef struct pdc_pat_io_get_io_addr_rtn_block {
    ulong_t		io_addr;	/* Processor view of I/O address. */
    ulong_t		filler[31];
} pdc_pat_io_get_io_addr_rtn_block_t;

/* PDC_PAT_IO_PROG_SBA_DIR_RANGE( Option 30) */
typedef struct pdc_pat_io_psba_rtn_block {
    ulong_t		address;	/* zzz directed address */
    ulong_t		reserved[31];
} pdc_pat_io_psba_rtn_block_t;

/* PDC_PAT_IO(Option 33) */
typedef struct pdc_pat_io_get_hard_fail_rtn_block {
    ulong_t		hf_enable;	/* Hard fail enable bit */
    ulong_t		filler[31];
} pdc_pat_io_get_hard_fail_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_EVENTS
********************************************************************/

/* PDC_PAT_EVENT[Get Capabilities] (Option 0) */
typedef struct pdc_pat_events_cap_rtn_block {
	/* 64-bit pattern where each set bit denotes a supported event */
	ulong_t supported_events;

	/* 64-bit pattern where each set bit denotes interrupt notification */
	ulong_t int_notify;

	ulong_t reserved[30];
} pdc_pat_events_cap_rtn_block_t;

/* PDC_PAT_EVENT[Set Notification Mode] (Option 1) */
typedef struct pdc_pat_events_notify_rtn_block {
	/* identifies I/O SAPIC to which signal is connected */
	ulong_t dest_io_sapic_addr;

	/* identifies INTIN n pin to which signal is connected */
	ulong_t dest_io_sapic_intin;

	ulong_t reserved[30];
} pdc_pat_events_notify_rtn_block_t;

/* PDC_PAT_EVENT[Scan Event] (Option 2) */
typedef struct pdc_pat_events_scan_rtn_block {
	/* 64-bit pattern indicating which event is reported */
	ulong_t which_event;

	/* 64-bit flag indicating pending events */
	ulong_t cont;

	/* physical location of module reporting the event */
	ulong_t location;

	/* context dependent data */
	ulong_t data[5];

	ulong_t reserved[24];
} pdc_pat_events_scan_rtn_block_t;

/* PDC_PAT_EVENT[Handle Event] (Option 3) */
typedef struct pdc_pat_events_handle_rtn_block {
	/* return status of the PDC routine handling the event */
	ulong_t action_status;

	ulong_t reserved[31];
} pdc_pat_events_handle_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_CPU
********************************************************************/

/* PDC_PAT_CPU(Option 0) */
typedef struct pdc_pat_cpu_config_rtn_block {
	ulong_t		config_info;
	ulong_t		filler[31];
} pdc_pat_cpu_config_rtn_block_t;

/* PDC_PAT_CPU(Option 3) */
typedef struct pdc_pat_cpu_num_rtn_block {
        ulong_t 	cpu_num;	/* Complex-wide cpu number */
        ulong_t 	location;	/* Physical location  */
        ulong_t 	reserved[30];
} pdc_pat_cpu_num_rtn_block_t;

/* PDC_PAT_CPU(Option 4) */
typedef struct pdc_pat_cpu_hpa_rtn_block {
	ulong_t 	cpu_hpa;	/* HPA of given cpu */
	ulong_t 	location;	/* Physical location */
	ulong_t 	reserved[30];
} pdc_pat_cpu_hpa_rtn_block_t;

/* PDC_PAT_CPU(Option 7) */
typedef struct pdc_pat_cpu_clock_rtn_block {
	ulong_t		ticks_10msec;	/* Number of cpu clock ticks */
					/* in 10msec */
	double		temp_calib;	/* Clock drift */
	ulong_t		clock_domain;	/* Clock domain id */
#if __LP64__
	ulong_t		filler[29];
#else
	ulong_t		filler[26];	/* Double alignment */
					/* of temp_calib causes */
					/* filler to be less than */
					/* one would expect */
#endif
} pdc_pat_cpu_clock_rtn_block_t;

/* PDC_PAT_CPU(Option 8) */
typedef struct pdc_pat_cpu_rendezvous_rtn_block {
	ulong_t		state;		/* Current state */
					/* 0 - Not in rendezvous */
					/* 1 - OS interrupt */
					/* 2 - TOC */
					/* 3 - Waiting for HPMC */
	ulong_t		filler[31];
} pdc_pat_cpu_rendezvous_rtn_block_t;


/********************************************************************
* Return block structures for PDC_PAT_REGISTER_TOC
********************************************************************/

/* PDC_PAT_REGISTER_TOC(Option 1) */

typedef struct pdc_pat_register_toc_get_rtn_block {
	ulong_t		vector;		/* Registered vector */
	ulong_t		length;		/* Registered length */
	ulong_t		filler[30];
} pdc_pat_register_toc_get_rtn_block_t;


/********************************************************************
* Return block structures for PDC_PAT_SYSTEM_INFO
********************************************************************/

/* PDC_PAT_SYSTEM_INFO(Option 0) */
typedef struct pdc_pat_sysinfo_size_rtn_block {
    ulong_t	size;			/* Largest memaddr buffer */
    ulong_t	filler[31];
} pdc_pat_sysinfo_size_rtn_block_t;

/* PDC_PAT_SYSTEM_INFO(Option 1) */
typedef struct pdc_pat_sysinfo_info_rtn_block {
    ulong_t	count;			/* Returned count */
    ulong_t	filler[31];
} pdc_pat_sysinfo_info_rtn_block_t;

/* PDC_PAT_SYSTEM_INFO(Option 2) */
typedef struct pdc_pat_sysinfo_error_rtn_block {
    ulong_t	count;			/* Returned count */
    ulong_t	filler[31];
} pdc_pat_sysinfo_error_rtn_block_t;

/* PDC_PAT_SYSTEM_INFO(Option 3) */
typedef struct pdc_pat_sysinfo_warn_rtn_block {
    ulong_t	count;			/* Returned count */
    ulong_t	filler[31];
} pdc_pat_sysinfo_warn_rtn_block_t;

/* PDC_PAT_SYSTEM_INFO(Option 4) */
typedef struct pdc_pat_sysinfo_rev_rtn_block {
    ulong_t	count;			/* Returned count */
    ulong_t	filler[31];
} pdc_pat_sysinfo_rev_rtn_block_t;

/* PDC_PAT_SYSTEM_INFO(Option 5) */
typedef struct pdc_pat_sysinfo_regs_rtn_block {
    /* HPUX_REVISIT. */
    ulong_t	status_regs[8];		/* Current CPU and PDH status */
					/* registers */
    ulong_t	filler[24];
} pdc_pat_sysinfo_regs_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_UFO
********************************************************************/

/* PDC_PAT_UFO(0) */
typedef struct pdc_pat_ufo_create_rtn {
    ulong_t	pair_id;
    ulong_t	filler[31];
} pdc_pat_ufo_create_rtn_block_t;

/* PDC_PAT_UFO(1) */
typedef struct pdc_pat_ufo_destroy_rtn {
    ulong_t	filler[32];
} pdc_pat_ufo_destroy_rtn_block_t;

/* PDC_PAT_UFO(2) */
typedef struct pdc_pat_ufo_join_rtn_block {
    ulong_t	filler[32];
} pdc_pat_ufo_join_rtn_block_t;

/* PDC_PAT_UFO(3) */
typedef struct pdc_pat_ufo_leave_rtn_block {
    ulong_t	lock_key;
    ulong_t	filler[31];
} pdc_pat_ufo_leave_rtn_block_t;

/* PDC_PAT_UFO(4) */
typedef struct pdc_pat_ufo_get_pair_id_rtn_block {
    ulong_t	pair_id;
    ulong_t	filler[31];
} pdc_pat_ufo_get_pair_id_rtn_block_t;

/* PDC_PAT_UFO(5) */
typedef struct pdc_pat_ufo_req_mstr_rtn_block {
    ulong_t	lock_key;
    ulong_t	filler[31];
} pdc_pat_ufo_req_mstr_rtn_block_t;

/* PDC_PAT_UFO(6) */
typedef struct pdc_pat_ufo_get_cell_addr_rtn_block {
    ulong_t	agent_conf_addr;
    ulong_t	vcell_num;
    ulong_t	filler[30];
} pdc_pat_ufo_get_cell_addr_rtn_block_t;

/********************************************************************
* Return block structures for PDC_PAT_GSM
********************************************************************/

/* PDC_PAT_GSM(1) */
typedef struct pdc_pat_gsm_num_ranges_rtn_block {
    ulong_t	num_ranges;			/* Number of memory windows */
    ulong_t	filler[31];
} pdc_pat_gsm_num_ranges_rtn_block_t;

/* PDC_PAT_GSM(2) */
typedef struct pdc_pat_gsm_range_status_rtn_block {
    ulong_t	mem_base;			/* arg2 from open range */
    ulong_t	mem_range;			/* arg3 from open range */
    ulong_t	access;				/* arg4 from open range */
    ulong_t	share_set;			/* arg5 from open range */
    ulong_t	filler[28];
} pdc_pat_gsm_range_status_rtn_block_t;

/* PDC_PAT_GSM(5) */
typedef struct pdc_pat_gsm_get_coh_set_rtn_block {
    ulong_t	cell_mask;
    ulong_t	tbd0;
    ulong_t	tbd1;
    ulong_t	tbd2;
    ulong_t	filler[28];
} pdc_pat_gsm_get_coh_set_rtn_block_t;

/* PDC_PAT_GSM(12) */
typedef struct pdc_pat_gsm_get_intr_rtn_block {
    ulong_t	vector;				/* Vector set by set gsm intr */
    ulong_t	filler[31];
} pdc_pat_gsm_get_intr_rtn_block_t;

/*******************************************************************/
union u_R_addr {
   u_long  Raddr_ints[32];  /* generic access */
   u_int   Raddr_int32s[32]; 
   
   struct pdc_chassis_rtn_block   	pdc_chassis_rtn;   /* PDC_CHASSIS     */
   struct pdc_pim_rtn_block 	  	pdc_pim_rtn;	   /* PDC_PIM_SIZE    */
   struct pdc_model_rtn_block	  	pdc_model_rtn;     /* PDC_MODEL	      */
   struct pdc_sys_model_rtn_block 	pdc_sys_model_rtn; 
						/* PDC_MODEL_SYS_MODEL*/
   struct pdc_firm_test_get_rtn_block 	pdc_firm_test_get_rtn; 
						 /* PDC_MODEL_FIRM_TEST_GET */
   struct pdc_cache_rtn_block	  	pdc_cache_rtn;     /* PDC_CACHE	     */
   struct pdc_hpa_rtn_block	  	pdc_hpa_rtn;	     /* PDC_HPA	     */
   struct pdc_coproc_rtn_block	  	pdc_coproc_rtn;    /* PDC_COPROC     */
   struct pdc_iodc_rtn_block	  	pdc_iodc_rtn;    /* PDC_IODC_READ    */
   struct iodc_rtn_block	  iodc_rtn;	     /* PDC_IODC_READ	    */
   struct pdc_mem_rtn_block	  pdc_mem_rtn;	     /* PDC_IODC_MEM_NDINIT */
   struct pdc_iodc_clr_rtn_block  pdc_iodc_clr_rtn;  /* PDC_IODC_MEM_CLEAR  */
   struct pdc_iodc_id_chief_block pdc_iodc_id_chief_rtn;  
                                                     /* PDC_IODC_ID_CHIEFTAIN*/
   struct pdc_relocate_info_block pdc_relocate_info; /* PDC_RELOCATE_INFO   */
   struct pdc_tod_time_rtn_block  pdc_tod_time_rtn;  /* PDC_TOD_READ	    */
   struct pdc_tod_calib_rtn_block pdc_tod_calib_rtn; /* PDC_TOD_ITFREQ	    */
   struct pdc_ss_size_rtn_block   pdc_ss_size_rtn;   /* PDC_STABLE	    */
   struct pdc_nvm_size_rtn_block  pdc_nvm_size_rtn;  /* PDC_NVOLATIVE 	    */
   struct pdc_memory_map_rtn_block pdc_memory_map_rtn;
                                                     /* PDC_MEMORY_MAP      */
   struct pdc_btlb_parms_rtn_block pdc_block_tlb_rtn;/* PDC_BLOCK_TLB       */
   struct pdc_hwtlb_parms_rtn_block pdc_hwtlb_rtn;   /* PDC_TLB             */
   struct pdc_hwtlb_state_rtn_block pdc_hwtlb_state_rtn;  
                                                     /* PDC_TLB             */
   struct pdc_mem_info_rtn_block  pdc_mem_info_rtn;  /* PDC_MEM             */
   struct pdc_mem_read_rtn_block  pdc_mem_read_rtn;  /* PDC_MEM             */
   struct pdc_mem_table_rtn_block pdc_mem_table_rtn; /* PDC_MEM             */
   struct pdc_sys_map_find_mod_rtn_block pdc_find_mod_rtn; 
                                                     /* PDC_SYSTEM_MAP      */
   struct pdc_sys_map_find_addr_rtn_block pdc_find_addr_rtn;
                                                     /* PDC_SYSTEM_MAP      */
   struct pdc_sys_map_trans_path_rtn_block pdc_trans_path_rtn;
                                                     /* PDC_SYSTEM_MAP      */
   struct pdc_scsi_parms_get_rtn_block		pdc_scsi_parms_get_rtn;
   struct pdc_scsi_parms_set_rtn_block		pdc_scsi_parms_set_rtn;
   struct pat_cell_num_rtn_block		pat_cell_num_rtn;
   struct pdc_pat_cell_mod_rtn_block		pat_cell_mod_rtn;
   struct pdc_pat_cell_info_rtn_block		pat_cell_info_rtn;
   struct pat_mem_cell_info_rtn_block		pat_mem_cell_info_rtn;
   struct pat_mem_read_rtn_block		pat_mem_read_rtn;
   struct pdc_pat_mem_get_phys_rtn_block	pat_mem_get_phys_rtn;
   struct pdc_pat_mem_pd_pdt_rtn_block		pat_mem_pd_pdt_rtn;
   struct pdc_pat_cell_loc2num_rtn_block	pat_cell_loc2num_rtn;
   struct pdc_pat_cell_num2loc_rtn_block	pat_cell_num2loc_rtn;
   struct pdc_pat_cell_walk_fabric_rtn_block	pat_cell_walk_fabric_rtn;
   struct pdc_pat_cell_route_size_rtn_block     pat_cell_route_size_rtn;
   struct pdc_pat_cell_route_table_rtn_block    pat_cell_route_table_rtn;
   struct pdc_pat_cell_get_pdh_size_rtn_block	pat_cell_get_pdh_size_rtn;
   struct pdc_pat_cell_read_pdh_rtn_block	pat_cell_read_pdh_rtn;
   struct pdc_pat_chassis_log_depth_rtn_block	pat_chassis_log_depth_rtn;
   struct pdc_pat_chassis_read_multi_rtn_block	pat_chassis_read_multi_rtn;
   struct pdc_pat_complex_get_dyn_profile_rtn_block pat_complex_get_dyn_prof_rtn; 
   struct pdc_pat_complex_lock_dyn_profile_rtn_block pat_complex_lock_dyn_prof_rtn;
   struct pdc_pat_complex_get_max_cabs_rtn_block pat_complex_get_cabs_rtn;
   struct pdc_pat_complex_get_chip_rtn_block	pat_complex_get_chip_rtn;
   struct pdc_pat_complex_get_loc_rtn_block	pat_complex_get_loc;
   struct pdc_pat_complex_get_rc_status_block	pat_complex_get_rc_status;
   struct pdc_pat_complex_lock_profile_rtn_block pat_complex_lock_profile;
   struct pdc_pat_complex_get_cab_info_rtn_block pat_complex_cab_info_rtn;
   struct pdc_pat_complex_get_cell_states_rtn_block pat_complex_get_cell_states_rtn;
   struct pdc_pat_complex_get_cell_list_rtn_block pat_complex_get_cell_list_rtn;
   struct pdc_pat_cpu_config_rtn_block		pat_cpu_config_rtn;
   struct pdc_pat_cpu_num_rtn_block		pat_cpu_num_rtn;
   struct pdc_pat_cpu_hpa_rtn_block		pat_cpu_hpa_rtn;
   struct pdc_pat_cpu_clock_rtn_block		pat_cpu_clock_rtn;
   struct pdc_pat_cpu_rendezvous_rtn_block	pat_cpu_rendezvous_rtn;
   struct pdc_pat_register_toc_get_rtn_block	pat_register_toc_get_rtn;
   struct pdc_pat_sysinfo_size_rtn_block	pat_sysinfo_size_rtn;
   struct pdc_pat_sysinfo_info_rtn_block	pat_sysinfo_info_rtn;
   struct pdc_pat_sysinfo_error_rtn_block	pat_sysinfo_error_rtn;
   struct pdc_pat_sysinfo_warn_rtn_block	pat_sysinfo_warn_rtn;
   struct pdc_pat_sysinfo_rev_rtn_block		pat_sysinfo_rev_rtn;
   struct pdc_pat_sysinfo_regs_rtn_block	pat_sysinfo_regs_rtn;
   struct pdc_pat_ufo_create_rtn		pat_ufo_create_rtn;
   struct pdc_pat_ufo_destroy_rtn		pat_ufo_destroy_rtn;
   struct pdc_pat_ufo_join_rtn_block		pat_ufo_join_rtn;
   struct pdc_pat_ufo_leave_rtn_block		pat_ufo_leave_rtn;
   struct pdc_pat_ufo_get_pair_id_rtn_block	pat_ufo_get_pair_id_rtn;
   struct pdc_pat_ufo_req_mstr_rtn_block	pat_ufo_req_mstr_rtn;
   struct pdc_pat_ufo_get_cell_addr_rtn_block	pat_ufo_get_cell_addr_rtn;
   struct pdc_pat_gsm_num_ranges_rtn_block	pat_gsm_num_ranges_rtn;
   struct pdc_pat_gsm_range_status_rtn_block	pat_gsm_range_status_rtn;
   struct pdc_pat_gsm_get_coh_set_rtn_block	pat_gsm_get_coh_set_rtn;
   struct pdc_pat_gsm_get_intr_rtn_block	pat_gsm_get_intr_rtn;
   struct pdc_pat_pd_addr_map_rtn_block		pat_pd_addr_map_rtn;
   struct pdc_pat_pd_get_pdc_rev_rtn_block	pat_pd_pdc_rev_rtn;
   struct pdc_pat_pd_get_profile_rtn_block	pat_pd_get_profile_rtn;
   struct pdc_pat_pd_lock_profile_rtn_block	pat_pd_lock_profile_rtn;
   struct pdc_pat_pd_get_olru_rtn_block		pat_pd_get_olru_rtn;
   struct pdc_pat_pd_get_platform_counter_rtn_block pat_pd_get_counter_rtn;
   struct pdc_pat_pd_get_alive_cells_rtn_block  pat_get_alive_cells_rtn;
   struct pdc_pat_io_num_rtn_block		pat_io_num_rtn;
   struct pdc_pat_io_psba_rtn_block		pat_io_psba_rtn;
   struct pdc_pat_io_slot_rtn_block		pat_io_slot_rtn;
   struct pdc_pat_io_path2loc_rtn_block		pat_io_path2loc_rtn;
   struct pdc_pat_io_loc2path_rtn_block		pat_io_loc2path_rtn;
   struct pdc_pat_io_path2pciaddr_rtn_block	pat_io_path2pciaddr_rtn;
   struct pdc_pat_io_pciaddr2path_rtn_block	pat_io_pciaddr2path_rtn;
   struct pdc_pat_io_get_bridge_info_rtn_block	pat_io_bridge_info_rtn;
   struct pdc_pat_io_get_pciint_table_size_rtn_block pat_io_pciint_size_rtn;
   struct pdc_pat_io_get_pcihint_table_size_rtn_block pat_io_pcihint_size_rtn;
   struct pdc_pat_io_pciread_rtn_block		pat_io_pciread_rtn;
   struct pdc_pat_io_get_slots_per_cab_rtn_block pat_io_slots_per_cab_rtn;
   struct pdc_pat_io_get_io_addr_rtn_block	pat_io_addr_rtn;
   struct pdc_pat_io_get_hard_fail_rtn_block	pat_io_hard_fail_rtn;
   struct pdc_pat_events_cap_rtn_block		pat_events_cap_rtn;
   struct pdc_pat_events_notify_rtn_block	pat_events_notify_rtn;
   struct pdc_pat_events_scan_rtn_block		pat_events_scan_rtn;
   struct pdc_pat_events_handle_rtn_block	pat_events_handle_rtn;

   double Raddr_dbls[15];  /* guarantees double-word alignment */
} ;


#endif /* LOCORE */

/*
 * These are the reserved memory locations used by PDC, IODC, etc. that
 * are defined in the IO ACD.
 */
#define	MEM_POW_FAIL	0x4	/* Power fail routines use PFAIL_VECTOR  */
				/* to refer to location 4 in real memory */
#define	MEM_TOC		0x8
#define	MEM_TOC_LEN	0xC
#define	MEM_RENDEZ	0x10
#define	MEM_PF_LEN	0x14
#define MEM_POW_FAIL_HI 0x18	/* Upper half of 64-bit power fail vector */

#define MEM_TOC_HI      0x24	/* Upper half of 64-bit TOC vector */
#define MEM_RENDEZ_HI   0x28	/* Upper half of 64-bit rendezvous vector */

#define MEM_OBP		0x200	/* OBP entry point on an SPP */

#define	MEM_ERR		0x360
#define MEM_CONT        0x350
#define MEM_CONT_HI	0x340   /* Upper half of 64-bit contiguous memory */
#define MEM_PHSIZE      0x354
#define MEM_PHSIZE_HI	0x344   /* Upper half of 64-bit physical mem size */
#define MEM_ADSIZE      0x358
#define MEM_ADSIZE_HI	0x348   /* Upper half of 64-bit spa value */
#define MEM_FREE        0x380
#define MEM_HPA         0x384
#define	MEM_PDC		0x388
#define MEM_PDC_HI	0x35C	/* Upper half of 64-bit pdc entry point */
#define	MEM_10MSEC	0x38C

#define FIRST_MEM_HPA		0x390
#define FIRST_MEM_SPA_SIZE	0x398
#define FIRST_MEM_SIZE		0x39c

#define PGZ_IMC_SPA_SIZE_64BITOS	0x0338
#define PGZ_IMC_MAX_MEM_64BITOS		0x033c
#define PGZ_IMC_SPA_SIZE_32BITOS	FIRST_MEM_SPA_SIZE
#define PGZ_IMC_MAX_MEM_32BITOS		FIRST_MEM_SIZE

#ifdef __LP64__

#define PA_GET_FIRST_MEM_HPA (0xffffffff00000000L | \
		     (uint64_t)(*(uint32_t *)(long)PAGE_ZERO(FIRST_MEM_HPA)))
	
#define PA_GET_FIRST_MEM_SIZE ((uint64_t) \
		  *(uint32_t *)(long)PAGE_ZERO(PGZ_IMC_MAX_MEM_64BITOS) << (PGSHIFT)) 

#define PA_GET_FIRST_MEM_SPA_SIZE ((uint64_t) \
		  *(uint32_t *)(long)PAGE_ZERO(PGZ_IMC_SPA_SIZE_64BITOS) << (PGSHIFT))

#else  /* ! _LP64__ */

#define PA_GET_FIRST_MEM_HPA (*(uint32_t *)PAGE_ZERO(FIRST_MEM_HPA))
#define PA_GET_FIRST_MEM_SIZE (*(uint32_t *)PAGE_ZERO(PGZ_IMC_MAX_MEM_32BITOS))
#define PA_GET_FIRST_MEM_SPA_SIZE (*(uint32_t *)PAGE_ZERO(PGZ_IMC_SPA_SIZE_32BITOS))

#endif /* _LP64__ */

#define	MEM_CONS_PATH	0x3A0
#define	MEM_CONS_HPA	0x3C0
#define	MEM_CONS_SPA	0x3C4
#define	MEM_CONS_IODC	0x3C8

#define MEM_BOOT_SPA    0x3F4
#define SPP_SPA_MATCH   0xFFFF597F

#define TORNADO_CPU_ID  0xB
#define PCXL_CPU_ID     0xD
#define PCXU_CPU_ID     0xE		/* U and U+ for all but C-class with bug */
#define VR_CPU_ID       0xF
#define PCXU_PLUS_CPU_ID	0x10	/* U+ only on C-class with bug */
#define PCXW_CPU_ID		0x11
#define PCXW_PLUS_CPU_ID	0x12
#define PIRANHA_CPU_ID          0x13
#define MAKO_CPU_ID             0x14


/*
 *  PDC Stack Definitions
 *
 *  The PDC stack frame is declared with a "non-variable" size to aide
 *  in debugging (as well as simplifying the code)  without any real loss
 *  in functionality.  The down side being that defining PDC procedures
 *  with more arguments than allowed for here will cause a change to the
 *  pdc interface routines.
 *    
 *  The frame layout is as follows:
 *
 *           +----------------------------+ <-- base of PDC stack frame  
 *           | Local variable space for   |
 *           | use by the kernel assembly |
 *           | routine which calls PDC.   |
 *           +----------------------------+   
 *           | PDC argument area.  Large  |
 *           | enough for arg0..arg9.     |
 *           +----------------------------+   
 *           | Standard Frame Marker      |
 *           | (see calling conventions)  |
 *           +----------------------------+ <-- top of PDC stack frame
 *    
 * NOTE: each of the above sections should be a "double-word" multiple in
 *       size while the entire frame MUST be.
 *
 * NOTE: For PA 2.0 (64-bit mode), the procedure calling convention has
 *       changed. The size of the local variables has been increased from
 *       32 bits to 64 bits. The argument area has grown from 10 32-bit
 *       parameters to 10 64-bit parameters. The direction of argument
 *       storage has also changed along with the notion of the argument
 *       pointer (sp relative references are no longer valid). The frame
 *       marker has changed from 8 32-bit entities to 2 64-bit entities.
 */

#ifndef __LP64__
#define PDC_LOCAL_VAR_SIZE	4	/* words; 4 int's (see below) */
#define PDC_ARG_SIZE		10	/* words; 10 32-bit args */
#define PDC_FM_SIZE		8 	/* words (see calling conventions) */
#else  /* __LP64__ */
#define PDC_LOCAL_VAR_SIZE	4       /* 4 double words */
#define PDC_ARG_SIZE		10      /* 10 double words */
#define PDC_FM_SIZE		2	/* 2 double words */
#endif  /* __LP64__ */

#ifdef __LP64__
#define PSW_DEFAULT_WIDE_BIT    0x0000000000000002L
#endif /* __LP64__ */


#define PDC_FRAME_SIZE	(PDC_LOCAL_VAR_SIZE + PDC_ARG_SIZE + PDC_FM_SIZE)

#define PDC_STACK_SIZE	(8 * 1024)	/* bytes */

/* Offsets of the local variables. */

#ifndef __LP64__
#define PDC_HTA_OFFSET	0x0
#define PDC_PPDP_OFFSET	0x4
#define PDC_PSW_OFFSET	0x8
#define PDC_PNUM_OFFSET	0xC
#else  /* __LP64__ */
#define PDC_HTA_OFFSET	0x0
#define PDC_PPDP_OFFSET	0x8
#define PDC_PSW_OFFSET	0x10
#define PDC_PNUM_OFFSET	0x18
#endif  /* __LP64__ */

#endif  /* ! _MACHINE_PDC_RQSTS_INCLUDED */
