/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/rdb/rdb.h		$Revision: $
 * 
 *  Purpose:
 *      <<please update with a synopis of the functionality provided by this file>>
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  no
 *
 *  Shipped:  yes
 *	/usr/include/pa/rdb.h
 *	/usr/conf/pa/rdb.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif


#ifndef _MACHINE_RDB_INCLUDED
#define _MACHINE_RDB_INCLUDED

/*
 * Flag values for the flag rdb_present used to determine if rdb booted the
 * kernel.
 */
#define RDB_BOOT	1
#define RDB_BOOT_BIT	31
#define RDB_STOP	2
#define RDB_STOP_BIT	30
#define RDB_NORS232   0x4000
#define RDB_NORS232_BIT 17

/* The rest of this module is rdb specific */

#define	RDB_BRK_EVENT        1		/* Break-point. */
#define	RDB_STEP_BRK_EVENT   2		/* Single-step break-point. */
#define	RDB_RECOV_CTR_EVENT  3		/* Single-step recovery counter. */
#define	RDB_SYNC_EVENT       4		/* MP syncronization */
#define RDB_COMMINT_EVENT    5		/* a remote debug comm dev int */
#define RDB_PF_STOP_EVENT    6		/* Power-fail event synch */
#define RDB_DATABREAK_EVENT  7		/* Data Mem Break event */

/*
 * LED data to be displayed when fatal errors occur.
 * codes below 0x60 are for communications driver lockups
 * codes above 0x59 are for monitor lockups
 */

/* driver lock ups */
#define	BAD_SEMA_ALIGN	0x5a	/* semaphore lock at bad addr */
#define BAD_BUFFER	0x5b	/* Left over words in the parallel card */
				/* buffer when it should have been empty. */
#define BAD_ALIGN	0x5c	/* Word alignment problem in command address */
#define HARD_ERR	0x5d	/* Hardware problem in parallel card */
#define BAD_CMD		0x5e	/* Unrecognized command */
#define BAD_STATE	0x5f	/* Bad State */

/* Monitor lockup codes */
#define	COMM_SEMA_NOT_LOCK 0x60	/* comm semaphore should be locked but isn't */
#define	BAD_COMM_ACCESS	0x61	/* a routine that is reserved to the comm */
				/* was called by a ConProc (non-comm) */
#define	BAD_MON_INT	0x62	/* a high priority interrupt screw up */
#define	BAD_EXIT	0x63	/* we returned when we weren't supposed to */
#define	BAD_GLOBAL_STATE 0x64	/* Global State, CommProc, etc don't agree */
#define	BAD_ENTRY_CODE	0x65	/* entry to stop_Handler mit bad code */
#define PDC_DEATH	0x66	/* PDC failed and shouldn't have */
#define	UNKNOWN_INT	0x67	/* got one we don't know how to handle */
#define	BAD_HPA		0x68	/* unreasonable HPA */
#define	BAD_PDC_LAUNCH	0x69	/* can't get to PDC */
#define	EIR_NOT_EXPECTED 0x6a	/* an EIR from the Remote was there and */
				/* it shouldn't have been */
#define	UNEXPECTED_COMM_INT 0x6b /*comm interrupt of wrong kind */
#define	BAD_STACK	0x71	/* seems to be a naughty stack */
#define	BAD_DEV_INT	0x72	/* a bad dev interrupt code was returned */
#define	BAD_GET_IODC	0x73

#ifndef LOCORE /* C code ahead */

/*
 * The data structure of type rdb_comm_area_t provides general info about
 * the target system.  We're starting it off with the same info as used to
 * be at the fixed addresses on the page beginning at 0x8000, before PCX-U
 * came along and stomped all over those addresses with its PDC procedures.
 *
 * The fields are:
 *
 *    rdb_version
 *
 *       A 32-bit integer identifying the version of RDB code in the target.
 *       The host debugger code uses this field to tell what sort of target
 *       they're looking at.  The value is set from rdb_version_value in
 *       stand/rdb_com.c .
 *
 *       Make sure that no matter what happens to the rdb_comm_area_t data
 *       type in the future, this field is always the first 32 bits.  Else
 *       some versions of the host debugger won't be able to use this field.
 *
 *       The value contains a major number in the high byte and a minor
 *       number in the low byte.  The major numbers are:
 *
 *	    0	non-mp
 *	    1	first mp implementation
 *	    2	first released mp implementation
 *	    3	first convergent mp version (phase II command abstractions)
 *	    4	full command abstraction convergence (s300 hpux 5.x remote)
 *          5   Undocumented; used by 10.x and perhaps before
 *
 *   executing_flag
 *
 *      Equals 1 if processor zero is executing, or zero if it isn't.  Maybe
 *      not needed anymore.
 *
 *   single_step_flag
 *
 *      Equals 1 if we're single-stepping an instruction, else zero.
 *
 *   cpu_state_change_flag
 *
 *      Equals 1 if a CPU has changed state, else zero.
 *
 *   MP_info_addr
 *
 *      Pointer to the MP_info array describing our processors.
 */

struct rdb_comm_area_t
{
    int rdb_version;			/* was ADR_VERSION */
    int executing_flag;			/* was ADR_CONTFLG */
    int single_step_flag;		/* was ADR_STEP_FLAG */
    int cpu_state_change_flag;          /* was ADR_PROC_STATE_CHANGE */
    struct MP_info *MP_info_addr;	/* was ADR_MP_INFO */
};
#endif /* LOCORE */

/*
 * Various offsets into save state areas.
 */
#if !defined(__LP64__)
#define	Sv_St_GR	0		/* start of general registers */
#define	Sv_St_SR	0x80		/* start of space   registers */
#define	Sv_St_CR	0xa0		/* start of control registers */
#define	Sv_St_IVA	0xbc		/* iva pointer */
#define	Sv_St_EIEM	0xc0		/* ext int enable mask */
#define	Sv_St_PCSQH	0xc8		/* pc space queue head */
#define	Sv_St_PCSQT	0xcc		/* pc space queue tail */
#define	Sv_St_PCOQH	0xd0		/* pc offset queue head */
#define	Sv_St_PCOQT	0xd4		/* pc offset queue tail */
#define	Sv_St_IIR	0xd8		/* interruption inst. reg */
#define	Sv_St_ISR	0xdc		/*       "      space reg */
#define	Sv_St_IOR	0xe0		/*	 "	offset    */
#define	Sv_St_IPSW	0xe4		/* 	 "	psw	  */
#define	Sv_St_EIRR	0xe8		/* ext int request reg (read only) */
#define	Sv_St_PDA	0xec		/* public display of affection */
#define	Sv_St_HTA	0xf0		/* horrible torture affection (pda) */
#define	Sv_St_TR	0xf4		/* temps start here */
#define	Sv_St_SS_FCODE	0x10c		/* stop code */
#define	Sv_St_SIZE	Sv_St_SS_FCODE+4 /* how big it is */
#define	Sv_St_WORDS	Sv_St_SS_FCODE/4+1
#else /* ! (__LP64__) */
#define	Sv_St_GR	0		/* start of general registers */
#define	Sv_St_SR	0x100		/* start of space   registers */
#define	Sv_St_CR	0x140		/* start of control registers */
#define	Sv_St_IVA	0x178		/* iva pointer */
#define	Sv_St_EIEM	0x180		/* ext int enable mask */
#define	Sv_St_PCSQH	0x190		/* pc space queue head */
#define	Sv_St_PCSQT	0x198		/* pc space queue tail */
#define	Sv_St_PCOQH	0x1a0		/* pc offset queue head */
#define	Sv_St_PCOQT	0x1a8		/* pc offset queue tail */
#define	Sv_St_IIR	0x1b0		/* interruption inst. reg */
#define	Sv_St_ISR	0x1b8		/*       "      space reg */
#define	Sv_St_IOR	0x1c0		/*	 "	offset    */
#define	Sv_St_IPSW	0x1c8		/* 	 "	psw	  */
#define	Sv_St_EIRR	0x1d0		/* ext int request reg (read only) */
#define	Sv_St_PDA	0x1d8		/* public display of affection */
#define	Sv_St_HTA	0x1e0		/* horrible torture affection (pda) */
#define	Sv_St_TR	0x1e8		/* temps start here */
#define	Sv_St_SS_FCODE	0x218		/* stop code */
#define	Sv_St_SIZE	Sv_St_SS_FCODE+4 /* how big it is */
#define	Sv_St_WORDS	Sv_St_SS_FCODE/4+1
#endif /* !defined(__LP64__) */

/*
 * States each processor can be in.  These are different from
 * Global State which describes the overall machine state.
 *
 */

#define	St_Unconfigured	1	/* not there ... lights on nobody's home */
#define St_Mi_Head	2	/* Monitor Idle-waiting for CommProc to see me*/
#define	St_Mi		3	/* Monitor Idle */
#define	St_Exiting	4	/* let's get the show on the road babe */
#define	St_Disabled	5	/* some one turned me off */
#define	St_Running	6	/* not stopped */

/*
 * For a probably suspect reason, vm_machdep.c uses the following:
 */

#define ITICK_PER_SEC	80000000

/*
 * Bit masks for values in rdb_cpu_flags[].
 */

#define RDB_CPU_FLAG_IN_RDB		0x1
#define RDB_CPU_FLAG_DBRK_STEP		0x2
#define RDB_CPU_FLAG_DBRK_STEP_EMUL	0x4
#define RDB_CPU_FLAG_RESTORE_I		0x8

#ifndef LOCORE
/*
 * The followinf ref'ed by spinlock.c .
 */

extern ulong_t comm_and_sync_mask;
#endif /* LOCORE */


#endif /* _MACHINE_RDB_INCLUDED */
