/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/sys/save_state.h		$Revision: $
 * 
 *  Purpose:
 *      <<please update with a synopis of the functionality provided by this file>>
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/save_state.h
 *	/usr/conf/pa/save_state.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif

  
#ifndef	_MACHINE_SAVE_STATE_INCLUDED
#define _MACHINE_SAVE_STATE_INCLUDED

#  include <sys/_inttypes.h>
#  include <sys/stdsyms.h>

/*
 * SAVESTATE RULES OF THE ROAD
 *
 *	These rules describe how to read and modify save_state register
 *	values.
 *
 *	If you are writing user program signal handlers that examine
 *	and/or modify saved registers, these instructions are for you.
 *
 *	If you are writing kernel code that manipulates save_states,
 *	these instructions are for you.
 *
 * INTRODUCTION
 *
 *	The sigcontext structure passed to user program signal handlers
 *	contains a save_state structure which is used to save the
 *	process' pre-signal delivery state so that it can be restored
 *	after the signal handler returns.
 *
 *	These saved registers can be examined and even modified by the
 *	signal handler prior to returning so as to affect the
 *	environment of the process at the place the signal was received.
 *
 *	EXAMINING AND/OR MODIFYING THIS STATE IS INHERENTLY NON-PORTABLE
 *	AND IS THEREFORE DISCOURAGED.  HP RECOMMENDS YOU TRY TO FIND
 *	SOME OTHER WAY TO ACCOMPLISH YOUR GOAL.  IF YOU MUST LOOK AT OR
 *	CHANGE SAVED REGISTERS, BE ADVISED THAT THIS "INTERFACE" COULD
 *	CHANGE AS NEW HARDWARE AND SOFTWARE IS INTRODUCED.
 *
 * INTRODUCTION
 *
 *	There are three kinds of PA-RISC computer:
 *
 *	PA-RISC 1.0 (sometimes called PA83)
 *		32-bit integer registers with 16 64-bit floating point
 *		registers.  No Series 700 is a PA 1.0 machine and only a
 *		few models of the oldest Series 600 and 800 machines are
 *		PA 1.0.
 *
 *	PA-RISC 1.1 (sometimes called PA89)
 *		32-bit integer registers with 32 64-bit floating point
 *		registers.  All Series 700 models are PA 1.1 or later.
 *		Most Series 600 and 800 models are PA 1.1 or later.
 *
 *	PA-RISC 2.0
 *		64-bit integer registers with 32 64-bit floating point
 *		registers.  These machines can be characterized by the
 *		PA8000 series processors inside (as opposed to the
 *		PA7000 series processors in PA 1.0 and PA 1.1 machines).
 *
 *	One save_state structure is defined for all three types of
 *	machine.  Different parts are used depending on the underlying
 *	machine.  These instructions should help you understand how it
 *	works and how to use it.  The scheme is (unfortunately)
 *	complicated so that old programs can be grandfathered in without
 *	having to change them.
 *
 *	The save_state can be divided into two basic parts:  the part
 *	that is the same for all classes of machine and the part that
 *	isn't.  The class-dependent part is controlled by flags in the
 *	ss_flags field (see below).
 *
 * THE (RELATIVELY) INVARIANT PARTS
 *
 *	The not-so-architecture-dependent parts are the ss_flags field,
 *	which is used to decode the very-architecture-dependent parts,
 *	and floating point registers 4 through 15.  These registers are
 *	stored in the same place regardless of architecture variant.
 *
 * THE VARIABLE PARTS
 *
 *	The difference between PA 1.0 and PA 1.1 machines is the
 *	addition 16 new 64-bit floating point registers (numbered 16
 *	through 31) and several new floating point opcodes.  PA 2.0 adds
 *	no new floating point registers, but it does add some new
 *	floating point opcodes.  These registers are stored in a single
 *	place in the save_state for all machines that support them.
 *
 *	Floating point registers 0 through 3 should be used carefully
 *	because they are the floating point exception registers and
 *	there are architectural differences between PA 1.0, 1.1, and
 *	2.0.  (New opcodes were added for PA 1.1 and again for PA 2.0.)
 *
 *	The general, space, and control registers are 32 bits wide on PA
 *	1.0 and PA 1.1 machines.  They are 64 bits wide on PA 2.0
 *	machines.  As a result, there are two separate integer register
 *	save areas in the save_state (ss_narrow and ss_wide).  Which set
 *	is valid at any given time depends on flags in the ss_flags
 *	field of the save_state:
 *
 *	if the SS_WIDEREGS flag is clear
 *		The ss_narrow area is valid and the ss_wide area is
 *		undefined.  This is the case for processes running on PA
 *		1.0 and PA 1.1 machines.
 *
 *	if the SS_WIDEREGS is set ...
 *		and the SS_NARROWISINVALID is clear
 *			The ss_wide area is valid and the ss_narrow area
 *			holds valid copies of the low-order 32 bits of
 *			the corresponding registers.  It is the
 *			responsibility of the process (archived code
 *			plus shared libraries plus signal handlers) to
 *			know if the high-order 32 bits of any of the
 *			registers is "live".  If any of the high halves
 *			are live, the signal handler should use ss_wide.
 *			If the program is "old" (uses no PA 2.0
 *			opcodes), it should be safe to use ss_narrow.
 *
 *		and the SS_NARROWISINVALID is set
 *			The ss_wide area is valid and the ss_narrow area
 *			is undefined.  In this case, only the PA 2.0
 *			save area is valid.  The program should not
 *			expect ss_narrow to contain anything useful.
 *
 *	To change a register in the environment the signal handler will
 *	return to, change registers in the following locations:
 *
 *	if the SS_WIDEREGS flag is clear
 *		Change registers in the ss_narrow area.  Leave the
 *		ss_wide area alone.  Make sure the SS_MODIFIEDWIDE flag
 *		is clear before returning from the signal handler.
 *
 *		Example: ssp->ss_narrow.ss_gr1 = foo;
 *			 ssp->ss_flags &= ~SS_MODIFIEDWIDE;
 *
 *	if the SS_WIDEREGS flag is set ...
 *		and the SS_NARROWISINVALID is clear
 *			There are two choices:
 *
 *			change registers in the ss_narrow area ...
 *				Make the necessary changes to the
 *				high-order halves in the ss_wide area
 *				and the low-order halves in the
 *				ss_narrow area.  (Changes to the high
 *				halves can be dispensed with at the
 *				handler's discretion if it knows that
 *				none of these register halves is
 *				"live".)  Make sure the SS_MODIFIEDWIDE
 *				flag is clear before returning from the
 *				signal handler.
 *
 *				Example: ssp->ss_wide.ss_32.ss_gr1_hi = foo_hi;
 *					 ssp->ss_narrow.ss_gr1 = foo_lo;
 *					 ssp->ss_flags &= ~SS_MODIFIEDWIDE;
 *
 *			change registers in the ss_wide area ...
 *				Make the necessary changes to the
 *				ss_wide area and make sure the
 *				SS_MODIFIEDWIDE flag is set before
 *				returning from the signal handler.
 *
 *				Example: ssp->ss_wide.ss_64.ss_gr1 = foo;
 *					 ssp->ss_flags |= SS_MODIFIEDWIDE;
 *
 *		and the SS_NARROWISINVALID is set
 *			Make the necessary changes to the ss_wide area
 *			and make sure the SS_MODIFIEDWIDE flag is set
 *			before returning from the signal handler.
 *
 *			Example: ssp->ss_wide.ss_64.ss_gr1 = foo;
 *				 ssp->ss_flags |= SS_MODIFIEDWIDE;
 */

#if defined(_KERNEL) || !defined(_XPG4_EXTENDED)
typedef struct fp_dbl_block {
#else /* defined(_KERNEL) || !defined(_XPG4_EXTENDED) */
typedef struct __fp_dbl_block {
#endif /* defined(_KERNEL) || !defined(_XPG4_EXTENDED) */
	double ss_fp0;			/* Must be double word aligned */
	double ss_fp1;
	double ss_fp2;
	double ss_fp3;
	double ss_fp4;
	double ss_fp5;
	double ss_fp6;
	double ss_fp7;
	double ss_fp8;
	double ss_fp9;
	double ss_fp10;
	double ss_fp11;
	double ss_fp12;
	double ss_fp13;
	double ss_fp14;
	double ss_fp15;
	double ss_fp16;
	double ss_fp17;
	double ss_fp18;
	double ss_fp19;
	double ss_fp20;
	double ss_fp21;
	double ss_fp22;
	double ss_fp23;
	double ss_fp24;
	double ss_fp25;
	double ss_fp26;
	double ss_fp27;
	double ss_fp28;
	double ss_fp29;
	double ss_fp30;
	double ss_fp31;
} fp_dbl_block_t;

#if defined(_KERNEL) || !defined(_XPG4_EXTENDED)
typedef struct fp_int_block {
#else /* defined(_KERNEL) || !defined(_XPG4_EXTENDED) */
typedef struct __fp_int_block {
#endif /* defined(_KERNEL) || !defined(_XPG4_EXTENDED) */
	int ss_fpstat;			/* Must be double word aligned */
	int ss_fpexcept1;
	int ss_fpexcept2;
	int ss_fpexcept3;
	int ss_fpexcept4;
	int ss_fpexcept5;
	int ss_fpexcept6;
	int ss_fpexcept7;
	int ss_fp4_hi;
	int ss_fp4_lo;
	int ss_fp5_hi;
	int ss_fp5_lo;
	int ss_fp6_hi;
	int ss_fp6_lo;
	int ss_fp7_hi;
	int ss_fp7_lo;
	int ss_fp8_hi;
	int ss_fp8_lo;
	int ss_fp9_hi;
	int ss_fp9_lo;
	int ss_fp10_hi;
	int ss_fp10_lo;
	int ss_fp11_hi;
	int ss_fp11_lo;
	int ss_fp12_hi;
	int ss_fp12_lo;
	int ss_fp13_hi;
	int ss_fp13_lo;
	int ss_fp14_hi;
	int ss_fp14_lo;
	int ss_fp15_hi;
	int ss_fp15_lo;
	int ss_fp16_hi;
	int ss_fp16_lo;
	int ss_fp17_hi;
	int ss_fp17_lo;
	int ss_fp18_hi;
	int ss_fp18_lo;
	int ss_fp19_hi;
	int ss_fp19_lo;
	int ss_fp20_hi;
	int ss_fp20_lo;
	int ss_fp21_hi;
	int ss_fp21_lo;
	int ss_fp22_hi;
	int ss_fp22_lo;
	int ss_fp23_hi;
	int ss_fp23_lo;
	int ss_fp24_hi;
	int ss_fp24_lo;
	int ss_fp25_hi;
	int ss_fp25_lo;
	int ss_fp26_hi;
	int ss_fp26_lo;
	int ss_fp27_hi;
	int ss_fp27_lo;
	int ss_fp28_hi;
	int ss_fp28_lo;
	int ss_fp29_hi;
	int ss_fp29_lo;
	int ss_fp30_hi;
	int ss_fp30_lo;
	int ss_fp31_hi;
	int ss_fp31_lo;
} fp_int_block_t;

#if defined (_KERNEL) || defined (_UNSUPPORTED)
typedef struct pa83_fp_dbl_block {
	double ss_fp0;			/* Must be double word aligned */
	double ss_fp1;
	double ss_fp2;
	double ss_fp3;
	double ss_fp4;
	double ss_fp5;
	double ss_fp6;
	double ss_fp7;
	double ss_fp8;
	double ss_fp9;
	double ss_fp10;
	double ss_fp11;
	double ss_fp12;
	double ss_fp13;
	double ss_fp14;
	double ss_fp15;
} pa83_fp_dbl_block_t;

typedef struct pa83_fp_int_block {
	int ss_fpstat;		/* Must be double word aligned */
	int ss_fpexcept1;
	int ss_fpexcept2;
	int ss_fpexcept3;
	int ss_fpexcept4;
	int ss_fpexcept5;
	int ss_fpexcept6;
	int ss_fpexcept7;
	int ss_fp4_hi;
	int ss_fp4_lo;
	int ss_fp5_hi;
	int ss_fp5_lo;
	int ss_fp6_hi;
	int ss_fp6_lo;
	int ss_fp7_hi;
	int ss_fp7_lo;
	int ss_fp8_hi;
	int ss_fp8_lo;
	int ss_fp9_hi;
	int ss_fp9_lo;
	int ss_fp10_hi;
	int ss_fp10_lo;
	int ss_fp11_hi;
	int ss_fp11_lo;
	int ss_fp12_hi;
	int ss_fp12_lo;
	int ss_fp13_hi;
	int ss_fp13_lo;
	int ss_fp14_hi;
	int ss_fp14_lo;
	int ss_fp15_hi;
	int ss_fp15_lo;
} pa83_fp_int_block_t;
#endif /* defined(_KERNEL) || defined(_UNSUPPORTED) */

/*
 * Save area for 64-bit registers.
 */
typedef struct __reg64 {
#if !defined(__STDC_32_MODE__)
	int64_t ss_reserved;
	int64_t ss_gr1;
	int64_t ss_rp;
	int64_t ss_gr3;
	int64_t ss_gr4;
	int64_t ss_gr5;
	int64_t ss_gr6;
	int64_t ss_gr7;
	int64_t ss_gr8;
	int64_t ss_gr9;
	int64_t ss_gr10;
	int64_t ss_gr11;
	int64_t ss_gr12;
	int64_t ss_gr13;
	int64_t ss_gr14;
	int64_t ss_gr15;
	int64_t ss_gr16;
	int64_t ss_gr17;
	int64_t ss_gr18;
	int64_t ss_gr19;
	int64_t ss_gr20;
	int64_t ss_gr21;
	int64_t ss_gr22;
	int64_t ss_arg3;
	int64_t ss_arg2;
	int64_t ss_arg1;
	int64_t ss_arg0;
	uint64_t ss_dp;
	uint64_t ss_ret0;
	uint64_t ss_ret1;
	uint64_t ss_sp;
	uint64_t ss_gr31;
	uint64_t ss_cr11;
	uint64_t ss_pcoq_head;
	uint64_t ss_pcsq_head;
	uint64_t ss_pcoq_tail;
	uint64_t ss_pcsq_tail;
	uint64_t ss_cr15;
	uint64_t ss_cr19;
	uint64_t ss_cr20;
	uint64_t ss_cr21;
	uint64_t ss_cr22;
	uint64_t ss_cpustate;
	uint64_t ss_sr4;
	uint64_t ss_sr0;
	uint64_t ss_sr1;
	uint64_t ss_sr2;
	uint64_t ss_sr3;
	uint64_t ss_sr5;
	uint64_t ss_sr6;
	uint64_t ss_sr7;
	uint64_t ss_cr0;
	uint64_t ss_cr8;
	uint64_t ss_cr9;
	uint64_t ss_cr10;
	uint64_t ss_cr12;
	uint64_t ss_cr13;
	uint64_t ss_cr24;
	uint64_t ss_cr25;
	uint64_t ss_cr26;
	uint64_t ss_cr27;
	uint64_t ss_reserved2[2];
	uint32_t ss_oldcksum;
	uint32_t ss_newcksum;
#else /* !defined(__STDC_32_MODE__) */
	int ss_reserved[2*64];
#endif /* !defined(__STDC_32_MODE__) */
} __reg64_t;

/*
 * Save area for 64-bit registers (formatted for access one word at a
 * time).  See the definition of __reg64_t above for more details.
 */
typedef struct __reg32 {
	uint32_t ss_reserved[2];
	uint32_t ss_gr1_hi;
	uint32_t ss_gr1_lo;
	uint32_t ss_rp_hi;
	uint32_t ss_rp_lo;
	uint32_t ss_gr3_hi;
	uint32_t ss_gr3_lo;
	uint32_t ss_gr4_hi;
	uint32_t ss_gr4_lo;
	uint32_t ss_gr5_hi;
	uint32_t ss_gr5_lo;
	uint32_t ss_gr6_hi;
	uint32_t ss_gr6_lo;
	uint32_t ss_gr7_hi;
	uint32_t ss_gr7_lo;
	uint32_t ss_gr8_hi;
	uint32_t ss_gr8_lo;
	uint32_t ss_gr9_hi;
	uint32_t ss_gr9_lo;
	uint32_t ss_gr10_hi;
	uint32_t ss_gr10_lo;
	uint32_t ss_gr11_hi;
	uint32_t ss_gr11_lo;
	uint32_t ss_gr12_hi;
	uint32_t ss_gr12_lo;
	uint32_t ss_gr13_hi;
	uint32_t ss_gr13_lo;
	uint32_t ss_gr14_hi;
	uint32_t ss_gr14_lo;
	uint32_t ss_gr15_hi;
	uint32_t ss_gr15_lo;
	uint32_t ss_gr16_hi;
	uint32_t ss_gr16_lo;
	uint32_t ss_gr17_hi;
	uint32_t ss_gr17_lo;
	uint32_t ss_gr18_hi;
	uint32_t ss_gr18_lo;
	uint32_t ss_gr19_hi;
	uint32_t ss_gr19_lo;
	uint32_t ss_gr20_hi;
	uint32_t ss_gr20_lo;
	uint32_t ss_gr21_hi;
	uint32_t ss_gr21_lo;
	uint32_t ss_gr22_hi;
	uint32_t ss_gr22_lo;
	uint32_t ss_arg3_hi;
	uint32_t ss_arg3_lo;
	uint32_t ss_arg2_hi;
	uint32_t ss_arg2_lo;
	uint32_t ss_arg1_hi;
	uint32_t ss_arg1_lo;
	uint32_t ss_arg0_hi;
	uint32_t ss_arg0_lo;
	unsigned int ss_dp_hi;
	unsigned int ss_dp_lo;
	unsigned int ss_ret0_hi;
	unsigned int ss_ret0_lo;
	unsigned int ss_ret1_hi;
	unsigned int ss_ret1_lo;
	unsigned int ss_sp_hi;
	unsigned int ss_sp_lo;
	unsigned int ss_gr31_hi;
	unsigned int ss_gr31_lo;
	unsigned int ss_cr11_hi;
	unsigned int ss_cr11_lo;
	unsigned int ss_pcoq_head_hi;
	unsigned int ss_pcoq_head_lo;
	unsigned int ss_pcsq_head_hi;
	unsigned int ss_pcsq_head_lo;
	unsigned int ss_pcoq_tail_hi;
	unsigned int ss_pcoq_tail_lo;
	unsigned int ss_pcsq_tail_hi;
	unsigned int ss_pcsq_tail_lo;
	unsigned int ss_cr15_hi;
	unsigned int ss_cr15_lo;
	unsigned int ss_cr19_hi;
	unsigned int ss_cr19_lo;
	unsigned int ss_cr20_hi;
	unsigned int ss_cr20_lo;
	unsigned int ss_cr21_hi;
	unsigned int ss_cr21_lo;
	unsigned int ss_cr22_hi;
	unsigned int ss_cr22_lo;
	unsigned int ss_cpustate_hi;
	unsigned int ss_cpustate_lo;
	unsigned int ss_sr4_hi;
	unsigned int ss_sr4_lo;
	unsigned int ss_sr0_hi;
	unsigned int ss_sr0_lo;
	unsigned int ss_sr1_hi;
	unsigned int ss_sr1_lo;
	unsigned int ss_sr2_hi;
	unsigned int ss_sr2_lo;
	unsigned int ss_sr3_hi;
	unsigned int ss_sr3_lo;
	unsigned int ss_sr5_hi;
	unsigned int ss_sr5_lo;
	unsigned int ss_sr6_hi;
	unsigned int ss_sr6_lo;
	unsigned int ss_sr7_hi;
	unsigned int ss_sr7_lo;
	unsigned int ss_cr0_hi;
	unsigned int ss_cr0_lo;
	unsigned int ss_cr8_hi;
	unsigned int ss_cr8_lo;
	unsigned int ss_cr9_hi;
	unsigned int ss_cr9_lo;
	unsigned int ss_cr10_hi;
	unsigned int ss_cr10_lo;
	unsigned int ss_cr12_hi;
	unsigned int ss_cr12_lo;
	unsigned int ss_cr13_hi;
	unsigned int ss_cr13_lo;
	unsigned int ss_cr24_hi;
	unsigned int ss_cr24_lo;
	unsigned int ss_cr25_hi;
	unsigned int ss_cr25_lo;
	unsigned int ss_cr26_hi;
	unsigned int ss_cr26_lo;
	unsigned int ss_cr27_hi;
	unsigned int ss_cr27_lo;
	unsigned int ss_reserved2[4];
	unsigned int ss_oldcksum;
	unsigned int ss_newcksum;
} __reg32_t;

/*
 * Save area for 32-bit registers.
 */
typedef struct __ss_narrow {
#if	defined(_KERNEL) || !defined(__LP64__) 
	int ss_gr1;			/* General Registers */
	int ss_rp;
	int ss_gr3;
	int ss_gr4;
	int ss_gr5;
	int ss_gr6;
	int ss_gr7;
	int ss_gr8;
	int ss_gr9;
	int ss_gr10;
	int ss_gr11;
	int ss_gr12;
	int ss_gr13;
	int ss_gr14;
	int ss_gr15;
	int ss_gr16;
	int ss_gr17;
	int ss_gr18;
	int ss_gr19;
	int ss_gr20;
	int ss_gr21;
	int ss_gr22;
	int ss_arg3;
	int ss_arg2;
	int ss_arg1;
	int ss_arg0;		/* Following on interrupt stack */
	unsigned int ss_dp;
	unsigned int ss_ret0;
	unsigned int ss_ret1;
	unsigned int ss_sp;
	unsigned int ss_gr31;
	unsigned int ss_cr11;		/* Control Registers */
	unsigned int ss_pcoq_head;
	unsigned int ss_pcsq_head;
	unsigned int ss_pcoq_tail;
	unsigned int ss_pcsq_tail;
	unsigned int ss_cr15;
	unsigned int ss_cr19;		/* For break and assist traps only */
	unsigned int ss_cr20;		/* For data page fault only */
	unsigned int ss_cr21;
	unsigned int ss_cr22;
	unsigned int ss_cpustate;		/* Local for thandler */
	unsigned int ss_sr4;		/* Previous sr4 value */
	unsigned int ss_sr0;
	unsigned int ss_sr1;
	unsigned int ss_sr2;
	unsigned int ss_sr3;
	unsigned int ss_sr5;
	unsigned int ss_sr6;
	unsigned int ss_sr7;
	unsigned int ss_cr0;
	unsigned int ss_cr8;
	unsigned int ss_cr9;
	unsigned int ss_cr10;
	unsigned int ss_cr12;
	unsigned int ss_cr13;
	unsigned int ss_cr24;
	unsigned int ss_cr25;
	unsigned int ss_cr26;
	unsigned int ss_cr27;
	unsigned int ss_mpsfu_low;
	unsigned int ss_mpsfu_ovflo;
#else	/* defined(_KERNEL) || !defined(__LP64__) */
	unsigned int	 ss_reserved[62];  /* 32-bit register save area is not
					    * accessible by lp64 application.
					    */
#endif	/* defined(_KERNEL) || !defined(__LP64__) */
} __ss_narrow_t;

typedef int	ssflags_t;

/* 
 * q4 needs the struct save_state name, but COSE can't define it, so
 * keep save_state for kernel and use __save_state for COSE users.
 */
#if defined(_KERNEL) || !defined(_XPG4_EXTENDED)
typedef struct save_state {
#else /* defined(_KERNEL) && !defined(XPG4_EXTENDED) */
typedef struct __save_state {
#endif /* defined(_KERNEL) || !defined(_XPG4_EXTENDED) */
	ssflags_t	ss_flags;	/* Save State Flags */
	__ss_narrow_t	ss_narrow;	/* Save area for 32-bit registers */
	int ss_pad;			 /* For padding to double word boundary */
	union {				 /* Must be double word aligned */
#if defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)
		fp_dbl_block_t fpdbl;
		fp_int_block_t fpint;
#else /* defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED) */
		fp_dbl_block_t __fpdbl;
		fp_int_block_t __fpint;
#endif /* defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED) */
	} ss_fpblock;
	char ss_xor[4*32];		/* reserved */
	union {
		/*
		 * PA2.0 register save area.
		 *
		 * Users of the integer register save areas (gr's, sr's,
		 * and cr's, but not fr's) should call the macro
		 * UseWideRegs() to decide whether to load/store from/to
		 * the old narrow save area or the new wide one.
		 *
		 * For example, if you want gr4, do something like this:
		 *
		 * 	if UseWideRegs(ssp)
		 *	then	use ssp->ss_wide.ss_64.ss_gr4
		 *	else	use ssp->ss_narrow.ss_gr4
		 *	fi
		 *
		 * Note that the ss_wide.ss_64 fields are not accessible
		 * in __STDC_32_MODE__.
		 *
		 * Similarly, all code that fills in a save_state is
		 * expected (from now on) to set the SS_WIDEREGS flag in
		 * the ss_flags field before exposing the save_state to
		 * other code.
		 *
		 * The floating point register save area has not been
		 * duplicated or otherwise expanded because it hasn't
		 * changed.
		 */
		__reg64_t ss_64;	/* formatted as 64-bit values */
		__reg32_t ss_32;	/* or as pairs of 32-bit values */
	} ss_wide;
} save_state_t;
#define _SAVESTATE_T

#if defined (_KERNEL) || defined (_UNSUPPORTED)
typedef struct pa89_save_state {
	ssflags_t ss_flags;		/* Save State Flags */
	int ss_gr1;			/* General Registers */
	int ss_rp;
	int ss_gr3;
	int ss_gr4;
	int ss_gr5;
	int ss_gr6;
	int ss_gr7;
	int ss_gr8;
	int ss_gr9;
	int ss_gr10;
	int ss_gr11;
	int ss_gr12;
	int ss_gr13;
	int ss_gr14;
	int ss_gr15;
	int ss_gr16;
	int ss_gr17;
	int ss_gr18;
	int ss_gr19;
	int ss_gr20;
	int ss_gr21;
	int ss_gr22;
	int ss_arg3;
	int ss_arg2;
	int ss_arg1;
	int ss_arg0;		/* Following on interrupt stack */
	unsigned int ss_dp;
	unsigned int ss_ret0;
	unsigned int ss_ret1;
	unsigned int ss_sp;
	unsigned int ss_gr31;
	unsigned int ss_cr11;		/* Control Registers */
	unsigned int ss_pcoq_head;
	unsigned int ss_pcsq_head;
	unsigned int ss_pcoq_tail;
	unsigned int ss_pcsq_tail;
	unsigned int ss_cr15;
	unsigned int ss_cr19;		/* For break and assist traps only */
	unsigned int ss_cr20;		/* For data page fault only */
	unsigned int ss_cr21;
	unsigned int ss_cr22;
	unsigned int ss_cpustate;		/* Local for thandler */
	unsigned int ss_sr4;		/* Previous sr4 value */
	unsigned int ss_sr0;
	unsigned int ss_sr1;
	unsigned int ss_sr2;
	unsigned int ss_sr3;
	unsigned int ss_sr5;
	unsigned int ss_sr6;
	unsigned int ss_sr7;
	unsigned int ss_cr0;
	unsigned int ss_cr8;
	unsigned int ss_cr9;
	unsigned int ss_cr10;
	unsigned int ss_cr12;
	unsigned int ss_cr13;
	unsigned int ss_cr24;
	unsigned int ss_cr25;
	unsigned int ss_cr26;
	unsigned int ss_mpsfu_high;
	unsigned int ss_mpsfu_low;
	unsigned int ss_mpsfu_ovflo;
	int ss_pad;			 /* For padding to double word boundary */
	union {				 /* Must be double word aligned */
		fp_dbl_block_t fpdbl;
		fp_int_block_t fpint;
	} ss_fpblock;
} pa89_save_state_t;
#endif /* defined(_KERNEL) || defined(_UNSUPPORTED) */

#if defined (_KERNEL) || defined (_UNSUPPORTED)
typedef struct pa83_save_state {
	ssflags_t ss_flags;		/* Save State Flags */
	int ss_gr1;			/* General Registers */
	int ss_rp;
	int ss_gr3;
	int ss_gr4;
	int ss_gr5;
	int ss_gr6;
	int ss_gr7;
	int ss_gr8;
	int ss_gr9;
	int ss_gr10;
	int ss_gr11;
	int ss_gr12;
	int ss_gr13;
	int ss_gr14;
	int ss_gr15;
	int ss_gr16;
	int ss_gr17;
	int ss_gr18;
	int ss_gr19;
	int ss_gr20;
	int ss_gr21;
	int ss_gr22;
	int ss_arg3;
	int ss_arg2;
	int ss_arg1;
	int ss_arg0;		/* Following on interrupt stack */
	unsigned int ss_dp;
	unsigned int ss_ret0;
	unsigned int ss_ret1;
	unsigned int ss_sp;
	unsigned int ss_gr31;
	unsigned int ss_cr11;		/* Control Registers */
	unsigned int ss_pcoq_head;
	unsigned int ss_pcsq_head;
	unsigned int ss_pcoq_tail;
	unsigned int ss_pcsq_tail;
	unsigned int ss_cr15;
	unsigned int ss_cr19;		/* For break and assist traps only */
	unsigned int ss_cr20;		/* For data page fault only */
	unsigned int ss_cr21;
	unsigned int ss_cr22;
	unsigned int ss_cpustate;		/* Local for thandler */
	unsigned int ss_sr4;		/* Previous sr4 value */
	unsigned int ss_sr0;
	unsigned int ss_sr1;
	unsigned int ss_sr2;
	unsigned int ss_sr3;
	unsigned int ss_sr5;
	unsigned int ss_sr6;
	unsigned int ss_sr7;
	unsigned int ss_cr0;
	unsigned int ss_cr8;
	unsigned int ss_cr9;
	unsigned int ss_cr10;
	unsigned int ss_cr12;
	unsigned int ss_cr13;
	unsigned int ss_cr24;
	unsigned int ss_cr25;
	unsigned int ss_cr26;
	unsigned int ss_mpsfu_high;
	unsigned int ss_mpsfu_low;
	unsigned int ss_mpsfu_ovflo;
	int ss_pad;			 /* For padding to double word boundary */
	union {				 /* Must be double word aligned */
		struct pa83_fp_dbl_block fpdbl;
		struct pa83_fp_int_block fpint;
	} ss_fpblock;
} pa83_save_state_t;
#endif /* defined(_KERNEL) || defined(_UNSUPPORTED) */

/* Save State Flags */
#if defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)
#  define	SS_INTRAP	0x01	/* State saved from trap */
#  define	SS_INSYSCALL	0x02	/* State saved from system call */
#  define 	SS_ININT	0x04	/* On the interrupt stack */
#  define	SS_PSPKERNEL	0x08	/* Previous context stack pointer kernel */
#  define	SS_ARGSVALID	0x10	/* Restore argument registers upon return */
#  define	SS_DORFI	0x20	/* Perform an RFI to set new psw */
#  define	SS_WIDEREGS	0x40	/* The 64-bit registers are live. */
#  define	SS_MODIFIEDWIDE	0x80
					/*
					 * When set this flag tells the kernel
					 * NOT to restore the low halves of the
					 * 64-bit registers from the 32-bit
					 * register save area.  Users should
					 * set this flag before returning from
					 * a signal handler that has modified
					 * the low half of any integer
					 * register in the 64-bit save area.
					 */
#  define	SS_NARROWISINVALID	0x100
					/*
					 * When set this flag tells the user's
					 * signal handler that the narrow
					 * integer register save area is NOT
					 * valid and that integer register
					 * contents can only be fetched from
					 * the wide area.
					 */
#  define	SS_LWP_STATE_INTFC2	0x200
					/*
					 * When set this flag tells the user
					 * the new _lwp_{get/set}state()
					 * syscall interface is being used.  
					 * SS_INTRAP -> PC in ss_pcoq_head/tail
					 * SS_INSYSCALL -> PC in ss_gr31.
					 * If this flag is not set, then the
					 * old interface is being used and
					 * the PC will be returned in ss_gr31.
					 *
					 * This flag is used only in the
					 * savestate structures returned/passed
					 * from/to _lwp_{get,set}state().  It
					 * is never set in the U-area nor
					 * kernel-level trap savestates and
					 * will never be seen in the savestate
					 * passed to signal handlers.
					 */

#define	KEEPERBITS	(SS_WIDEREGS|SS_MODIFIEDWIDE|SS_NARROWISINVALID)
					/* Isolate the "wide registers" bits */

#else /* defined(_KERNEL) ||  defined(_INCLUDE_HPUX_SOURCE)  || !defined(_XPG4_EXTENDED) */
#  define	_SS_INTRAP	0x01	/* State saved from trap */
#  define	_SS_INSYSCALL	0x02	/* State saved from system call */
#  define 	_SS_ININT	0x04	/* On the interrupt stack */
#  define	_SS_PSPKERNEL	0x08	/* Previous context stack pointer kernel */
#  define	_SS_ARGSVALID	0x10	/* Restore argument registers upon return */
#  define	_SS_DORFI	0x20	/* Perform an RFI to set new psw */
#  define	_SS_WIDEREGS	0x40	/* The 64-bit registers are live. */
#  define	_SS_MODIFIEDWIDE	0x80
					/*
					 * When set this flag tells the kernel
					 * NOT to restore the low halves of the
					 * 64-bit registers from the 32-bit
					 * register save area.  Users should
					 * set this flag before returning from
					 * a signal handler that has modified
					 * the low half of any integer
					 * register in the 64-bit save area.
					 */
#  define	_SS_NARROWISINVALID	0x100
					/*
					 * When set this flag tells the user's
					 * signal handler that the narrow
					 * integer register save area is NOT
					 * valid and that integer register
					 * contents can only be fetched from
					 * the wide area.
					 */
#  define	_SS_LWP_STATE_INTFC2	0x200
					/*
					 * When set this flag tells the user
					 * the new _lwp_{get/set}state()
					 * syscall interface is being used.  
					 * SS_INTRAP -> PC in ss_pcoq_head/tail
					 * SS_INSYSCALL -> PC in ss_gr31.
					 * If this flag is not set, then the
					 * old interface is being used and
					 * the PC will be returned in ss_gr31.
					 *
					 * This flag is used only in the
					 * savestate structures returned/passed
					 * from/to _lwp_{get,set}state().  It
					 * is never set in the U-area nor
					 * kernel-level trap savestates and
					 * will never be seen in the savestate
					 * passed to signal handlers.
					 */
#endif /* defined(KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)  */

/* Macros */

#if defined(_KERNEL) || !defined(_XPG4_EXTENDED)
#  define	ssoff(x)       ((long)&((struct save_state *)0L)->x)
#  define	ssoff_reg32(x) ((long)&((struct save_state *)0L)->ss_wide.ss_32.x)
#  define	UseWideRegs(ssp)	(((ssp)->ss_flags)&SS_WIDEREGS)

#ifdef	__LP64__
#  define       WideOrNarrowSSReg(ssp,ss_reg) \
		(ssp)->ss_wide.ss_64.ss_reg
#else /* __LP64__ */
#  define       WideOrNarrowSSReg(ssp,ss_reg) \
			WideOrNarrowSSReg32(ssp, ss_reg)
#endif	/* __LP64__ */


/*
 * GetSSReg(): Get the value of a register from the right save area.
 */
#define GetSSReg(ssp, reg)      (WideOrNarrowSSReg(ssp, reg))

/*
 * SetSSReg(): Set the value of a register in the right save area.
 */
#ifdef	__LP64__
#define SetSSReg(ssp, reg, v) \
	(((ssp)->ss_wide.ss_64.reg) = (v))
#else  /* __LP64__ */
#define SetSSReg(ssp, reg, v) \
                (UseWideRegs(ssp) \
                        ? (((ssp)->ss_wide.ss_64.reg) = (v)) \
                        : (((ssp)->ss_narrow.reg) = (v)))
#endif	/* __LP64__ */

/*
 * ModifySSReg(): Modify the value of a register in the right save area.
 */
#ifdef	__LP64__
#define ModifySSReg(ssp, reg, op, v) \
	(((ssp)->ss_wide.ss_64.reg) op (v))
#else  /* __LP64__ */
#define ModifySSReg(ssp, reg, op, v) \
                (UseWideRegs(ssp) \
                        ? (((ssp)->ss_wide.ss_64.reg) op (v)) \
                        : (((ssp)->ss_narrow.reg) op (v)))
#endif	/* __LP64__ */

#if defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__)
#  define	WideOrNarrowSSReg32(ssp,ss_reg) \
		(UseWideRegs(ssp) ? \
			(ssp)->ss_wide.ss_32.ss_reg ## _lo : (ssp)->ss_narrow.ss_reg)
#else	/* defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__) */
#  define	WideOrNarrowSSReg32(ssp,ss_reg) \
		(UseWideRegs(ssp) ? \
			(ssp)->ss_wide.ss_32.ss_reg/**/_lo : (ssp)->ss_narrow.ss_reg)
#endif	/* defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__) */

/*
 * Keepers(): Isolate the "wide registers" bits.
 */
#define	Keepers(ssp)	((ssp)->ss_flags & KEEPERBITS)

#else /* defined(_KERNEL) ||  !defined(_XPG4_EXTENDED) */
#  if defined(_INCLUDE_HPUX_SOURCE)
#     define	ssoff(x)	((long)&((struct __save_state *)0L)->x)
#     define	ssoff_reg32(x)	((long)&((struct __save_state *)0L)->ss_wide.ss_32.x)
#     define	UseWideRegs(ssp)	(((ssp)->ss_flags)&SS_WIDEREGS)

#ifdef	__LP64__
#  define       WideOrNarrowSSReg(ssp,ss_reg) \
		(ssp)->ss_wide.ss_64.ss_reg
#else  /* __LP64__ */
#  define       WideOrNarrowSSReg(ssp,ss_reg) \
			WideOrNarrowSSReg32(ssp, ss_reg)
#endif	/* __LP64__ */

/*
 * GetSSReg(): Get the value of a register from the right save area.
 */
#define GetSSReg(ssp, reg)      (WideOrNarrowSSReg(ssp, reg))

/*
 * SetSSReg(): Set the value of a register in the right save area.
 */
#ifdef	__LP64__
#define SetSSReg(ssp, reg, v) \
	(((ssp)->ss_wide.ss_64.reg) = (v))
#else  /* __LP64__ */
#define SetSSReg(ssp, reg, v) \
                (UseWideRegs(ssp) \
                        ? (((ssp)->ss_wide.ss_64.reg) = (v)) \
                        : (((ssp)->ss_narrow.reg) = (v)))
#endif	/* __LP64__ */

/*
 * ModifySSReg(): Modify the value of a register in the right save area.
 */
#ifdef	__LP64__
#define ModifySSReg(ssp, reg, op, v) \
	(((ssp)->ss_wide.ss_64.reg) op (v))
#else  /* __LP64__ */
#define ModifySSReg(ssp, reg, op, v) \
                (UseWideRegs(ssp) \
                        ? (((ssp)->ss_wide.ss_64.reg) op (v)) \
                        : (((ssp)->ss_narrow.reg) op (v)))
#endif	/* __LP64__ */

#if defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__)
#     define	WideOrNarrowSSReg32(ssp,ss_reg) \
		(UseWideRegs(ssp) ? \
			(ssp)->ss_wide.ss_32.ss_reg ## _lo : (ssp)->ss_narrow.ss_reg)
#else	/* defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__) */
#     define	WideOrNarrowSSReg32(ssp,ss_reg) \
		(UseWideRegs(ssp) ? \
			(ssp)->ss_wide.ss_32.ss_reg/**/_lo : (ssp)->ss_narrow.ss_reg)
#endif	/* defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__) */

#  else /* defined(_INCLUDE_HPUX_SOURCE) */
#     define	__ssoff(x)	((long)&((struct __save_state *)0L)->x)
#     define   __ssoff_reg32(x) ((long)&((struct __save_state *)0L)->ss_wide.ss_32.x)
#     define	__UseWideRegs(ssp)	(((ssp)->ss_flags)&_SS_WIDEREGS)
#if defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__)
#     define	__WideOrNarrowSSReg32(ssp,ss_reg) \
		(__UseWideRegs(ssp) ? \
			(ssp)->ss_wide.ss_32.ss_reg ## _lo : (ssp)->ss_narrow.ss_reg)
#else	/* defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__) */
#     define	__WideOrNarrowSSReg32(ssp,ss_reg) \
		(__UseWideRegs(ssp) ? \
			(ssp)->ss_wide.ss_32.ss_reg/**/_lo : (ssp)->ss_narrow.ss_reg)
#endif	/* defined(__STDC__) || defined(__cplusplus) || defined(__STDCPP__) */

#ifdef  __LP64__
#  define       __WideOrNarrowSSReg(ssp,ss_reg) \
                (ssp)->ss_wide.ss_64.ss_reg
#else  /* __LP64__ */
#  define       __WideOrNarrowSSReg(ssp,ss_reg) \
                        __WideOrNarrowSSReg32(ssp, ss_reg)
#endif	/* __LP64__ */

#  endif   /* defined(_INCLUDE_HPUX_SOURCE)  */
#endif /* defined(KERNEL) || !defined(_XPG4_EXTENDED)  */



/* Useful Aliases */
#if defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)
#  define	ss_gr2	ss_rp
#  define	ss_T3	ss_gr20		/* Third Temp Register */
#  define	ss_T2	ss_gr21		/* Second Temp Register */
#  define	ss_T1	ss_gr22		/* First Temp Register */
#  define	ss_gr23	ss_arg3		/* Fourth Argument */
#  define	ss_gr24	ss_arg2		/* Third Argument */
#  define	ss_gr25	ss_arg1		/* Second Argument */
#  define	ss_gr26	ss_arg0		/* First Argument */
#  define	ss_gr27	ss_dp		/* Data Pointer */
#  define	ss_gr28	ss_ret0		/* Return Zero */
#  define	ss_gr29	ss_ret1		/* Return One */
#  define	ss_gr30	ss_sp		/* Stack Pointer */
#ifdef	__LP64__
#  define	ss_arg7 ss_gr19		/* Eighth Argument */
#  define	ss_arg6 ss_gr20		/* Seventh Argument */
#  define	ss_arg5 ss_gr21		/* Sixth Argument */
#  define	ss_arg4	ss_gr22		/* Fifth Argument */
#  define	ss_gp	ss_gr27		/* Global Pointer */
#  define	ss_ap	ss_gr29		/* Argument Pointer */
#endif	/* __LP64__ */
#  define	ss_rec	ss_cr0		/* Recovery Counter */
#  define	ss_pid1	ss_cr8		/* Protection ID 1 */
#  define	ss_pid2	ss_cr9		/* Protection ID 2 */
#  define	ss_ccr	ss_cr10		/* Coprocessor Configuration Register */
#  define	ss_sar	ss_cr11		/* Shift Amount Register */
#  define	ss_pid3	ss_cr12		/* Protection ID 3 */
#  define	ss_pid4	ss_cr13		/* Protection ID 4 */
#  define	ss_eiem	ss_cr15		/* External Interrupt Mask Register */
#  define	ss_iir	ss_cr19		/* Interruption Instruction Register */
#  define	ss_isr	ss_cr20		/* Interruption Space Register */
#  define	ss_ior	ss_cr21		/* Interruption Offset Register */
#  define	ss_ipsw	ss_cr22		/* Interruption Processor Status Word */
#  define	ss_pdir ss_cr24		/* Temp Reg 0 -- PDIR pointer */
#  define	ss_htbl ss_cr25		/* Temp Reg 1 -- hash table pointer */
#  define	ss_htm	ss_cr26		/* Temp Reg 2 -- hash table mask */
#  define	ss_tr2	ss_cr26		/* Temp Reg 2 */
#  define	ss_tlsp ss_cr27		/* Thread Local Storage Pointer Reg */
#else /* defined(_KERNEL) ||  defined(_INCLUDE_HPUX_SOURCE)  || !defined(_XPG4_EXTENDED) */
#  define	__ss_gr2	ss_rp
#  define	__ss_T3		ss_gr20		/* Third Temp Register */
#  define	__ss_T2		ss_gr21		/* Second Temp Register */
#  define	__ss_T1		ss_gr22		/* First Temp Register */
#  define	__ss_gr23	ss_arg3		/* Fourth Argument */
#  define	__ss_gr24	ss_arg2		/* Third Argument */
#  define	__ss_gr25	ss_arg1		/* Second Argument */
#  define	__ss_gr26	ss_arg0		/* First Argument */
#  define	__ss_gr27	ss_dp		/* Data Pointer */
#  define	__ss_gr28	ss_ret0		/* Return Zero */
#  define	__ss_gr29	ss_ret1		/* Return One */
#  define	__ss_gr30	ss_sp		/* Stack Pointer */
#ifdef	__LP64__
#  define	__ss_arg7 	ss_gr19		/* Eighth Argument */
#  define	__ss_arg6 	ss_gr20		/* Seventh Argument */
#  define	__ss_arg5 	ss_gr21		/* Sixth Argument */
#  define	__ss_arg4	ss_gr22		/* Fifth Argument */
#  define	__ss_gp		ss_gr27		/* Global Pointer */
#  define	__ss_ap		ss_gr29		/* Argument Pointer */
#endif	/* __LP64__ */
#  define	__ss_rec	ss_cr0		/* Recovery Counter */
#  define	__ss_pid1	ss_cr8		/* Protection ID 1 */
#  define	__ss_pid2	ss_cr9		/* Protection ID 2 */
#  define	__ss_ccr	ss_cr10		/* Coprocessor Configuration Register */
#  define	__ss_sar	ss_cr11		/* Shift Amount Register */
#  define	__ss_pid3	ss_cr12		/* Protection ID 3 */
#  define	__ss_pid4	ss_cr13		/* Protection ID 4 */
#  define	__ss_eiem	ss_cr15		/* External Interrupt Mask Register */
#  define 	__ss_iir	ss_cr19		/* Interruption Instruction Register */
#  define 	__ss_isr	ss_cr20		/* Interruption Space Register */
#  define 	__ss_ior	ss_cr21		/* Interruption Offset Register */
#  define 	__ss_ipsw	ss_cr22		/* Interruption Processor Status Word */
#  define 	__ss_pdir	ss_cr24		/* Temp Reg 0 -- PDIR pointer */
#  define 	__ss_htbl	ss_cr25		/* Temp Reg 1 -- hash table pointer */
#  define 	__ss_htm	ss_cr26		/* Temp Reg 2 -- hash table mask */
#  define 	__ss_tr2	ss_cr26		/* Temp Reg 2 */
#  define	__ss_tlsp	ss_cr27		/* Thread Local Storage Pointer Reg */
#endif /* defined(KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)  */
 
#if defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)
# define	ss_fr0	ss_fpblock.fpdbl.ss_fp0
# define	ss_fr1	ss_fpblock.fpdbl.ss_fp1
# define	ss_fr2	ss_fpblock.fpdbl.ss_fp2
# define	ss_fr3	ss_fpblock.fpdbl.ss_fp3
# define	ss_fr4	ss_fpblock.fpdbl.ss_fp4
# define	ss_fr5	ss_fpblock.fpdbl.ss_fp5
# define	ss_fr6	ss_fpblock.fpdbl.ss_fp6
# define	ss_fr7	ss_fpblock.fpdbl.ss_fp7
# define	ss_fr8	ss_fpblock.fpdbl.ss_fp8
# define	ss_fr9	ss_fpblock.fpdbl.ss_fp9
# define	ss_fr10	ss_fpblock.fpdbl.ss_fp10
# define	ss_fr11	ss_fpblock.fpdbl.ss_fp11
# define	ss_fr12	ss_fpblock.fpdbl.ss_fp12
# define	ss_fr13	ss_fpblock.fpdbl.ss_fp13
# define	ss_fr14	ss_fpblock.fpdbl.ss_fp14
# define	ss_fr15	ss_fpblock.fpdbl.ss_fp15
# define	ss_fr16	ss_fpblock.fpdbl.ss_fp16
# define	ss_fr17	ss_fpblock.fpdbl.ss_fp17
# define	ss_fr18	ss_fpblock.fpdbl.ss_fp18
# define	ss_fr19	ss_fpblock.fpdbl.ss_fp19
# define	ss_fr20	ss_fpblock.fpdbl.ss_fp20
# define	ss_fr21	ss_fpblock.fpdbl.ss_fp21
# define	ss_fr22	ss_fpblock.fpdbl.ss_fp22
# define	ss_fr23	ss_fpblock.fpdbl.ss_fp23
# define	ss_fr24	ss_fpblock.fpdbl.ss_fp24
# define	ss_fr25	ss_fpblock.fpdbl.ss_fp25
# define	ss_fr26	ss_fpblock.fpdbl.ss_fp26
# define	ss_fr27	ss_fpblock.fpdbl.ss_fp27
# define	ss_fr28	ss_fpblock.fpdbl.ss_fp28
# define	ss_fr29	ss_fpblock.fpdbl.ss_fp29
# define	ss_fr30	ss_fpblock.fpdbl.ss_fp30
# define	ss_fr31	ss_fpblock.fpdbl.ss_fp31
# define	ss_frstat	ss_fpblock.fpint.ss_fpstat
# define	ss_frexcp1	ss_fpblock.fpint.ss_fpexcept1
# define	ss_frexcp2	ss_fpblock.fpint.ss_fpexcept2
# define	ss_frexcp3	ss_fpblock.fpint.ss_fpexcept3
# define	ss_frexcp4	ss_fpblock.fpint.ss_fpexcept4
# define	ss_frexcp5	ss_fpblock.fpint.ss_fpexcept5
# define	ss_frexcp6	ss_fpblock.fpint.ss_fpexcept6
# define	ss_frexcp7	ss_fpblock.fpint.ss_fpexcept7
# define	ss_fr4_hi	ss_fpblock.fpint.ss_fp4_hi
# define	ss_fr4_lo	ss_fpblock.fpint.ss_fp4_lo
# define	ss_fr5_hi	ss_fpblock.fpint.ss_fp5_hi
# define	ss_fr5_lo	ss_fpblock.fpint.ss_fp5_lo
# define	ss_fr6_hi	ss_fpblock.fpint.ss_fp6_hi
# define	ss_fr6_lo	ss_fpblock.fpint.ss_fp6_lo
# define	ss_fr7_hi	ss_fpblock.fpint.ss_fp7_hi
# define	ss_fr7_lo	ss_fpblock.fpint.ss_fp7_lo
# define	ss_fr8_hi	ss_fpblock.fpint.ss_fp8_hi
# define	ss_fr8_lo	ss_fpblock.fpint.ss_fp8_lo
# define	ss_fr9_hi	ss_fpblock.fpint.ss_fp9_hi
# define	ss_fr9_lo	ss_fpblock.fpint.ss_fp9_lo
# define	ss_fr10_hi	ss_fpblock.fpint.ss_fp10_hi
# define	ss_fr10_lo	ss_fpblock.fpint.ss_fp10_lo
# define	ss_fr11_hi	ss_fpblock.fpint.ss_fp11_hi
# define	ss_fr11_lo	ss_fpblock.fpint.ss_fp11_lo
# define	ss_fr12_hi	ss_fpblock.fpint.ss_fp12_hi
# define	ss_fr12_lo	ss_fpblock.fpint.ss_fp12_lo
# define	ss_fr13_hi	ss_fpblock.fpint.ss_fp13_hi
# define	ss_fr13_lo	ss_fpblock.fpint.ss_fp13_lo
# define	ss_fr14_hi	ss_fpblock.fpint.ss_fp14_hi
# define	ss_fr14_lo	ss_fpblock.fpint.ss_fp14_lo
# define	ss_fr15_hi	ss_fpblock.fpint.ss_fp15_hi
# define	ss_fr15_lo	ss_fpblock.fpint.ss_fp15_lo
# define	ss_fr16_hi	ss_fpblock.fpint.ss_fp16_hi
# define	ss_fr16_lo	ss_fpblock.fpint.ss_fp16_lo
# define	ss_fr17_hi	ss_fpblock.fpint.ss_fp17_hi
# define	ss_fr17_lo	ss_fpblock.fpint.ss_fp17_lo
# define	ss_fr18_hi	ss_fpblock.fpint.ss_fp18_hi
# define	ss_fr18_lo	ss_fpblock.fpint.ss_fp18_lo
# define	ss_fr19_hi	ss_fpblock.fpint.ss_fp19_hi
# define	ss_fr19_lo	ss_fpblock.fpint.ss_fp19_lo
# define	ss_fr20_hi	ss_fpblock.fpint.ss_fp20_hi
# define	ss_fr20_lo	ss_fpblock.fpint.ss_fp20_lo
# define	ss_fr21_hi	ss_fpblock.fpint.ss_fp21_hi
# define	ss_fr21_lo	ss_fpblock.fpint.ss_fp21_lo
# define	ss_fr22_hi	ss_fpblock.fpint.ss_fp22_hi
# define	ss_fr22_lo	ss_fpblock.fpint.ss_fp22_lo
# define	ss_fr23_hi	ss_fpblock.fpint.ss_fp23_hi
# define	ss_fr23_lo	ss_fpblock.fpint.ss_fp23_lo
# define	ss_fr24_hi	ss_fpblock.fpint.ss_fp24_hi
# define	ss_fr24_lo	ss_fpblock.fpint.ss_fp24_lo
# define	ss_fr25_hi	ss_fpblock.fpint.ss_fp25_hi
# define	ss_fr25_lo	ss_fpblock.fpint.ss_fp25_lo
# define	ss_fr26_hi	ss_fpblock.fpint.ss_fp26_hi
# define	ss_fr26_lo	ss_fpblock.fpint.ss_fp26_lo
# define	ss_fr27_hi	ss_fpblock.fpint.ss_fp27_hi
# define	ss_fr27_lo	ss_fpblock.fpint.ss_fp27_lo
# define	ss_fr28_hi	ss_fpblock.fpint.ss_fp28_hi
# define	ss_fr28_lo	ss_fpblock.fpint.ss_fp28_lo
# define	ss_fr29_hi	ss_fpblock.fpint.ss_fp29_hi
# define	ss_fr29_lo	ss_fpblock.fpint.ss_fp29_lo
# define	ss_fr30_hi	ss_fpblock.fpint.ss_fp30_hi
# define	ss_fr30_lo	ss_fpblock.fpint.ss_fp30_lo
# define	ss_fr31_hi	ss_fpblock.fpint.ss_fp31_hi
# define	ss_fr31_lo	ss_fpblock.fpint.ss_fp31_lo
#else /* defined(_KERNEL) ||  defined(_INCLUDE_HPUX_SOURCE)  || !defined(_XPG4_EXTENDED) */
#  if defined(_INCLUDE_HPUX_SOURCE)
#     define	ss_fr0	ss_fpblock.__fpdbl.ss_fp0
#     define	ss_fr1	ss_fpblock.__fpdbl.ss_fp1
#     define	ss_fr2	ss_fpblock.__fpdbl.ss_fp2
#     define	ss_fr3	ss_fpblock.__fpdbl.ss_fp3
#     define	ss_fr4	ss_fpblock.__fpdbl.ss_fp4
#     define	ss_fr5	ss_fpblock.__fpdbl.ss_fp5
#     define	ss_fr6	ss_fpblock.__fpdbl.ss_fp6
#     define	ss_fr7	ss_fpblock.__fpdbl.ss_fp7
#     define	ss_fr8	ss_fpblock.__fpdbl.ss_fp8
#     define	ss_fr9	ss_fpblock.__fpdbl.ss_fp9
#     define	ss_fr10	ss_fpblock.__fpdbl.ss_fp10
#     define	ss_fr11	ss_fpblock.__fpdbl.ss_fp11
#     define	ss_fr12	ss_fpblock.__fpdbl.ss_fp12
#     define	ss_fr13	ss_fpblock.__fpdbl.ss_fp13
#     define	ss_fr14	ss_fpblock.__fpdbl.ss_fp14
#     define	ss_fr15	ss_fpblock.__fpdbl.ss_fp15
#     define	ss_fr16	ss_fpblock.__fpdbl.ss_fp16
#     define	ss_fr17	ss_fpblock.__fpdbl.ss_fp17
#     define	ss_fr18	ss_fpblock.__fpdbl.ss_fp18
#     define	ss_fr19	ss_fpblock.__fpdbl.ss_fp19
#     define	ss_fr20	ss_fpblock.__fpdbl.ss_fp20
#     define	ss_fr21	ss_fpblock.__fpdbl.ss_fp21
#     define	ss_fr22	ss_fpblock.__fpdbl.ss_fp22
#     define	ss_fr23	ss_fpblock.__fpdbl.ss_fp23
#     define	ss_fr24	ss_fpblock.__fpdbl.ss_fp24
#     define	ss_fr25	ss_fpblock.__fpdbl.ss_fp25
#     define	ss_fr26	ss_fpblock.__fpdbl.ss_fp26
#     define	ss_fr27	ss_fpblock.__fpdbl.ss_fp27
#     define	ss_fr28	ss_fpblock.__fpdbl.ss_fp28
#     define	ss_fr29	ss_fpblock.__fpdbl.ss_fp29
#     define	ss_fr30	ss_fpblock.__fpdbl.ss_fp30
#     define	ss_fr31	ss_fpblock.__fpdbl.ss_fp31
#     define	ss_frstat	ss_fpblock.__fpint.ss_fpstat
#     define	ss_frexcp1	ss_fpblock.__fpint.ss_fpexcept1
#     define	ss_frexcp2	ss_fpblock.__fpint.ss_fpexcept2
#     define	ss_frexcp3	ss_fpblock.__fpint.ss_fpexcept3
#     define	ss_frexcp4	ss_fpblock.__fpint.ss_fpexcept4
#     define	ss_frexcp5	ss_fpblock.__fpint.ss_fpexcept5
#     define	ss_frexcp6	ss_fpblock.__fpint.ss_fpexcept6
#     define	ss_frexcp7	ss_fpblock.__fpint.ss_fpexcept7
#     define	ss_fr4_hi	ss_fpblock.__fpint.ss_fp4_hi
#     define	ss_fr4_lo	ss_fpblock.__fpint.ss_fp4_lo
#     define	ss_fr5_hi	ss_fpblock.__fpint.ss_fp5_hi
#     define	ss_fr5_lo	ss_fpblock.__fpint.ss_fp5_lo
#     define	ss_fr6_hi	ss_fpblock.__fpint.ss_fp6_hi
#     define	ss_fr6_lo	ss_fpblock.__fpint.ss_fp6_lo
#     define	ss_fr7_hi	ss_fpblock.__fpint.ss_fp7_hi
#     define	ss_fr7_lo	ss_fpblock.__fpint.ss_fp7_lo
#     define	ss_fr8_hi	ss_fpblock.__fpint.ss_fp8_hi
#     define	ss_fr8_lo	ss_fpblock.__fpint.ss_fp8_lo
#     define	ss_fr9_hi	ss_fpblock.__fpint.ss_fp9_hi
#     define	ss_fr9_lo	ss_fpblock.__fpint.ss_fp9_lo
#     define	ss_fr10_hi	ss_fpblock.__fpint.ss_fp10_hi
#     define	ss_fr10_lo	ss_fpblock.__fpint.ss_fp10_lo
#     define	ss_fr11_hi	ss_fpblock.__fpint.ss_fp11_hi
#     define	ss_fr11_lo	ss_fpblock.__fpint.ss_fp11_lo
#     define	ss_fr12_hi	ss_fpblock.__fpint.ss_fp12_hi
#     define	ss_fr12_lo	ss_fpblock.__fpint.ss_fp12_lo
#     define	ss_fr13_hi	ss_fpblock.__fpint.ss_fp13_hi
#     define	ss_fr13_lo	ss_fpblock.__fpint.ss_fp13_lo
#     define	ss_fr14_hi	ss_fpblock.__fpint.ss_fp14_hi
#     define	ss_fr14_lo	ss_fpblock.__fpint.ss_fp14_lo
#     define	ss_fr15_hi	ss_fpblock.__fpint.ss_fp15_hi
#     define	ss_fr15_lo	ss_fpblock.__fpint.ss_fp15_lo
#     define	ss_fr16_hi	ss_fpblock.__fpint.ss_fp16_hi
#     define	ss_fr16_lo	ss_fpblock.__fpint.ss_fp16_lo
#     define	ss_fr17_hi	ss_fpblock.__fpint.ss_fp17_hi
#     define	ss_fr17_lo	ss_fpblock.__fpint.ss_fp17_lo
#     define	ss_fr18_hi	ss_fpblock.__fpint.ss_fp18_hi
#     define	ss_fr18_lo	ss_fpblock.__fpint.ss_fp18_lo
#     define	ss_fr19_hi	ss_fpblock.__fpint.ss_fp19_hi
#     define	ss_fr19_lo	ss_fpblock.__fpint.ss_fp19_lo
#     define	ss_fr20_hi	ss_fpblock.__fpint.ss_fp20_hi
#     define	ss_fr20_lo	ss_fpblock.__fpint.ss_fp20_lo
#     define	ss_fr21_hi	ss_fpblock.__fpint.ss_fp21_hi
#     define	ss_fr21_lo	ss_fpblock.__fpint.ss_fp21_lo
#     define	ss_fr22_hi	ss_fpblock.__fpint.ss_fp22_hi
#     define	ss_fr22_lo	ss_fpblock.__fpint.ss_fp22_lo
#     define	ss_fr23_hi	ss_fpblock.__fpint.ss_fp23_hi
#     define	ss_fr23_lo	ss_fpblock.__fpint.ss_fp23_lo
#     define	ss_fr24_hi	ss_fpblock.__fpint.ss_fp24_hi
#     define	ss_fr24_lo	ss_fpblock.__fpint.ss_fp24_lo
#     define	ss_fr25_hi	ss_fpblock.__fpint.ss_fp25_hi
#     define	ss_fr25_lo	ss_fpblock.__fpint.ss_fp25_lo
#     define	ss_fr26_hi	ss_fpblock.__fpint.ss_fp26_hi
#     define	ss_fr26_lo	ss_fpblock.__fpint.ss_fp26_lo
#     define	ss_fr27_hi	ss_fpblock.__fpint.ss_fp27_hi
#     define	ss_fr27_lo	ss_fpblock.__fpint.ss_fp27_lo
#     define	ss_fr28_hi	ss_fpblock.__fpint.ss_fp28_hi
#     define	ss_fr28_lo	ss_fpblock.__fpint.ss_fp28_lo
#     define	ss_fr29_hi	ss_fpblock.__fpint.ss_fp29_hi
#     define	ss_fr29_lo	ss_fpblock.__fpint.ss_fp29_lo
#     define	ss_fr30_hi	ss_fpblock.__fpint.ss_fp30_hi
#     define	ss_fr30_lo	ss_fpblock.__fpint.ss_fp30_lo
#     define	ss_fr31_hi	ss_fpblock.__fpint.ss_fp31_hi
#     define	ss_fr31_lo	ss_fpblock.__fpint.ss_fp31_lo
#  else /* defined(_INCLUDE_HPUX_SOURCE) */
#     define	__ss_fr0	ss_fpblock.__fpdbl.ss_fp0
#     define	__ss_fr1	ss_fpblock.__fpdbl.ss_fp1
#     define	__ss_fr2	ss_fpblock.__fpdbl.ss_fp2
#     define	__ss_fr3	ss_fpblock.__fpdbl.ss_fp3
#     define	__ss_fr4	ss_fpblock.__fpdbl.ss_fp4
#     define	__ss_fr5	ss_fpblock.__fpdbl.ss_fp5
#     define	__ss_fr6	ss_fpblock.__fpdbl.ss_fp6
#     define	__ss_fr7	ss_fpblock.__fpdbl.ss_fp7
#     define	__ss_fr8	ss_fpblock.__fpdbl.ss_fp8
#     define	__ss_fr9	ss_fpblock.__fpdbl.ss_fp9
#     define	__ss_fr10	ss_fpblock.__fpdbl.ss_fp10
#     define	__ss_fr11	ss_fpblock.__fpdbl.ss_fp11
#     define	__ss_fr12	ss_fpblock.__fpdbl.ss_fp12
#     define	__ss_fr13	ss_fpblock.__fpdbl.ss_fp13
#     define	__ss_fr14	ss_fpblock.__fpdbl.ss_fp14
#     define	__ss_fr15	ss_fpblock.__fpdbl.ss_fp15
#     define	__ss_fr16	ss_fpblock.__fpdbl.ss_fp16
#     define	__ss_fr17	ss_fpblock.__fpdbl.ss_fp17
#     define	__ss_fr18	ss_fpblock.__fpdbl.ss_fp18
#     define	__ss_fr19	ss_fpblock.__fpdbl.ss_fp19
#     define	__ss_fr20	ss_fpblock.__fpdbl.ss_fp20
#     define	__ss_fr21	ss_fpblock.__fpdbl.ss_fp21
#     define	__ss_fr22	ss_fpblock.__fpdbl.ss_fp22
#     define	__ss_fr23	ss_fpblock.__fpdbl.ss_fp23
#     define	__ss_fr24	ss_fpblock.__fpdbl.ss_fp24
#     define	__ss_fr25	ss_fpblock.__fpdbl.ss_fp25
#     define	__ss_fr26	ss_fpblock.__fpdbl.ss_fp26
#     define	__ss_fr27	ss_fpblock.__fpdbl.ss_fp27
#     define	__ss_fr28	ss_fpblock.__fpdbl.ss_fp28
#     define	__ss_fr29	ss_fpblock.__fpdbl.ss_fp29
#     define	__ss_fr30	ss_fpblock.__fpdbl.ss_fp30
#     define	__ss_fr31	ss_fpblock.__fpdbl.ss_fp31
#     define	__ss_frstat	ss_fpblock.__fpint.ss_fpstat
#     define	__ss_frexcp1	ss_fpblock.__fpint.ss_fpexcept1
#     define	__ss_frexcp2	ss_fpblock.__fpint.ss_fpexcept2
#     define	__ss_frexcp3	ss_fpblock.__fpint.ss_fpexcept3
#     define	__ss_frexcp4	ss_fpblock.__fpint.ss_fpexcept4
#     define	__ss_frexcp5	ss_fpblock.__fpint.ss_fpexcept5
#     define	__ss_frexcp6	ss_fpblock.__fpint.ss_fpexcept6
#     define	__ss_frexcp7	ss_fpblock.__fpint.ss_fpexcept7
#     define	__ss_fr4_hi	ss_fpblock.__fpint.ss_fp4_hi
#     define	__ss_fr4_lo	ss_fpblock.__fpint.ss_fp4_lo
#     define	__ss_fr5_hi	ss_fpblock.__fpint.ss_fp5_hi
#     define	__ss_fr5_lo	ss_fpblock.__fpint.ss_fp5_lo
#     define	__ss_fr6_hi	ss_fpblock.__fpint.ss_fp6_hi
#     define	__ss_fr6_lo	ss_fpblock.__fpint.ss_fp6_lo
#     define	__ss_fr7_hi	ss_fpblock.__fpint.ss_fp7_hi
#     define	__ss_fr7_lo	ss_fpblock.__fpint.ss_fp7_lo
#     define	__ss_fr8_hi	ss_fpblock.__fpint.ss_fp8_hi
#     define	__ss_fr8_lo	ss_fpblock.__fpint.ss_fp8_lo
#     define	__ss_fr9_hi	ss_fpblock.__fpint.ss_fp9_hi
#     define	__ss_fr9_lo	ss_fpblock.__fpint.ss_fp9_lo
#     define	__ss_fr10_hi	ss_fpblock.__fpint.ss_fp10_hi
#     define	__ss_fr10_lo	ss_fpblock.__fpint.ss_fp10_lo
#     define	__ss_fr11_hi	ss_fpblock.__fpint.ss_fp11_hi
#     define	__ss_fr11_lo	ss_fpblock.__fpint.ss_fp11_lo
#     define	__ss_fr12_hi	ss_fpblock.__fpint.ss_fp12_hi
#     define	__ss_fr12_lo	ss_fpblock.__fpint.ss_fp12_lo
#     define	__ss_fr13_hi	ss_fpblock.__fpint.ss_fp13_hi
#     define	__ss_fr13_lo	ss_fpblock.__fpint.ss_fp13_lo
#     define	__ss_fr14_hi	ss_fpblock.__fpint.ss_fp14_hi
#     define	__ss_fr14_lo	ss_fpblock.__fpint.ss_fp14_lo
#     define	__ss_fr15_hi	ss_fpblock.__fpint.ss_fp15_hi
#     define	__ss_fr15_lo	ss_fpblock.__fpint.ss_fp15_lo
#     define	__ss_fr16_hi	ss_fpblock.__fpint.ss_fp16_hi
#     define	__ss_fr16_lo	ss_fpblock.__fpint.ss_fp16_lo
#     define	__ss_fr17_hi	ss_fpblock.__fpint.ss_fp17_hi
#     define	__ss_fr17_lo	ss_fpblock.__fpint.ss_fp17_lo
#     define	__ss_fr18_hi	ss_fpblock.__fpint.ss_fp18_hi
#     define	__ss_fr18_lo	ss_fpblock.__fpint.ss_fp18_lo
#     define	__ss_fr19_hi	ss_fpblock.__fpint.ss_fp19_hi
#     define	__ss_fr19_lo	ss_fpblock.__fpint.ss_fp19_lo
#     define	__ss_fr20_hi	ss_fpblock.__fpint.ss_fp20_hi
#     define	__ss_fr20_lo	ss_fpblock.__fpint.ss_fp20_lo
#     define	__ss_fr21_hi	ss_fpblock.__fpint.ss_fp21_hi
#     define	__ss_fr21_lo	ss_fpblock.__fpint.ss_fp21_lo
#     define	__ss_fr22_hi	ss_fpblock.__fpint.ss_fp22_hi
#     define	__ss_fr22_lo	ss_fpblock.__fpint.ss_fp22_lo
#     define	__ss_fr23_hi	ss_fpblock.__fpint.ss_fp23_hi
#     define	__ss_fr23_lo	ss_fpblock.__fpint.ss_fp23_lo
#     define	__ss_fr24_hi	ss_fpblock.__fpint.ss_fp24_hi
#     define	__ss_fr24_lo	ss_fpblock.__fpint.ss_fp24_lo
#     define	__ss_fr25_hi	ss_fpblock.__fpint.ss_fp25_hi
#     define	__ss_fr25_lo	ss_fpblock.__fpint.ss_fp25_lo
#     define	__ss_fr26_hi	ss_fpblock.__fpint.ss_fp26_hi
#     define	__ss_fr26_lo	ss_fpblock.__fpint.ss_fp26_lo
#     define	__ss_fr27_hi	ss_fpblock.__fpint.ss_fp27_hi
#     define	__ss_fr27_lo	ss_fpblock.__fpint.ss_fp27_lo
#     define	__ss_fr28_hi	ss_fpblock.__fpint.ss_fp28_hi
#     define	__ss_fr28_lo	ss_fpblock.__fpint.ss_fp28_lo
#     define	__ss_fr29_hi	ss_fpblock.__fpint.ss_fp29_hi
#     define	__ss_fr29_lo	ss_fpblock.__fpint.ss_fp29_lo
#     define	__ss_fr30_hi	ss_fpblock.__fpint.ss_fp30_hi
#     define	__ss_fr30_lo	ss_fpblock.__fpint.ss_fp30_lo
#     define	__ss_fr31_hi	ss_fpblock.__fpint.ss_fp31_hi
#     define	__ss_fr31_lo	ss_fpblock.__fpint.ss_fp31_lo
#  endif   /* defined(_INCLUDE_HPUX_SOURCE)  */
#endif /* defined(KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED) */


#if defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)
#  define	ss_mdsfu_hi	ss_mpsfu_high
#  define	ss_mdsfu_lo	ss_mpsfu_low
#  define	ss_mdsfu_ov	ss_mpsfu_ovflo
#  define	FP_TBIT	0x40
#else /* defined(_KERNEL) ||  defined(_INCLUDE_HPUX_SOURCE)  || !defined(_XPG4_EXTENDED) */
#  define	__ss_mdsfu_hi	ss_mpsfu_high
#  define	__ss_mdsfu_lo	ss_mpsfu_low
#  define	__ss_mdsfu_ov	ss_mpsfu_ovflo
#  define	_FP_TBIT	0x40
#endif /* defined(_KERNEL0 || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED) */


/*
 * Defines for high/low parts of 64 bit registers.
 */
#if defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED)
#define ss_gr2_hi  ss_rp_hi
#define ss_T3_hi   ss_gr20_hi         /* Third Temp Register */
#define ss_T2_hi   ss_gr21_hi         /* Second Temp Register */
#define ss_T1_hi   ss_gr22_hi         /* First Temp Register */
#define ss_gr23_hi ss_arg3_hi         /* Fourth Argument */
#define ss_gr24_hi ss_arg2_hi         /* Third Argument */
#define ss_gr25_hi ss_arg1_hi         /* Second Argument */
#define ss_gr26_hi ss_arg0_hi         /* First Argument */
#define ss_gr27_hi ss_dp_hi           /* Data Pointer */
#define ss_gr28_hi ss_ret0_hi         /* Return Zero */
#define ss_gr29_hi ss_ret1_hi         /* Return One */
#define ss_gr30_hi ss_sp_hi           /* Stack Pointer */
#ifdef	__LP64__
#define ss_arg7_hi ss_gr19_hi	      /* Eighth Argument */
#define ss_arg6_hi ss_gr20_hi	      /* Seventh Argument */
#define ss_arg5_hi ss_gr21_hi	      /* Sixth Argument */
#define ss_arg4_hi ss_gr22_hi	      /* Fifth Argument */
#define ss_gp_hi   ss_gr27_hi	      /* Global Pointer */
#define ss_ap_hi   ss_gr29_hi	      /* Argument Pointer */
#endif	/* __LP64__ */
#define ss_sar_hi  ss_cr11_hi	      /* Shift Amount Register */
#define ss_eiem_hi ss_cr15_hi
#define ss_iir_hi  ss_cr19_hi
#define ss_isr_hi  ss_cr20_hi
#define ss_ior_hi  ss_cr21_hi
#define ss_ipsw_hi ss_cr22_hi
#define ss_tlsp_hi ss_cr27_hi	      /* Thread Local Storage Pointer Reg */

#define ss_gr2_lo  ss_rp_lo
#define ss_T3_lo   ss_gr20_lo         /* Third Temp Register */
#define ss_T2_lo   ss_gr21_lo         /* Second Temp Register */
#define ss_T1_lo   ss_gr22_lo         /* First Temp Register */
#define ss_gr23_lo ss_arg3_lo         /* Fourth Argument */
#define ss_gr24_lo ss_arg2_lo         /* Third Argument */
#define ss_gr25_lo ss_arg1_lo         /* Second Argument */
#define ss_gr26_lo ss_arg0_lo         /* First Argument */
#define ss_gr27_lo ss_dp_lo           /* Data Pointer */
#define ss_gr28_lo ss_ret0_lo         /* Return Zero */
#define ss_gr29_lo ss_ret1_lo         /* Return One */
#define ss_gr30_lo ss_sp_lo           /* Stack Pointer */
#ifdef	__LP64__
#define ss_arg7_lo ss_gr19_lo	      /* Eighth Argument */
#define ss_arg6_lo ss_gr20_lo	      /* Seventh Argument */
#define ss_arg5_lo ss_gr21_lo	      /* Sixth Argument */
#define ss_arg4_lo ss_gr22_lo	      /* Fifth Argument */
#define ss_gp_lo   ss_gr27_lo	      /* Global Pointer */
#define ss_ap_lo   ss_gr29_lo	      /* Argument Pointer */
#endif	/* __LP64__ */
#define ss_sar_lo  ss_cr11_lo	      /* Shift Amount Register */
#define ss_eiem_lo ss_cr15_lo
#define ss_iir_lo  ss_cr19_lo
#define ss_isr_lo  ss_cr20_lo
#define ss_ior_lo  ss_cr21_lo
#define ss_ipsw_lo ss_cr22_lo
#define ss_tlsp_lo ss_cr27_lo	      /* Thread Local Storage Pointer Reg */
#endif /* defined(_KERNEL) || defined(_INCLUDE_HPUX_SOURCE) || !defined(_XPG4_EXTENDED) */

#endif /* !defined(_MACHINE_SAVE_STATE_INCLUDED) */
