/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	common/sys/som.h		$Revision: $
 * 
 *  Purpose:
 *	DO NOT USE THIS FILE.
 *
 *	THE PROPER FILE TO USE IS EITHER <a.out.h> or <magic.h> WHICH CONTAIN
 *	SIMILAR DEFINITIONS TO THOSE FOUND HERE.
 *
 *	C structure definitions for structures in the SOM ACD
 *
 *	HP 9000 Series 800 Object Modules (Program files) consist of:
 *
 *	   Header
 *	   Auxiliary header(s)
 *	   Space dictionary
 *	   Subspace dictionary
 *	   Loader fixup dictionary
 *	   Space/Subspace string area for space/subspace dictionary
 *	   Initialization pointers
 *	   Data pages
 *	   Compilation unit dictionary
 *	   Symbol dictionary
 *	   Fixup information
 *	   Compilation unit/symbol string area for comp. unit/symbol dictionary
 *	   Data for unloadable spaces
 *
 *	A word of every block which is associated with a name is
 *	the offset within the corresponding string area of
 *	the beginning of the name (word aligned).
 *	The name is length-preceeded , \0 terminated and padded
 *	to a word boundary.
 *	When the blocks are in memory the offset will be
 *	replaced with a pointer to the string.
 *
 *	A file location entry of zero always means that the entry in question
 *	is absent.
 *
 *	An index number of -1 indicates no entries
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/som.h
 *	/usr/conf/pa/som.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/


#ifndef _MACHINE_SOM_INCLUDED /* allows multiple inclusion */
#define _MACHINE_SOM_INCLUDED

#ifndef _A_OUT_INCLUDED	/* already picked up from <a.out.h> */

/*              union definition for pointer to names                        */

union name_pt {
                char        *n_name;
                unsigned int n_strx;
              };

/*              struct definition for the system clock                       */

struct sys_clock { 
                   unsigned int secs; 
                   unsigned int nanosecs; 
                 };

/* macro definitions to give simple names to union structure components */

#define NAME_PT name.n_name
#define STR_INDEX name.n_strx



/************************************************************************/
/*                              					*/
/*      HP 9000 Series 800 magic number, system id and SOM version    	*/
/*                              					*/
/* 	The magic numbers defined for HP PA are:     	 		*/

#define SPECMAGIC       0401  /* relocatable SOM  			*/
#define EXECMAGIC       0410  /* normal, page_aligned, executable SOM   */
#define NAEXECMAGIC     0403  /* non-aligned executable SOM 		*/
#define EXECLIBMAGIC    0404  /* executable SOM library		   	*/
#define LIBMAGIC        03031 /* relocatable SOM library ( archive )    */

#define SPECID          0x20B 
#define VERSION_ID      85042212

  


/***************************************************************************
*               the header consists of a single record                     *
****************************************************************************/

struct header{
        short int system_id;                 /* magic number - system        */
        short int a_magic;                   /* magic number - file type     */
        unsigned int version_id;             /* version id; format= YYMMDDHH */
        struct sys_clock file_time;          /* system clock- zero if unused */
        unsigned int entry_space;            /* index of space containing 
                                                entry point                  */
        unsigned int entry_subspace;         /* index of subspace for 
                                                entry point                  */
        unsigned int entry_offset;           /* offset of entry point        */
        unsigned int aux_header_location;    /* auxiliary header location    */
        unsigned int aux_header_size;        /* auxiliary header size        */
        unsigned int som_length;             /* length in bytes of entire som*/
        unsigned int presumed_dp;            /* DP value assumed during 
						compilation 		     */
        unsigned int space_location;         /* location in file of space 
                                                dictionary                   */
        unsigned int space_total;            /* number of space entries      */
        unsigned int subspace_location;      /* location of subspace entries */
        unsigned int subspace_total;         /* number of subspace entries   */
        unsigned int loader_fixup_location;  /* space reference array        */
        unsigned int loader_fixup_total;     /* total number of space 
                                                reference records            */
        unsigned int space_strings_location; /* file location of string area
                                                for space and subspace names */
        unsigned int space_strings_size;     /* size of string area for space 
                                                 and subspace names          */
        unsigned int init_array_location;    /* location in file of 
                                                initialization pointers      */
        unsigned int init_array_total;       /* number of init. pointers     */
        unsigned int compiler_location;      /* location in file of module 
                                                dictionary                   */
        unsigned int compiler_total;         /* number of modules            */
        unsigned int symbol_location;        /* location in file of symbol 
                                                dictionary                   */
        unsigned int symbol_total;           /* number of symbols            */
        unsigned int fixup_request_location; /* location in file of fix_up 
                                                requests                     */
        unsigned int fixup_request_total;    /* number of fixup requests     */
        unsigned int symbol_strings_location;/* file location of string area 
                                                for module and symbol names  */
        unsigned int symbol_strings_size;    /* size of string area for 
                                                module and symbol names      */
        unsigned int unloadable_sp_location; /* byte offset of first byte of 
                                                data for unloadable spaces   */
        unsigned int unloadable_sp_size;     /* byte length of data for 
                                                unloadable spaces            */
        unsigned int checksum;
};
#define _HEADER_T


/*****************************************************************************/
/*      The (optional) auxiliary header consists of an auxiliary header id   */
/*      followed by an arbitrary number of auxiliary header blocks           */
/*****************************************************************************/

        struct aux_id { 
                        unsigned int mandatory : 1;
                        unsigned int copy : 1;
                        unsigned int append : 1;
                        unsigned int ignore : 1;
                        unsigned int reserved : 12;
                        unsigned int type : 16;
                        unsigned int length;
                      };


/*****************************************************************************/
/*      The following auxiliary header definitions are currently             */
/*      defined in version 2.0 of the SOM ACD                                */
/*****************************************************************************/

/*      This auxialiary header is primarily for the Unix environment         */
/*      It must be placed directly after the SOM header in the object file   */

/*DISCLAIMER.............Being reviewed at DSD -- Steve*/
/* hpux_somhdr is not yet finalized --- ** subject to change **           */

/*
 * The exec auxiliary header is used to contain run time information
 * for executable SOM files which conform to the notion of a 32-bit
 * local address space.  
 * If the field exec_tsize is not zero, then the SOM contains sharable
 * text. 
 */

struct som_exec_auxhdr {
   struct aux_id som_auxhdr; /* som auxiliary header header  */
   int     	 exec_tsize; /* text size in bytes           */
   int     	 exec_tmem;  /* offset of text in memory     */
   int     	 exec_tfile; /* location of text in file     */
   int     	 exec_dsize; /* initialized data             */
   int     	 exec_dmem;  /* offset of data in memory     */
   int     	 exec_dfile; /* location of data in file     */
   int     	 exec_bsize; /* uninitialized data (bss)     */
   int     	 exec_entry; /* offset of entrypoint         */
   int     	 exec_flags; /* loader flags ( undefined )   */
   int     	 exec_bfill; /* bss initialization value     */
};

/* Product specific aux header format */

#define PRODUCT_SPECIFIC_AUX_ID 11

struct product_specific_aux_hdr {
        struct aux_id   header_id;
        unsigned int key[1];
};




/******************** HPE auxiliary headers *******************************/

struct cap_list {
        unsigned int    null 		:1;
        unsigned int    field_support 	:1;
        unsigned int    network_mgr 	:1;
        unsigned int    network_admin 	:1;
        unsigned int    system_sup 	:1;
        unsigned int    console_op 	:1;
        unsigned int    process_hand 	:1;
        unsigned int    priv_mode 	:1;
        unsigned int    user_priv_mode 	:1;
        unsigned int    user_mode 	:1;
        unsigned int    inter_acc 	:1;
        unsigned int    batch_acc 	:1;
        unsigned int    perf_spec 	:1;
        unsigned int    sys_mgr 	:1;
        unsigned int    acct_mgr 	:1;
        unsigned int    acct_lib 	:1;
        unsigned int    group_lib 	:1;
        unsigned int    diagnostic 	:1;
        unsigned int    sys_sup 	:1;
        unsigned int    save_files 	:1;
        unsigned int    non_share_dev 	:1;
        unsigned int    comm_subsys 	:1;
        unsigned int    use_volumes 	:1;
        unsigned int    creat_volumes 	:1;
        unsigned int    extra_data_seg 	:1;
        unsigned int    multiple_rins 	:1;
        unsigned int    user_logging 	:1;
        unsigned int    dummy :5;
};

struct linker_footprint {
        struct aux_id    header_id;
        char             product_id [12];
        char             version_id [8];
        struct sys_clock htime;
};

struct hpe_aux_header { 
         
        struct aux_id   header_id;
        unsigned int    internal_flag :1;
        unsigned int    reserved      :31;
        unsigned int    num_xrts;
        unsigned int    entry_name;
        unsigned int    unsat_name;
        int             search_list;
        struct cap_list capabilities;
        unsigned int    max_stacksize;
        unsigned int    max_heap_size;
                 
};       


/*****************************************************************************/
/*                   IPL AUXILIARY HEADER                                    */
/*****************************************************************************/

struct ipl_aux_hdr {
        struct aux_id   header_id;
	unsigned int    address_dest;
	unsigned int    entry_offset;
	unsigned int    checksum;
};



/******************************************************************************
*       the space dictionary consists of a sequence of space records          *
*******************************************************************************/

struct space_dictionary_record {
        union name_pt name;                 /* index to subspace name         */
        unsigned int  is_loadable: 1;       /* space is loadable              */
        unsigned int  is_defined : 1;       /* space is defined within file   */
        unsigned int  is_private : 1;       /* space is not sharable 	      */
        unsigned int  reserved   : 29;      /* reserved for future expansion  */
        int           space_number;         /* space index                    */
        int           subspace_index;       /* index into subspace dictionary */
        unsigned int  subspace_quantity;    /* number of subspaces in space   */
        int           loader_fix_index;     /* index into loader fixup array  */
        unsigned int  loader_fix_quantity;  /* number of loader fixups in 
					       this space                     */
        int           init_pointer_index;   /* index into data(initialization)
                                               pointer array                  */
        unsigned int  init_pointer_quantity;/* number of data (init) pointers */
};


/******************************************************************************/
/*                                                                            */
/*     The subspace dictionary consists of a sequence of subspace records     */
/*                                                                            */
/******************************************************************************/

struct subspace_dictionary_record {
        int             space_index;
        unsigned int    access_control_bits: 7; /* access for PDIR entries    */
        unsigned int    memory_resident    : 1; /* lock in memory during 
                                                   execution                  */
        unsigned int    dup_common         : 1; /* data name clashes allowed  */
        unsigned int    is_common          : 1; /* subspace is a common block */
        unsigned int    is_loadable        : 1;
        unsigned int    quadrant	   : 2; /* quadrant request           */
        unsigned int    initially_frozen   : 1; /* must be locked into memory 
                                                   when OS is booted          */
        unsigned int    is_first   	   : 1; /* must be first subspace     */
        unsigned int    code_only   	   : 1; /* must contain only code     */
        unsigned int    reserved           : 16;
        int             file_loc_init_value;    /* file location or 
                                                   initialization value       */
        unsigned int    initialization_length;
        unsigned int    subspace_start;         /* starting offset            */
        unsigned int    subspace_length;        /* number of bytes defined 
                                                   by this subspace           */
        unsigned int    reserved2	   :16;   
        unsigned int    alignment	   :16; /* alignment required for the
                                                   subspace (largest alignment 
                                                   requested for any item in 
                                                   the subspace)              */
        union name_pt   name;                   /* index of subspace name     */
        int             fixup_request_index;    /* index into fixup array     */
        unsigned int    fixup_request_quantity; /* number of fixup requests   */
};



/******************************************************************************/
/*                                                                            */
/*      Loader Fixups consist of a sequence of loader fixup records           */
/*                                                                            */
/******************************************************************************/

struct loader_fixup {
        unsigned int fixup_type;
        unsigned int space_index;   /* index of space to fix up         */
        unsigned int space_offset;  /* offset at which to patch         */
        int 	     constant;	    /* constant used to patch the space */
};

/* There are currently two valid fixup types for the loader fixups */
#define LD_SPACE_REF 0  /* space reference fixup */
#define LD_DATA_REF  1  /* data reference fixup  */


/*******************************************************************************
*       The virtual addresses into which the data pages are to be loaded       *
*       are specified by a sequence of data pointers.                          *
*******************************************************************************/

struct init_pointer_record {
        unsigned int space_index;               /* index of space entry       */
        unsigned int access_control_bits: 7;    /* access field for PDIR      */
        unsigned int has_data	 	: 1;    /* file pages exist for this 
                                                   area of memory             */
        unsigned int memory_resident	: 1;
        unsigned int initially_frozen	: 1;     /* must be locked into memory 
                                                    when OS is booted         */
        unsigned int reserved: 22;
        unsigned int file_loc_init_value;       /* starting location in file 
                                                  (page aligned)              */
        unsigned int initialization_length;
        unsigned int space_offset;              /* starting offset in space 
                                                   (page aligned)             */
};



/******************************************************************************
*                                                                             *
*       The module dictionary consists of a sequence of module blocks         *
*       There is one compilation unit entry for each object file produced     *
*                                                                             *
*******************************************************************************/

struct compilation_unit {
        union name_pt    name;
        union name_pt    language_name;
        union name_pt    product_id;
        union name_pt    version_id;
        union name_pt    copyright;
        struct sys_clock compile_time;
        struct sys_clock source_time;
};





/******************************************************************************
*                                                                             *
*       The symbol dictionary consists of a sequence of symbol blocks         *
*  An entry into the symbol dictionary requires at least the         	      *
*  symbol_dictionary_record; The symbol_extension and argument_descriptor     *
*  records are optional                                  		      *
*                                                                             *
*******************************************************************************/

struct symbol_dictionary_record{

        unsigned int  symbol_type      : 8;
        unsigned int  symbol_scope     : 4;
        unsigned int  check_level      : 3;
        unsigned int  must_qualify     : 1;
        unsigned int  initially_frozen : 1;
        unsigned int  memory_resident  : 1;
        unsigned int  is_common        : 1;
        unsigned int  dup_common       : 1;
        unsigned int  xleast           : 2;
        unsigned int  arg_reloc        :10;
        union name_pt name;
        union name_pt qualifier_name; 
        unsigned int  symbol_info;
        unsigned int  symbol_value;
         
};

/*    The following is a list of the valid symbol types     */

#define    ST_NULL         0
#define    ST_ABSOLUTE     1
#define    ST_DATA         2
#define    ST_CODE         3
#define    ST_PRI_PROG     4
#define    ST_SEC_PROG     5
#define    ST_ENTRY        6
#define    ST_STORAGE      7
#define    ST_STUB         8
#define    ST_MODULE       9
#define    ST_SYM_EXT     10
#define    ST_ARG_EXT     11
#define    ST_MILLICODE   12
#define    ST_PLABEL      13

/*    The following is a list of the valid symbol scopes    */

#define    SS_UNSAT        0
#define    SS_EXTERNAL     1
#define    SS_GLOBAL       2
#define    SS_UNIVERSAL    3



/*****************************************************************************
*
*   The argument descriptor is a one-word quantity which specifies
*   type, structure and passing mode of the argument. A value of zero 
*   in any of the fields matches any of the valid values for that field.
*
*   The valid values for mode are:
*               Wild card :         0
*               Pass by Value :     1
*               Pass by reference : 2
*               Pass by name :      3
*
*   The valid values for the structure field are:
*               Wild card :                        0
*               Simple variable :                  1
*               Array :                            2
*               Composite :                        3
*               Short pointer to simple variable : 4
*               Long pointer to simple variable :  5
*               Short pointer to an array :        6
*               Long pointer to an array :         7
*               Short pointer to composite :       8
*               Long pointer to composite :        9
*               Procedure/Function :               10
*
*   The valid values for the type field are:
*               User defined type                       < 0
*               Wild card                                 0
*               Void                                      1
*               Byte            signed    (8 bits)        2
*               Short integer   signed    (16 bits)       3
*               Integer         signed    (32 bits)       4
*               Long integer    signed    (64 bits)       5
*               Byte            unsigned  (8 bits)        6
*               Short integer   unsigned  (16 bits)       7
*               Integer         unsigned  (32 bits)       8
*               Long integer    unsigned  (64 bits)       9
*               Short real                (32 bits)       10
*               real                      (64 bits)       11
*               Long real                 (128 bits)      12
*               Short complex             (2 short reals) 13
*               Complex                   (2 reals)       14
*               Long complex              (2 long reals)  15
*               Packed decimal                            16
*               String  null-terminated                   17
*               String  length preceeded  (4 byte length) 18
*               Label                                     19
*
*****************************************************************************/



union arg_descriptor {  
      struct {

        unsigned int mode      :2;   /* a value of zero is used to match with 
                                        any valid value                       */
        unsigned int structure :4;   /* a value of zero is used to match with 
                                        any valid value                       */
        int arg_type           :26;  /* a value of zero is used to match with 
                                        any valid value. A negative value
                                        is treated as a user- defined type    */
      } arg_desc;

      unsigned int 	word;
};


struct symbol_extension_record {
        unsigned int          type         :8; /* always type SYM_EXT (12) for 
                      			          this record                 */
        unsigned int          max_num_args :8;
        unsigned int          min_num_args :8;
        unsigned int          num_args     :8;
        union arg_descriptor  symbol_desc;
        union arg_descriptor  argument_desc[3];
};


struct argument_desc_array {
        unsigned int            type    : 8; /*  always type ARG_EXT (13) for 
                       				 this record          */
        unsigned int            reserved: 24; 
        union arg_descriptor    argument_desc[4];
};




/******************************************************************************
*       The fixup information consists of a sequence of fixup request         *
*       records.                                                              *
*
*       fixup involves replacing the specified component (fixup_format)
*       of the instruction with the specified field (fixup_field) of the
*       result of calculating:
*
*       (expression_type == e_one)
*
*    loc(symbol_index_one) + constant   
*
*   
*  or       
*       (expression_type == e_two)
*
*    loc(symbol_index_one) - loc(symbol_index_two) + constant  
*
*  or
*       (expression_type == e_pcrel)
*
*    loc(symbol_index_one) - address + constant 
*     ( where 'address' is the subspace offset 
*       of the item containing the expression )
*
*  or
*     constant 
*
*  or
*    subspace.subspace_start + constant 
*
*  or
*    subspace.subspace_start + subspace_length + constant 
*
*  or
*    loc(symbol_index_one) - subspace.subspace_start + constant 
*
******************************************************************************/

/*************************************************************/
/* valid values for the fixup fields are as follows :        */
/*                                                           */
/*      enumeration of field selectors:                      */

typedef unsigned int selt;

#define e_fsel  0    /* F'  : no change           	     */
#define e_lssel 1    /* LS' : if (bit 21) then add 0x800 
                              arithmetic shift right 11 bits */
#define e_rssel 2    /* RS' : Sign extend from bit 21        */
#define e_lsel  3    /* L'  : Arithmetic shift right 11 bits */
#define e_rsel  4    /* R'  : Set bits 0-20 to zero          */
#define e_ldsel 5    /* LD' : Add 0x800, arithmetic shift
                              right 11 bits          	     */
#define e_rdsel 6    /* RD' : Set bits 0-20 to one           */


/*      enumeration of expression types:                     */

typedef unsigned int etype;

#define e_one    0      /* label + constant                   */
#define e_two    1      /* label - label + constant           */
#define e_pcrel  2      /* label - subspace offset + constant */
#define e_con    3      /* constant                           */
#define e_subst  4      /* subspace start                     */
#define e_subend 5      /* subspace end                       */
#define e_subrel 6      /* subspace_offset (label)            */
#define e_local  7      /* resolved pc-rel displacement       */

/*       enumeration of fixup formats:                        */

typedef unsigned int which_expr;

#define i_exp14 0      /* 14 bit long displacement            */
#define i_exp21 1      /* 21 bit immediate                    */
#define i_exp11 2      /* signed 11 bit immediate             */
#define i_rel17 3      /* 17 bit, pc relative branch disp.    */
#define i_rel12 4      /* 12 bit, pc relative branch disp.    */
#define i_data  5      /* whole word                          */
#define i_none  6      /* no expression in this word          */
#define i_abs17 7      /* 17 bit absolute branch displacement */


/**************************************************************/




struct fixup_request_record {
        unsigned int need_data_ref  : 1;
        unsigned int reserved       : 10;
        unsigned int expression_type: 5;  /* type of expr. providing value  */
        unsigned int fixup_format   : 8;  /* inst. or data format           */
        unsigned int fixup_field    : 8;  /* field from fixup value         */
        unsigned int subspace_offset;     /* offset of word to be fixed up  */
        unsigned int symbol_index_one;    /* sequence # of symbol           */
        unsigned int symbol_index_two;    /* sequence # of second symbol    */
        int          fixup_constant;      /* constant part           	    */
};

#endif /* _A_OUT_INCLUDED */
#endif /* ! _MACHINE_SOM_INCLUDED */
