/*
 * BEGIN_DESC
 * 
 *  File: 
 *      @(#)B.11.11_LR	pa/vm/vmparam.h		$Revision: $
 * 
 *  Purpose:
 *	Machine dependent constants for the HP 9000 Series 800.
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/pa/vmparam.h
 *	/usr/conf/pa/vmparam.h
 *
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/

#ifdef __NO_PA_HDRS
    PA header file -- do not include this header file for non-PA builds.
#endif

#ifndef _MACHINE_VMPARAM_INCLUDED /* allow multiple inclusions */
#define _MACHINE_VMPARAM_INCLUDED

#ifdef _KERNEL	/* note entire file is kernel only */

#include <sys/types.h>
#include <machine/param.h>	/* kernel src: <machine/sys/param.h> */
#ifndef _CLEAN_BE
#include <machine/cpu.h>	/* kernel src: <machine/sys/cpu.h> */
#endif /* _CLEAN_BE */


/*
 * Direction stack grows
 *   STACKDIR_LOW_TO_HIGH = to higher numbered memory
 *   STACKDIR_HIGH_TO_LOW1 = to lower numbered memory
 */
#define STACKDIR	STACKDIR_LOW_TO_HIGH

/*
 * Size of each region is a configurable parameter.
 */
extern pgcnt_t maxtsiz;
extern pgcnt_t maxdsiz;
extern pgcnt_t maxssiz;
extern pgcnt_t maxrsessiz;
#ifdef __LP64__
extern pgcnt_t maxtsiz_64bit;
extern pgcnt_t maxdsiz_64bit;
extern pgcnt_t maxssiz_64bit;
extern pgcnt_t maxrsessiz_64bit;
#endif /* __LP64__ */

/*
 * Tunables for Variable Size Pages.  vps_pagesize specifies the initial page
 * size "hint" for all regions.  It can be overridden for user pregions either
 * because of an explicit chatr(1) or madvise(2), or because we transparently
 * selected a different size.  vps_ceiling caps the transparent size selected
 * and must be >= vps_pagesize (otherwise it is made equal to vps_pagesize).
 *
 * vps_chatr_ceiling caps the amount to which we can chatr.
 */
extern pgcnt_t vps_pagesize;    /* Default page size (in KB) for machines that
                                 * support pages > 4K.
                                 */
extern pgcnt_t vps_ceiling;     /* Transparent Ceiling (in KB), no page sizes
                                 * greater than this unless the user
                                 * has chatr'd or segment is memory
                                 * locked.
                                 */
extern pgcnt_t vps_chatr_ceiling; /* Chatr Ceiling (in KB), no chatr'd page sizes
                                 * greater than this.
                                 */
extern pgcnt_t vps_pagesize_num4k;      /* num4k equivalents for internal use */
extern pgcnt_t vps_ceiling_num4k;
extern pgcnt_t vps_chatr_ceiling_num4k;

extern int vps_stack_increment;   /* 64K */

/*
 * Preallocated space numbers
 */
#ifndef KERNELSPACE
# define KERNELSPACE   ((space_t)0)
#endif

#ifndef KERNELIOSPACE
# define KERNELIOSPACE ((space_t)0)
#endif

/* Space ID for Buffer Cache (kernel mode) */

extern space_t bufcache_spaceid;

#define BUFFERCACHESPACE (bufcache_spaceid)


/*
 * Vas for the Kernel
 */
extern struct vas kernvas;
#define KERNVAS		(&kernvas)

/*
 * Pregion for the Kernel
 */
extern struct pregion *kernpreg;
#define KERNPREG	(kernpreg)

/*
 * The time for a process to be blocked before being very swappable.
 * This is a number of seconds which the system takes as being a
 * non-trivial amount of real time.  You probably shouldn't change
 * this; it is used in subtle ways (fractions and multiples of it are,
 * that is, like half of a ``long time'', almost a long time, etc.)
 * It is related to human patience and other factors which don't really
 * change over time.
 */
#define	MAXSLP 		20

/*
 * A swapped in process is given a small amount of core without being
 * bothered by the page replacement algorithm.  Basically this says
 * that if you are swapped in you deserve some resources.  We protect
 * the last SAFERSS pages against paging and will just swap you out
 * rather than paging you.
 * Note that each process has at least UPAGES+CLSIZE pages which are not
 * paged anyways (this is currently 8+2=10 pages or 5k bytes), so this
 * number just means a swapped in process is given around 25k bytes.
 *
 * Just for fun:
 *     current memory prices are 4600$ a megabyte on VAX (4/22/81),
 *     so we loan each swapped in process memory worth 100$, or just
 *     admit that we don't consider it worthwhile and swap it out to
 *     disk which costs $30/mb or about $0.75.
 */
#define	SAFERSS		32	/* nominal ``small'' resident set size
				   protected against replacement */

/*
 * DISKRPM is used to estimate the number of paging i/o operations
 * which one can expect from a single disk controller.
 */
#define	DISKRPM		45


/*
 * Define VM paging thresholds.  
 * See setmemthresholds() for details.
 * Note that LOTSFREEFRACTMIN also determines the amount of available memory
 * to be used for pseudo-swap; currently set to 7/8th of freemem at boot time.
 * (The pseudo-swap limits were previously unconverged with 700's using 7/8th
 * of freemem and 800's using 3/4th of freemem.  This prevented the recovery
 * kernel -- a memory based kernel -- from being able to run in a small memory
 * 800 system due to the lack of available pseudo-swap.  Therefore the various
 * paging thresholds have been converged by adopting the 700 values.)
 */
#define LOTSFREELOW             (btop(1024 * 1024))
#define LOTSFREEFRACTMIN        8

#define DESFREELOW              (btop(240  * 1024))
#define DESFREEFRACTMIN         16

#define MINFREELOW              (btop(100 * 1024))
#define MINFREEFRACTMIN         2

#define LOTSFREEHIGH            (btop(32 * 1024 * 1024))
#define LOTSFREEFRACTMAX        16

#define DESFREEHIGH             (btop(4 * 1024 * 1024))
#define DESFREEFRACTMAX         64

#define MINFREEHIGH             (btop(1024 * 1024))
#define MINFREEFRACTMAX         4

/*
 * Several useful physical memory defines.
 */
#ifdef __LP64__
/* 
 * Limit maximum physical memory to 512 Gbytes for PA 2.0 systems.
 * This is limited by the pg_pfn field (27 bits) in the vfd structure.
 * Increasing beyond this limit will require enlarging the vfd.
 */
#define MAXPHYSMEM      (btop(0x0000008000000000L))
#else
#define MAXPHYSMEM 	(btop(0xF0000000))		/* 3.75 Gbyte */
#endif /* __LP64__ */

/*
 * For non-contiguous physical memory, PFN_CONTIGUOUS_PAGES
 * is the granularity of implemented memory vs. holes.
 */
#define PFN_CONTIGUOUS_PAGES	0x1000	/* 16Mb for 4K pages */
#define PFN_BASE(PFN)		((PFN) / PFN_CONTIGUOUS_PAGES)
#define PFN_OFFSET(PFN)		((PFN) % PFN_CONTIGUOUS_PAGES)

/*
 * BUFFERZONE is no. of pages between tune.gpgslo and tune.gpgshi. If this
 * number is zero, vhand may oscillate, getting freemem up to lotsfree and
 * quitting, then being activated again as soon as freemem dips below lotsfree
 * This is a software "schmidt trigger" it may buy some X response time.
 */
#define BUFFERZONE      ((100 * 1024)/NBPG)

/*
 * Believed threshold (in megabytes) for which interleaved
 * swapping area is desirable.
 *
 * This is dead, right? 950330 -ESCXXX
 */
#define	LOTSOFMEM	8	/* make this reasonable for the 800 */

/*
 * Structure for dealing with unresolvable faults when referencing user
 * memory.  There is one of these per thread of control (uarea).
 * Like a light-weight machine dependent label_t for traps.
 */
typedef struct tlabel {
	void		(*t_dopc)();	/* pc to vector back to */
/*
 * Filled in at trap time
 */
	space_t		t_pcsq;
	caddr_t		t_pcoq;
	space_t		t_isr;
	caddr_t		t_ior;
	unsigned int	t_type;		/* trap type */
/*
 * Filled in after trap vector taken
 */
	space_t		t_okspace;	/* memory it's ok to trap on */
	caddr_t		t_okoffset;
	unsigned int	t_oklen;
} tlabel_t;

/*
 * The following are arbitrary flag bits with no association to a particular
 * data structure field.  However, these bits must be unique.
 */
#define MAP_EXCLUSIVE		0x1
#define MAP_NOPAGE		0x2
#define MAP_STATIC_PREDICT	0x4
#define MAP_FLIPPER_DISABLE	0x8
#define MAP_EXECUTABLE_STACK    0x10

/* If page number is in I/O space, produce a sign-extended page number */
#define pgndx_sext(ndx) \
	(((ndx) >= btop(IOSPACE)) ? \
	((ndx) | (~(((ulong_t)1<<PGNDXWDTH)-1))): (ndx))
#define iobtop(x)	((ulong_t)(pgndx_sext(btop(x))))

#endif /* _KERNEL */
#endif /* _MACHINE_VMPARAM_INCLUDED */
